@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: checks.c 4121 2007-12-18 13:07:50Z haypo $
+ ** $Id: checks.c 5466 2008-12-30 15:55:50Z regit $
  **
  **
  ** This program is free software; you can redistribute it and/or modify
@@ -82,12 +82,8 @@ void *recv_message(void *data)
 		ret =
 		    gnutls_record_recv(session->tls, dgram, sizeof dgram);
 		if (ret <= 0) {
-			if (gnutls_error_is_fatal(ret)) {
-				ask_session_end(session);
-				break;
-			} else {
-				continue;
-			}
+			ask_session_end(session);
+			break;
 		}
 
 		switch (dgram[0]) {
@@ -160,6 +156,51 @@ int nu_client_check(nuauth_session_t * session, nuclient_error_t * err)
 {
 	pthread_mutex_lock(&(session->mutex));
 
+	if (session->crl_file && session->check_counter++ > 100) {
+		struct stat stats;
+		session->check_counter = 0;
+
+		if (stat(session->crl_file, &stats) < 0) {
+			nu_error_log(err, "Can't access CRL file \"%s\"",
+					session->crl_file);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			/* returning -1 will cause client disconnect */
+			pthread_mutex_unlock(&(session->mutex));
+			return -1;
+		}
+		/* only test if certificate file has changed */
+		if (session->crl_file_mtime <= stats.st_mtime) {
+			session->crl_file_mtime = stats.st_mtime;
+			session->crl_file_exptime = check_crl_validity(session->crl_file,
+								       session->ca_file, err);
+			if ( session->crl_file_exptime == -1) {
+				pthread_mutex_unlock(&(session->mutex));
+				return -1;
+			} else {
+				int ret;
+				ret = gnutls_certificate_set_x509_crl_file(session->cred,
+						session->crl_file,
+						GNUTLS_X509_FMT_PEM);
+				if (ret < 0) {
+					SET_ERROR(err, GNUTLS_ERROR, ret);
+					nu_error_log(err, "CRL file (name %s) problem: %s",
+							session->crl_file,
+							gnutls_strerror(ret));
+					pthread_mutex_unlock(&(session->mutex));
+					return -1;
+				}
+			}
+		}
+
+		if (session->crl_file_exptime < time(NULL)) {
+			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_EXPIRED);
+			nu_error_log(err, "CRL file \"%s\" has expired, ending TLS session.",
+				     session->crl_file);
+			pthread_mutex_unlock(&(session->mutex));
+			return -1;
+		}
+	}
+
 	/* test is a thread has detected problem with the session */
 	if (session->connected == 0) {
 		/* if we are here, threads are dead */
@@ -174,6 +215,8 @@ int nu_client_check(nuauth_session_t * session, nuclient_error_t * err)
 			pthread_mutex_init(&session->checkthread_stop, NULL);
 			pthread_create(&(session->checkthread), NULL,
 				       nu_client_thread_check, session);
+		} else {
+			session->count_msg_cond = 0;
 		}
 		pthread_create(&(session->recvthread), NULL, recv_message,
 			       session);
@@ -243,7 +286,7 @@ void *nu_client_thread_check(void *data)
 
 		if (do_check) {
 			do_check = 0;
-		nu_client_real_check(session, NULL);
+			nu_client_real_check(session, NULL);
 		}
 		/* Do we need to do an other check ? */
 		pthread_mutex_lock(&session->check_count_mutex);
@@ -293,6 +336,7 @@ int nu_client_real_check(nuauth_session_t * session, nuclient_error_t * err)
 		return -1;
 	}
 	if (tcptable_read(session, new) == 0) {
+		tcptable_free(new);
 		SET_ERROR(err, INTERNAL_ERROR, TCPTABLE_ERR);
 		return -1;
 	}
