@@ -1,10 +1,11 @@
 /*
- ** Copyright 2004-2007 - INL
+ ** Copyright 2004-2008 - INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
+ **            Pierre Chifflier <chifflier@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: internal.c 4475 2008-02-16 23:15:45Z lds $
+ ** $Id: internal.c 5469 2008-12-31 10:58:57Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -72,6 +73,20 @@ void do_panic(const char *filename, unsigned long line, const char *fmt,
 	va_end(args);
 }
 
+void nu_error_log(nuclient_error *err, const char *fmt, ...)
+{
+	va_list args;
+	if (! err) {
+		return;
+	}
+	if (err->strerror[0]) {
+		return;
+	}
+	va_start(args, fmt);
+	vsnprintf(err->strerror, NU_STRERROR_LEN, fmt, args);
+	va_end(args);
+
+}
 
 static int samp_send(gnutls_session session, const char *buffer,
 		     unsigned length, nuclient_error_t * err)
@@ -117,6 +132,7 @@ static unsigned samp_recv(gnutls_session session, char *buf, int bufsize,
 
 	tls_len = gnutls_record_recv(session, buf, bufsize);
 	if (tls_len <= 0) {
+		printf("ERROR gnutls_record_recv returned %d (requested %d bytes)\n", tls_len, bufsize);
 		SET_ERROR(err, GNUTLS_ERROR, tls_len);
 		return 0;
 	}
@@ -124,6 +140,7 @@ static unsigned samp_recv(gnutls_session session, char *buf, int bufsize,
 	result = sasl_decode64(buf + 3, (unsigned) strlen(buf + 3), buf,
 			       bufsize, &len);
 	if (result != SASL_OK) {
+		printf("ERROR sasl_decode64 returned %d\n", result);
 		SET_ERROR(err, SASL_ERROR, result);
 		return 0;
 	}
@@ -142,14 +159,35 @@ int mysasl_negotiate(nuauth_session_t * user_session, sasl_conn_t * conn,
 	unsigned len;
 	int result;
 	gnutls_session session = user_session->tls;
+	sasl_ssf_t extssf = 0;
 
 	memset(buf, 0, sizeof buf);
 	/* get the capability list */
 	len = samp_recv(session, buf, 8192, err);
 	if (len == 0) {
+		printf("TLS session lost, check your certificate validity.\n");
+		SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_CERTIFICATE_ERROR);
 		return SASL_FAIL;
 	}
 
+	if (! user_session->username){
+		/* set username taken from console */
+		if (user_session->username_callback){
+			user_session->username = user_session->username_callback();
+		} else {
+			if (user_session->verbose)
+				printf("Can't call username callback\n");
+		}
+	}
+
+	sasl_setprop(conn, SASL_SSF_EXTERNAL, &extssf);
+	result = sasl_setprop(conn, SASL_AUTH_EXTERNAL, user_session->username);
+	if (result != SASL_OK) {
+		errno = EACCES;
+		SET_ERROR(err, SASL_ERROR, result);
+		return 0;
+	}
+
 	result = sasl_client_start(conn,
 				   buf, NULL, &data, &len, &chosenmech);
 
@@ -201,7 +239,7 @@ int mysasl_negotiate(nuauth_session_t * user_session, sasl_conn_t * conn,
 		}
 		if (data && len) {
 			if (user_session->verbose)
-				puts("Sending response...\n");
+				puts("Sending response...");
 			if (!samp_send(session, data, len, err)) {
 				return SASL_FAIL;
 			}
@@ -218,7 +256,7 @@ int mysasl_negotiate(nuauth_session_t * user_session, sasl_conn_t * conn,
 		return SASL_FAIL;
 	} else {
 		if (user_session->verbose)
-			puts("Authentication started...\n");
+			puts("Authentication started...");
 	}
 
 	return SASL_OK;
@@ -434,7 +472,8 @@ int send_os(nuauth_session_t * session, nuclient_error_t * err)
 	if (buf[0] == SRV_TYPE) {
 		session->server_mode = buf[1];
 	} else {
-		session->server_mode = SRV_TYPE_POLL;
+		SET_ERROR(err, GNUTLS_ERROR, ret);
+		return 0;
 	}
 	return 1;
 }
@@ -534,13 +573,15 @@ static int nu_get_userdatas(void *context __attribute__ ((unused)),
  * and then call mysasl_negotiate()
  *
  * \param session Pointer to client session
+ * \param hostname Name (FQDN) of the Nuauth server
  * \param err Pointer to a nuclient_error_t: which contains the error
  */
-int init_sasl(nuauth_session_t * session, nuclient_error_t * err)
+int init_sasl(nuauth_session_t * session, const char *hostname, nuclient_error_t * err)
 {
 	int ret;
 	sasl_conn_t *conn;
-	sasl_ssf_t extssf = 0;
+	char * krb5_service = NULL;
+	const char * server_fqdn = hostname;
 
 	/* SASL time */
 	sasl_callback_t callbacks[] = {
@@ -550,16 +591,18 @@ int init_sasl(nuauth_session_t * session, nuclient_error_t * err)
 		{SASL_CB_LIST_END, NULL, NULL}
 	};
 
-	ret =
-	    gnutls_record_send(session->tls, "PROTO 4", strlen("PROTO 4"));
+	ret = gnutls_record_send(session->tls, "PROTO 4", strlen("PROTO 4"));
 	if (ret < 0) {
 		SET_ERROR(err, GNUTLS_ERROR, ret);
 		return 0;
 	}
 
+	krb5_service = session->krb5_service;
+	if (krb5_service == NULL)
+		krb5_service = DEFAULT_KRB5_REALM;
+
 	/* client new connection */
-	ret =
-	    sasl_client_new("nuauth", "", NULL, NULL, callbacks, 0, &conn);
+	ret = sasl_client_new(krb5_service, server_fqdn, NULL, NULL, callbacks, 0, &conn);
 	if (ret != SASL_OK) {
 		if (session->verbose)
 			printf("Failed allocating connection state");
@@ -568,25 +611,6 @@ int init_sasl(nuauth_session_t * session, nuclient_error_t * err)
 		return 0;
 	}
 
-	if (! session->username){
-		/* set username taken from console */
-		if (session->username_callback){
-			session->username = session->username_callback();
-		} else {
-			if (session->verbose)
-				printf("Can't call username callback\n");
-		}
-	}
-
-	sasl_setprop(conn, SASL_SSF_EXTERNAL, &extssf);
-	ret = sasl_setprop(conn, SASL_AUTH_EXTERNAL, session->username);
-	if (ret != SASL_OK) {
-		errno = EACCES;
-		SET_ERROR(err, SASL_ERROR, ret);
-		return 0;
-	}
-
-
 	ret = mysasl_negotiate(session, conn, err);
 	if (ret != SASL_OK) {
 		errno = EACCES;
@@ -629,13 +653,10 @@ int init_socket(nuauth_session_t * session,
 	/* get address informations */
 	ecode = getaddrinfo(hostname, service, &hints, &res);
 	if (ecode != 0) {
-		if (session->verbose) {
-			fprintf(stderr,
-				"Fail to create host address: %s\n",
-				gai_strerror(ecode));
-			fprintf(stderr, "(host=\"%s\", service=\"%s\")\n",
-				hostname, service);
-		}
+		nu_error_log(err,
+			     "Fail to create host address (host=\"%s\", service=\"%s\"): %s",
+			     hostname, service,
+			     gai_strerror(ecode));
 		SET_ERROR(err, INTERNAL_ERROR, DNS_RESOLUTION_ERR);
 		return 0;
 	}
@@ -652,11 +673,9 @@ int init_socket(nuauth_session_t * session,
 		} else if (res->ai_family == AF_INET6 && session->src_addr.ss_family == AF_INET) {
 			uint32_to_ipv6(src4->sin_addr.s_addr, &src6->sin6_addr);
 		} else {
-			if (session->verbose) {
-				fprintf(stderr,
-						"Unable to set source address: host (%s) is not IPv6!",
-						hostname);
-			}
+			nu_error_log(err,
+					"Unable to set source address: host (%s) is not IPv6!",
+					hostname);
 			SET_ERROR(err, INTERNAL_ERROR, BINDING_ERR);
 			return 0;
 		}
@@ -728,14 +747,19 @@ int get_first_x509_cert_from_tls_session(gnutls_session session,
 }
 
 
-int certificate_check(nuauth_session_t *session)
+int certificate_check(nuauth_session_t *session, const char *hostname,
+		      nuclient_error_t *err)
 {
 	time_t expiration_time, activation_time;
 	gnutls_x509_crt cert;
+	char certbuf[256];
+	size_t size;
+	int ret;
 
-	if (get_first_x509_cert_from_tls_session(session->tls, &cert)
+	if ((ret = get_first_x509_cert_from_tls_session(session->tls, &cert))
 			!= SASL_OK) {
-		printf("TLS error: Cannot get first x509 cert from TLS session\n");
+		nu_error_log(err, "TLS error: Cannot get first x509 cert from TLS session");
+		SET_ERROR(err, INTERNAL_ERROR, ret);
 		return SASL_BADPARAM;
 	}
 
@@ -745,13 +769,15 @@ int certificate_check(nuauth_session_t *session)
 	/* verify date */
 	if (expiration_time < time(NULL)) {
 		gnutls_x509_crt_deinit(cert);
-		printf("TLS error: Certificate expired\n");
+		nu_error_log(err, "TLS error: Certificate expired");
+		SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_EXPIRED);
 		return SASL_EXPIRED;
 	}
 
 	if (activation_time > time(NULL)) {
 		gnutls_x509_crt_deinit(cert);
-		printf("TLS error: Certificate not yet activated\n");
+		nu_error_log(err, "TLS error: Certificate not yet activated");
+		SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_INSUFFICIENT_CREDENTIALS);
 		return SASL_DISABLED;
 	}
 
@@ -765,18 +791,56 @@ int certificate_check(nuauth_session_t *session)
 		}
 		if (strcmp(dn, session->nuauth_cert_dn)) {
 			gnutls_x509_crt_deinit(cert);
-			printf("TLS error: DN does not match the certificate DN\n");
+			nu_error_log(err, "TLS error: DN does not match the certificate DN");
+			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_INSUFFICIENT_CREDENTIALS);
 			return SASL_DISABLED;
 		}
 	}
 
+	size = sizeof(certbuf);
+	ret = gnutls_x509_crt_get_dn_by_oid(cert, GNUTLS_OID_X520_COMMON_NAME,
+			0, /* the first and only one */
+			0,
+			certbuf,
+			&size);
+
+	if (ret) {
+		nu_error_log(err, "TLS: error fetching CN from cert:%s",
+				gnutls_strerror(ret));
+		SET_ERROR(err, GNUTLS_ERROR, ret);
+		gnutls_x509_crt_deinit(cert);
+		return 1;
+	}
+
+	/* This function will check if the given certificate's subject matches the
+	 * given hostname. This is a basic implementation of the matching described
+	 * in RFC2818 (HTTPS), which takes into account wildcards, and the subject
+	 * alternative name PKIX extension. Returns non zero on success, and zero on
+	 * failure. */
+	ret = gnutls_x509_crt_check_hostname(cert, hostname);
+
+	if (!ret) {
+		if (session->need_fqdn_verif) {
+			nu_error_log(err, "TLS: certificate subject name (%s) does not match target host name '%s'\n",
+				certbuf, hostname);
+			SET_ERROR(err, GNUTLS_ERROR, ret);
+			gnutls_x509_crt_deinit(cert);
+			return 1;
+		}
+		else
+			printf("SSL: certificate subject name (%s) does not match target host name '%s', but continuing (check is disabled by config)\n",
+				certbuf, hostname);
+	}
+
+
+	gnutls_x509_crt_deinit(cert);
 	return SASL_OK;
 }
 
 /**
  * Do the TLS handshake and check server certificate
  */
-int tls_handshake(nuauth_session_t * session, nuclient_error_t * err)
+int tls_handshake(nuauth_session_t * session, const char *hostname, nuclient_error_t * err)
 {
 	int ret;
 	unsigned int status = 0;
@@ -791,51 +855,98 @@ int tls_handshake(nuauth_session_t * session, nuclient_error_t * err)
 	} while (ret < 0 && !gnutls_error_is_fatal(ret));
 
 	if (ret < 0) {
+		/* Verify the certificate even if handshake failed, so that we
+		 * have a good error message if the cause of the failure is an
+		 * invalid certificate
+		 */
+		gnutls_certificate_verify_peers2(session->tls, &status);
+		if (status) {
+			char msgstring[512];
+			char *prefix = "Certificate authority verification failed:";
+			int length = 512 - strlen(prefix) - 1;
+			snprintf(msgstring, 512, prefix);
+			if (status & GNUTLS_CERT_INVALID) {
+				char *msg = " invalid";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+			if (status & GNUTLS_CERT_REVOKED) {
+				char *msg = ", revoked";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+			if (status & GNUTLS_CERT_SIGNER_NOT_FOUND) {
+				char *msg = ", signer not found";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+			if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
+				char *msg = ", signer not a CA";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
+			if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
+				strncat(msgstring, ", use of insecure algorithm", length);
+			}
+#endif
+			nu_error_log(err, msgstring);
+			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_CERTIFICATE_ERROR);
+		}
 		gnutls_perror(ret);
 		errno = ECONNRESET;
-		SET_ERROR(err, GNUTLS_ERROR, ret);
+		SET_ERROR(err, INTERNAL_ERROR, TLS_HANDSHAKE_ERR);
 		return 0;
 	}
 
 	/* certificate verification */
-	if ( session->need_ca_verif )
-	{
+	if (session->need_ca_verif) {
 		ret = gnutls_certificate_verify_peers2(session->tls, &status);
 		if (ret < 0) {
-			if (session->verbose) {
-				printf("Certificate authority verification failed: %s\n",
-				       gnutls_strerror(ret));
-			}
+			nu_error_log(err, "Certificate authority verification failed: %s",
+				     gnutls_strerror(ret));
 			SET_ERROR(err, GNUTLS_ERROR, ret);
 			return 0;
 		}
 		if (status) {
-			if (session->verbose) {
-				printf("Certificate authority verification failed:");
-				if( status & GNUTLS_CERT_INVALID )
-					printf(" invalid");
-				if( status & GNUTLS_CERT_REVOKED )
-					printf(", revoked");
-				if( status & GNUTLS_CERT_SIGNER_NOT_FOUND )
-					printf(", signer not found");
-				if( status & GNUTLS_CERT_SIGNER_NOT_CA )
-					printf(", signer not a CA");
-#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
-				if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
-					printf(", use of insecure algorithm");
-#endif
-
-				printf("\n");
+			char msgstring[512];
+			char *prefix = "Certificate authority verification failed:";
+			int length = 512 - strlen(prefix) - 1;
+			snprintf(msgstring, 512, prefix);
+			if (status & GNUTLS_CERT_INVALID) {
+				char *msg = " invalid";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
 			}
+			if (status & GNUTLS_CERT_REVOKED) {
+				char *msg = ", revoked";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+			if (status & GNUTLS_CERT_SIGNER_NOT_FOUND) {
+				char *msg = ", signer not found";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+			if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
+				char *msg = ", signer not a CA";
+				strncat(msgstring, msg, length);
+				length -= strlen(msg);
+			}
+#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
+			if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
+				strncat(msgstring, ", use of insecure algorithm", 512);
+#endif
+			nu_error_log(err, msgstring);
 			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_CERTIFICATE_ERROR);
 			return 0;
 		}
 	}
 
-	ret = certificate_check(session);
+	ret = certificate_check(session, hostname, err);
 	if (ret != SASL_OK) {
 		if (session->verbose) {
-			printf("Certificate check  failed: %s\n",
+			nu_error_log(err, "Certificate check failed: %s",
 			       gnutls_strerror(ret));
 		}
 		SET_ERROR(err, GNUTLS_ERROR, ret);
@@ -844,6 +955,7 @@ int tls_handshake(nuauth_session_t * session, nuclient_error_t * err)
 
 	if (session->verbose)
 		printf("Server Certificate OK\n");
+
 	return 1;
 }
 
