@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: internal.h 3991 2007-11-30 16:02:50Z lds $
+ ** $Id: internal.h 4996 2008-10-21 13:14:16Z pollux $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -28,9 +28,9 @@ int init_socket(nuauth_session_t * session,
 		const char *hostname, const char *service,
 		nuclient_error_t *err);
 
-int tls_handshake(nuauth_session_t * session, nuclient_error_t * err);
+int tls_handshake(nuauth_session_t * session, const char *hostname, nuclient_error_t * err);
 
-int init_sasl(nuauth_session_t * session, nuclient_error_t * err);
+int init_sasl(nuauth_session_t * session, const char *hostname, nuclient_error_t * err);
 
 int send_os(nuauth_session_t * session, nuclient_error_t * err);
 
