@@ -1,10 +1,10 @@
 /*
- ** Copyright 2004-2007 - INL
+ ** Copyright 2004-2008 - INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: libnuclient.c 4544 2008-03-06 12:51:03Z gryzor $
+ ** $Id: libnuclient.c 5466 2008-12-30 15:55:50Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -78,6 +78,7 @@ void nu_exit_clean(nuauth_session_t * session)
 
 	secure_str_free(session->username);
 	secure_str_free(session->password);
+	free(session->ca_file);
 
 	if (session->cred) {
 		gnutls_certificate_free_keys(session->cred);
@@ -154,8 +155,9 @@ int nu_client_global_init(nuclient_error_t * err)
 	/* get local charset */
 	nu_locale_charset = nl_langinfo(CODESET);
 	if (nu_locale_charset == NULL) {
-		fprintf(stderr, "Can't get locale charset!\n");
-		exit(EXIT_FAILURE);
+		nu_error_log(err, "Can't get locale charset!");
+		SET_ERROR(err, INTERNAL_ERROR, UNKNOWN_ERR);
+		return 0;
 	}
 
 	load_sys_config();
@@ -173,6 +175,7 @@ void nu_client_global_deinit()
 {
 	sasl_done();
 	gnutls_global_deinit();
+	free_sys_config();
 }
 
 /**
@@ -227,6 +230,30 @@ char *nu_get_home_dir()
 	return dir;
 }
 
+/**
+ * \ingroup nuclientAPI
+ * Get user name
+ *
+ * \return A string that need to be freed
+ */
+
+char *nu_get_user_name()
+{
+	uid_t uid;
+	struct passwd *pwd;
+	char *name = NULL;
+
+	uid = getuid();
+	if (!(pwd = getpwuid(uid))) {
+		printf("Unable to get password file record\n");
+		endpwent();
+		return NULL;
+	}
+	name = strdup(pwd->pw_name);
+	endpwent();
+	return name;
+}
+
 int check_key_perms(const char* filename)
 {
 	struct stat infos;
@@ -235,14 +262,148 @@ int check_key_perms(const char* filename)
 		return 0;
 	}
 
-	/* Check it's owned by the current user */
-	if (infos.st_mode & S_IRGRP || infos.st_mode & S_IROTH) {
+	/* File should not be readable or writable by others */
+	if (infos.st_mode & S_IROTH || infos.st_mode & S_IWOTH) {
 		return 0;
 	}
 
 	return 1;
 }
 
+static int _cb_request_cert(gnutls_session_t session, const gnutls_datum_t* req_ca_dn, int nreqs, const gnutls_pk_algorithm_t* pk_algos, int pk_algos_length, gnutls_retr_st* st)
+{
+	printf("TLS error: server requests certificate, none configured\n");
+	return 0;
+}
+
+/* read the file indicated (by na1me) in the fname parameter.  store
+   its entire contents in a single datum. */
+int set_datum_file(gnutls_datum_t* d, const char* fname) {
+	struct stat sbuf;
+	unsigned char* c = NULL;
+	FILE* file = NULL;
+	size_t x = 0;
+
+	if (0 != stat(fname, &sbuf)) {
+		fprintf(stderr, "Could not access file '%s'\n", fname);
+		return -1;
+	}
+
+	c = gnutls_realloc(d->data, sbuf.st_size);
+	if (NULL == c) {
+		fprintf(stderr, "failed to allocate %ld bytes for '%s'\n", (long)sbuf.st_size, fname);
+		return -1;
+	}
+
+	d->data = c;
+	d->size = sbuf.st_size;
+
+	file = fopen(fname, "r");
+	if (NULL == file) {
+		fprintf(stderr, "failed to open '%s' for reading\n",  fname);
+		return -1;
+	}
+
+	x = fread(d->data, d->size, 1, file);
+	if (x != 1) {
+		fprintf(stderr, "tried to read %d bytes, read %zd instead from '%s'\n", d->size, x, fname);
+		fclose(file);
+		return -1;
+	}
+
+	fclose(file);
+	return 0;
+}
+
+
+/* return -1 in case of error or expiration time if operation is successful */
+int check_crl_validity(const char *crl_file, const char *ca_file,
+			      nuclient_error_t *err)
+{
+	gnutls_x509_crt_t ca;
+	gnutls_x509_crl_t crl;
+	gnutls_datum_t datum_crl;
+	gnutls_datum_t datum_ca;
+	time_t t, now;
+	int ret;
+
+	if (!ca_file || !crl_file)
+		return -1;
+
+	datum_ca.data = NULL;
+	datum_crl.data = NULL;
+
+	ret = set_datum_file(&datum_crl, crl_file);
+	if (ret) {
+	        nu_error_log(err, "TLS: Could not read CRL file \"%s\"", crl_file);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		return -1;
+	}
+	ret = set_datum_file(&datum_ca, ca_file);
+	if (ret) {
+	        nu_error_log(err, "TLS: Could not read CA file \"%s\"", ca_file);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		gnutls_free(datum_crl.data);
+		return -1;
+	}
+
+	gnutls_x509_crt_init(&ca);
+	gnutls_x509_crl_init(&crl);
+
+	ret = gnutls_x509_crl_import(crl, &datum_crl, GNUTLS_X509_FMT_PEM);
+	if (ret) {
+	        nu_error_log(err, "TLS: Could not import CRL data from \"%s\"",
+			     crl_file);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		gnutls_free(datum_ca.data);
+		gnutls_free(datum_crl.data);
+		return -1;
+	}
+
+	ret = gnutls_x509_crt_import(ca, &datum_ca, GNUTLS_X509_FMT_PEM);
+	if (ret) {
+	        nu_error_log(err, "TLS: Could not import CA data from \"%s\"",
+			     crl_file);
+		SET_ERROR(err, GNUTLS_ERROR, ret);
+		gnutls_free(datum_ca.data);
+		gnutls_free(datum_crl.data);
+		return -1;
+	}
+
+
+	/* Check if CRL was signed by configured CA */
+	ret = gnutls_x509_crl_check_issuer (crl, ca);
+	if (ret != 1) {
+	        nu_error_log(err, "TLS: CRL issuer is NOT the configured certificate authority for \"%s\"",
+			     crl_file);
+		SET_ERROR(err, GNUTLS_ERROR, ret);
+		return -1;
+	}
+
+	/* Check if CRL has expired */
+	now = time(NULL);
+	t = gnutls_x509_crl_get_next_update (crl);
+	/* This field is optional in a CRL */
+	if (t != (time_t)-1 ) {
+		if (now > t) {
+			nu_error_log(err, "TLS: CRL \"%s\" has expired and should be re-issued."
+				          " TLS session will not be established.",
+				     crl_file);
+			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_EXPIRED);
+			/* this is only a warning, do not decrement return_value */
+			return -1;
+		}
+	}
+
+	gnutls_x509_crt_deinit(ca);
+	gnutls_x509_crl_deinit(crl);
+	gnutls_free(datum_ca.data);
+	gnutls_free(datum_crl.data);
+
+	return t;
+}
+
+
 /**
  * \ingroup nuclientAPI
  * Initialize TLS:
@@ -253,151 +414,122 @@ int check_key_perms(const char* filename)
  *    - Init. TLS session
  *
  * \param session Pointer to client session
- * \param keyfile Complete path to a key file stored in PEM format (can be NULL)
- * \param certfile Complete path to a certificate file stored in PEM format (can be NULL)
- * \param cafile Complete path to a certificate authority file stored in PEM format (can be NULL)
+ * \param user_keyfile Complete path to a key file stored in PEM format (can be NULL)
+ * \param user_certfile Complete path to a certificate file stored in PEM format (can be NULL)
+ * \param user_cafile Complete path to a certificate authority file stored in PEM format (can be NULL)
  * \param tls_password Certificate password string
  * \param err Pointer to a nuclient_error_t: which contains the error
  * \return Returns 0 on error (error description in err), 1 otherwise
  */
 int nu_client_setup_tls(nuauth_session_t * session,
-			char *keyfile, char *certfile, char *cafile,
+			char *user_keyfile, char *user_certfile, char *user_cafile,
 			char *tls_password, nuclient_error_t * err)
 {
-	char castring[256];
-	char certstring[256];
-	char keystring[256];
-	char *home = nu_get_home_dir();
-	int exit_on_error = 0;
-	int ok;
+	const char *keyfile = NULL;
+	const char *certfile = NULL;
+	const char *cafile = NULL;
 	int ret;
 
 	session->tls_password = tls_password;
 
-	/* If the user specified a certficate and a key on command line,
-	 * exit if we fail loading them.
-	 * Elsewise, try loading certs from ~/.nufw/, but continue if we fail
-	 */
-	if (certfile || keyfile)
-		exit_on_error = 1;
-
-	/* compute patch keyfile */
-	if (keyfile == NULL && home != NULL) {
-		ok = secure_snprintf(keystring, sizeof(keystring),
-				     "%s/.nufw/key.pem", home);
-		if (ok)
-			keyfile = keystring;
+	if (user_cafile) {
+		cafile = user_cafile;
+	} else {
+		cafile = nu_client_default_tls_ca();
 	}
 
-	/* test if key file exists */
-	if (keyfile != NULL && access(keyfile, R_OK) != 0) {
-		if (exit_on_error) {
-			if (session->verbose)
-			    printf("Unable to load key file: %s\n", keyfile);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			if (home) {
-				free(home);
-			}
-			errno = EBADF;
-			return 0;
-		}
-		keyfile = NULL;
-	}
-
-	if (certfile == NULL && home != NULL) {
-		ok = secure_snprintf(certstring, sizeof(certstring),
-				     "%s/.nufw/cert.pem", home);
-		if (ok)
-			certfile = certstring;
-	}
-	/* test if cert exists */
-	if (certfile != NULL && access(certfile, R_OK) != 0) {
-		if (exit_on_error) {
-			printf("Unable to load certificate file : %s\n", certfile);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			if (home) {
-				free(home);
-			}
-			errno = EBADF;
-			return 0;
-		}
-		certfile = NULL;
-	}
-	if (cafile == NULL && home != NULL) {
-		ok = secure_snprintf(castring, sizeof(castring),
-				     "%s/.nufw/cacert.pem", home);
-		if (ok)
-			cafile = castring;
-		/* if computed cafile is not present we reset to cafile to NULL */
-		if (access(cafile, R_OK) != 0) {
-			if (errno == EACCES) {
-				printf("Warning, unable to access CA file : %s\n", cafile);
-			}
-			cafile = NULL;
-		}
-	}
-	/* test if cert exists */
+	/* test if ca exists */
 	if (cafile != NULL && access(cafile, R_OK) != 0) {
-		if (exit_on_error) {
-			printf("Unable to load CA file : %s\n", cafile);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			errno = EBADF;
-			if (home) {
-				free(home);
-			}
-			return 0;
-		}
-		cafile = NULL;
+		nu_error_log(err, "Unable to read CA file \"%s\"", cafile);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		errno = EBADF;
+		return 0;
 	}
 
 	/* sets the trusted cas file */
-	if (cafile != NULL)
-	{
-		ret =
-		    gnutls_certificate_set_x509_trust_file(session->cred, cafile,
+	if (cafile != NULL) {
+		ret = gnutls_certificate_set_x509_trust_file(session->cred, cafile,
 							   GNUTLS_X509_FMT_PEM);
 		if (ret < 0) {
 			SET_ERROR(err, GNUTLS_ERROR, ret);
 			return 0;
 		}
 		session->need_ca_verif = 1;
+		session->ca_file = strdup(cafile);
+	} else {
+		session->ca_file = NULL;
+	}
+
+	/* compute patch keyfile */
+	if (user_keyfile) {
+		keyfile = user_keyfile;
+	} else {
+		keyfile = nu_client_default_tls_key();
+	}
+
+	/* test if key file exists */
+	if (keyfile != NULL && access(keyfile, R_OK) != 0) {
+		nu_error_log(err, "Unable to read key file \"%s\"", keyfile);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		errno = EBADF;
+		return 0;
+	}
+
+	if (user_certfile) {
+		certfile = user_certfile;
+	} else {
+		certfile = nu_client_default_tls_cert();
+	}
+
+	/* test if cert exists */
+	if (certfile != NULL && access(certfile, R_OK) != 0) {
+		nu_error_log(err, "Unable to read certificate file \"%s\"", certfile);
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		errno = EBADF;
+		return 0;
 	}
 
 	if (keyfile && !check_key_perms(keyfile)) {
-		printf("Private key has too open permissions.\n");
+		nu_error_log(err, "Permissions on private key \"%s\" are not restrictive enough, file should not be readable or writable by others.",
+			     keyfile);
 		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		errno = EPERM;
 		return 0;
 	}
 
 	if (certfile != NULL && keyfile != NULL) {
-		ret =
-		    gnutls_certificate_set_x509_key_file(session->cred,
+		ret = gnutls_certificate_set_x509_key_file(session->cred,
 							 certfile, keyfile,
 							 GNUTLS_X509_FMT_PEM);
 		if (ret < 0) {
+			nu_error_log(err, "Can't use x509 files (%s)", gnutls_strerror(ret));
 			SET_ERROR(err, GNUTLS_ERROR, ret);
-			if (home) {
-				free(home);
-			}
 			return 0;
 		}
+	} else {
+		if (keyfile != NULL) {
+			nu_error_log(err, "key file \"%s\" is set but no cert"
+					" file is.", keyfile);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			return 0;
+		} else if (certfile != NULL) {
+			nu_error_log(err, "cert file \"%s\" is set but no key"
+					" file is.", certfile);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			return 0;
+		}
+		gnutls_certificate_client_set_retrieve_function(session->cred,
+				&_cb_request_cert);
 	}
 
 	/* put the x509 credentials to the current session */
-	ret =
-	    gnutls_credentials_set(session->tls, GNUTLS_CRD_CERTIFICATE,
+	ret = gnutls_credentials_set(session->tls, GNUTLS_CRD_CERTIFICATE,
 				   session->cred);
 	if (ret < 0) {
 		SET_ERROR(err, GNUTLS_ERROR, ret);
-		if (home) {
-			free(home);
-		}
 		return 0;
 	}
 	session->need_set_cred = 0;
-	if (home) {
-		free(home);
-	}
 	return 1;
 }
 /**
@@ -413,6 +545,33 @@ int nu_client_set_nuauth_cert_dn(nuauth_session_t * session,
 	return 1;
 }
 
+/**
+ * \ingroup nuclientAPI
+ */
+int nu_client_set_crlfile(nuauth_session_t * session,
+				char *crlfile,
+				nuclient_error_t *err)
+{
+	if (crlfile && *crlfile) {
+		session->crl_file = crlfile;
+		return 1;
+	}
+	return 0;
+}
+
+
+/**
+ * \ingroup nuclientAPI
+ */
+int nu_client_set_krb5_service(nuauth_session_t * session,
+				char *service)
+{
+	if (service) {
+		session->krb5_service = service;
+	}
+	return 1;
+}
+
 /**
  * \ingroup nuclientAPI
  * Set IP source of the socket used to connect to nuauth server
@@ -548,7 +707,7 @@ nuauth_session_t *_nu_client_new(nuclient_error_t * err)
 
 nuauth_session_t *nu_client_new_callback(void *username_callback,
 		      void *passwd_callback,
-		      /* TODO: To remove */ unsigned char diffie_hellman, nuclient_error_t * err)
+		      unsigned char diffie_hellman, nuclient_error_t * err)
 {
 	nuauth_session_t *session = NULL;
 
@@ -582,7 +741,7 @@ nuauth_session_t *nu_client_new_callback(void *username_callback,
 
 nuauth_session_t *nu_client_new(const char *username,
 		      const char *password,
-		      /* TODO: To remove */ unsigned char diffie_hellman, nuclient_error_t * err)
+		      unsigned char diffie_hellman, nuclient_error_t * err)
 {
 	nuauth_session_t *session = NULL;
 
@@ -654,12 +813,12 @@ int nu_client_connect(nuauth_session_t * session,
 		      const char *hostname, const char *service,
 		      nuclient_error_t * err)
 {
+	int ret;
 	if (session->need_set_cred) {
 		/* put the x509 credentials to the current session */
-		int ret =
-		    gnutls_credentials_set(session->tls,
-					   GNUTLS_CRD_CERTIFICATE,
-					   session->cred);
+		ret = gnutls_credentials_set(session->tls,
+					     GNUTLS_CRD_CERTIFICATE,
+					     session->cred);
 		if (ret < 0) {
 			SET_ERROR(err, GNUTLS_ERROR, ret);
 			return 0;
@@ -667,16 +826,41 @@ int nu_client_connect(nuauth_session_t * session,
 		session->need_set_cred = 0;
 	}
 
+	if (session->crl_file) {
+		struct stat stats;
+		if (stat(session->crl_file, &stats) < 0) {
+			nu_error_log(err, "Can't access CRL file \"%s\"",
+					session->crl_file);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			return 0;
+		}
+		session->crl_file_mtime = stats.st_mtime;
+
+		if (check_crl_validity(session->crl_file, session->ca_file, err) == -1) {
+			return 0;
+		}
+		ret = gnutls_certificate_set_x509_crl_file(session->cred,
+							session->crl_file,
+							GNUTLS_X509_FMT_PEM);
+		if (ret < 0) {
+			SET_ERROR(err, GNUTLS_ERROR, ret);
+			nu_error_log(err, "CRL file (name %s) problem: %s",
+				session->crl_file,
+				gnutls_strerror(ret));
+			return 0;
+		}
+	}
+
 	/* set field about host */
 	if (!init_socket(session, hostname, service, err)) {
 		return 0;
 	}
 
-	if (!tls_handshake(session, err)) {
+	if (!tls_handshake(session, hostname, err)) {
 		return 0;
 	}
 
-	if (!init_sasl(session, err)) {
+	if (!init_sasl(session, hostname, err)) {
 		return 0;
 	}
 
@@ -723,6 +907,7 @@ int nu_client_error_init(nuclient_error_t ** err)
 	*err = malloc(sizeof(nuclient_error_t));
 	if (*err == NULL)
 		return -1;
+	(*err)->strerror[0] = 0;
 	return 0;
 }
 
@@ -744,6 +929,9 @@ const char *nu_client_strerror(nuclient_error_t * err)
 {
 	if (err == NULL)
 		return "Error structure was not initialised";
+	if (err->strerror[0]) {
+		return err->strerror;
+	}
 	switch (err->family) {
 	case GNUTLS_ERROR:
 		return gnutls_strerror(err->error);
@@ -777,6 +965,8 @@ const char *nu_client_strerror(nuclient_error_t * err)
 			return "Bad credentials";
 		case BINDING_ERR:
 			return "Binding (source address) error";
+		case TLS_HANDSHAKE_ERR:
+			return "TLS handshake failed";
 		default:
 			return "Unknown internal error code";
 		}
