@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: libnuclient.h 3890 2007-10-31 13:59:18Z regit $
+ ** $Id: libnuclient.h 5466 2008-12-30 15:55:50Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -34,13 +34,16 @@ int compare(nuauth_session_t * session, conntable_t * old, conntable_t * new,
 void do_panic(const char *filename, unsigned long line, const char *fmt,
 	      ...);
 
+void nu_error_log(nuclient_error *err, const char *fmt, ...);
+
+
 /**
  * \def panic(format, ...)
  *
  * Call do_panic(__FILE__, __LINE__, format, ...)
  */
 #define panic(format, args...) \
-    do_panic(__FILE__, __LINE__, format, ##args )
+	do_panic(__FILE__, __LINE__, format, ##args )
 
 /**
  * \def nu_assert(test, format, ...)
@@ -48,8 +51,10 @@ void do_panic(const char *filename, unsigned long line, const char *fmt,
  * If test fails, call do_panic(__FILE__, __LINE__, format, ...)
  */
 #define nu_assert(test, format, args...) \
-    do { if (!(test)) do_panic(__FILE__, __LINE__, format, ##args ); } while (0)
+	do { if (!(test)) do_panic(__FILE__, __LINE__, format, ##args ); } while (0)
 
 void ask_session_end(nuauth_session_t * session);
+int check_crl_validity(const char *crl_file, const char *ca_file,
+			      nuclient_error_t *err);
 
 #endif
