@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nuclient.h 4462 2008-02-14 16:26:51Z lds $
+ ** $Id: nuclient.h 5466 2008-12-30 15:55:50Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -87,6 +87,9 @@ extern "C" {
 /** Default filename of key */
 #define KEYFILE "key.pem"
 
+/** Default Kerberos realm */
+#define DEFAULT_KRB5_REALM "nuauth"
+
 /** Timeout of UDP connections */
 #define UDP_TIMEOUT 30
 
@@ -131,13 +134,9 @@ typedef struct {
 
 /* only publicly seen structure but data are private */
 
-#ifndef USE_SHA1
 #  define PACKET_ITEM_MAXSIZE \
 	( sizeof(struct nu_authreq) + sizeof(struct nu_authfield_ipv6) \
 	  + sizeof(struct nu_authfield_app) + PROGNAME_BASE64_WIDTH )
-#else
-#  error "TODO: Compute PACKET_ITEM_MAXSIZE with SHA1 checksum"
-#endif
 
 #define PACKET_SIZE \
 	( sizeof(struct nu_header) + CONN_MAX * PACKET_ITEM_MAXSIZE )
@@ -162,16 +161,20 @@ typedef struct {
 
 	gnutls_session tls;	/*!< TLS session over TCP socket */
 	gnutls_certificate_credentials cred;	/*!< TLS credentials */
+	char *ca_file;
+	char *crl_file;
 	char *tls_password;	/*!< TLS password */
 	char *nuauth_cert_dn;
 
+	char *krb5_service;
+
 	int socket;	/*!< TCP socket used to exchange message with nuauth */
 	conntable_t *ct;	/*!< Connection table */
 	u_int32_t packet_seq;	/*!< Packet sequence number (start at zero) */
 	int auth_by_default;	/*!< Auth. by default (=1) */
 	unsigned char debug_mode;	/*!< Debug mode, enabled if different than zero */
 	unsigned char verbose;	/*!< Verbose mode (default: enabled) */
-	/* TODO: To remove */ unsigned char diffie_hellman;	/*!< Use Diffie Hellman for key exchange? */
+	unsigned char diffie_hellman;	/*!< Use Diffie Hellman for key exchange? */
 	int has_src_addr;		/*!< Has source address? */
 	struct sockaddr_storage src_addr;	/*!< Source address */
 
@@ -221,9 +224,17 @@ typedef struct {
 	/** Timestamp (Epoch format) of last packet send to nuauth */
 	time_t timestamp_last_sent;
 
+	/** CRL file modification time */
+	time_t crl_file_mtime;
+	time_t crl_file_exptime;
+	int check_counter;
+
 	/** Do we need to check the CA certificate */
 	unsigned char need_ca_verif;
 
+	/** Do we need to check the FQDN of server certificate */
+	unsigned char need_fqdn_verif;
+
 
 } nuauth_session_t;
 
@@ -253,15 +264,18 @@ enum {
 	SEND_ERR = 10,	     /** Fail to send packet to nuauth */
 	BAD_CREDENTIALS_ERR, /** Username/password error */
 	BINDING_ERR,	     /** bind() call failed */
+	TLS_HANDSHAKE_ERR,   /** TLS handshake failed */
 };
 
 /* Define for backward compatibility */
 #define nuclient_error nuclient_error_t
+#define NU_STRERROR_LEN 256
 
 /* libnuclient return code structure */
 typedef struct {
 	nuclient_error_family_t family;
 	int error;
+	char strerror[NU_STRERROR_LEN];
 } nuclient_error_t;
 
 /* Exported functions */
@@ -290,9 +304,16 @@ void nu_client_set_password(nuauth_session_t *session,
 		const char *password);
 
 const char* nu_client_default_hostname();
-	
+
 const char* nu_client_default_port();
 
+const char* nu_client_default_tls_ca();
+const char* nu_client_default_tls_cert();
+const char* nu_client_default_tls_key();
+const char* nu_client_default_tls_crl();
+
+int nu_client_default_suppress_fqdn_verif();
+
 void nu_client_set_debug(nuauth_session_t * session, unsigned char enabled);
 void nu_client_set_verbose(nuauth_session_t * session,
 		unsigned char enabled);
@@ -308,6 +329,13 @@ int nu_client_set_nuauth_cert_dn(nuauth_session_t * session,
 		char *nuauth_cert_dn,
 		nuclient_error_t *err);
 
+int nu_client_set_crlfile(nuauth_session_t * session,
+				char *crlfile,
+				nuclient_error_t *err);
+
+int nu_client_set_krb5_service(nuauth_session_t * session,
+		char *service);
+
 int nu_client_connect(nuauth_session_t * session,
 		const char *hostname,
 		const char *service,
@@ -325,6 +353,7 @@ const char *nu_get_version();
 int nu_check_version(const char *version);
 
 char *nu_get_home_dir();
+char *nu_get_user_name();
 
 int secure_snprintf(char *buffer, unsigned int buffer_size,
 		char *format, ...);
