@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: sending.c 3774 2007-09-26 00:18:39Z regit $
+ ** $Id: sending.c 5202 2008-11-18 17:06:26Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -132,21 +132,11 @@ int send_user_pckt(nuauth_session_t * session, conn_t * carray[CONN_MAX])
 		/* application field  */
 		appfield = (struct nu_authfield_app *) (authfield + 1);
 		appfield->type = APP_FIELD;
-#ifdef USE_SHA1
-		appfield->option = APP_TYPE_SHA1;
-#else
 		appfield->option = APP_TYPE_NAME;
-#endif
 		app_ptr = (char *) (appfield + 1);
 		sasl_encode64(appname, strlen(appname), app_ptr,
 			      PROGNAME_BASE64_WIDTH, &len);
-#ifdef USE_SHA1
-		*(app_ptr + len) = ';';
-		len++;
-		strcpy(app_ptr + len, sha1_sig);
-		len += strlen(sha1_sig);
-#endif
-		appfield->length = sizeof(appfield) + len;
+		appfield->length = sizeof(struct nu_authfield_app) + len;
 		authreq->packet_length += appfield->length;
 
 		/* glue piece together on data if packet is not too long */
