@@ -1,10 +1,10 @@
 /*
  ** Copyright 2004-2008 - INL
- ** Written by Eric Leblond <eric.leblond@inl.fr>
+ ** Written by Eric Leblond <eric@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nutcpc.c 4607 2008-04-03 12:58:08Z pollux $
+ ** $Id: nutcpc.c 5468 2008-12-31 10:34:06Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include <signal.h>
 #include "proto.h"
 #include "security.h"
-#define NUTCPC_VERSION PACKAGE_VERSION " $Revision: 4607 $"
+#define NUTCPC_VERSION PACKAGE_VERSION " $Revision: 5468 $"
 
 #define RETURN_NO_LOG return
 
@@ -47,6 +47,9 @@ nuclient_error_t *err = NULL;
 struct sigaction old_sigterm;
 struct sigaction old_sigint;
 int forced_reconnect = 0;
+int connected;
+static int suppress_ca_warning = 0;
+static int suppress_fqdn_verif = 0;
 
 typedef struct {
 	char port[10];		/*!< Port (service) number / name */
@@ -60,7 +63,9 @@ typedef struct {
 	char *certfile;
 	char *keyfile;
 	char *cafile;
+	char *crlfile;
 	char *cert_password;
+	char *krb5_service;
 } nutcpc_context_t;
 
 /**
@@ -107,33 +112,60 @@ char *compute_run_pid()
  *
  * Exit the program at the end of this function.
  */
-void kill_nutcpc()
+int test_nutcpc(pid_t *pid)
 {
-	pid_t pid;
 	FILE *fd;
-	int ret;
-	int ok = 0;
+	int ok = EXIT_FAILURE;
 	char *runpid = compute_run_pid();
 
 	if (runpid) {
 		fd = fopen(runpid, "r");
 		if (fd) {
-			fscanf(fd, "%d", &pid);
+			fscanf(fd, "%d", pid);
 			fclose(fd);
-			ret = kill(pid, SIGTERM);
-			ok = (ret == 0);
-			if (ok) {
-				printf("nutcpc process killed (pid %lu)\n",
-				       (unsigned long) pid);
-			} else {
-				printf
-				    ("Fail to kill process: remove pid file\n");
-				unlink(runpid);
-			}
+			/** \todo test if process pid is running */
+			ok = EXIT_SUCCESS;
 		}
 		free(runpid);
 	}
-	if (!ok) {
+	return ok;
+}
+
+/**
+ * Kill existing instance of nutcpc: read pid file,
+ * and then send SIGTERM to the process.
+ *
+ * Exit the program at the end of this function.
+ */
+void kill_nutcpc()
+{
+	pid_t pid;
+
+	if (test_nutcpc(&pid) == EXIT_SUCCESS) {
+		int ret;
+		char *runpid = compute_run_pid();
+		ret = kill(pid, SIGTERM);
+		if (ret == 0) {
+			printf("nutcpc process killed (pid %lu)\n",
+					(unsigned long) pid);
+			unlink(runpid);
+			free(runpid);
+			exit(EXIT_SUCCESS);
+		} else {
+			switch (errno) {
+				case ESRCH:
+					printf("Process does not exist: removing pid file\n");
+					unlink(runpid);
+					break;
+				case EINVAL:
+				case EPERM:
+				default:
+					printf("Bad return from kill\n");
+			}
+			free(runpid);
+			exit(EXIT_FAILURE);
+		}
+	} else {
 		printf("No nutcpc seems to be running\n");
 		exit(EXIT_FAILURE);
 	}
@@ -321,6 +353,35 @@ char *get_username()
 	return username;
 }
 
+/**
+ * Callback used for user validation
+ *
+ * \return New allocated buffer containing the answer,
+ *         or NULL if it fails
+ */
+char *get_user_validation(const char *msg)
+{
+	char *answer;
+	int nread;
+	size_t answer_size = 32;
+
+	if (msg) {
+		fprintf(stdout,"%s",msg);
+		fflush(stdout);
+	}
+	answer = (char *) calloc(answer_size, sizeof(char));
+	nread = getline(&answer, &answer_size, stdin);
+	if (nread < 0) {
+		free(answer);
+		printf("Problem when reading answer\n");
+		return NULL;
+	}
+	if (0 < nread && answer[nread - 1] == '\n') {
+		answer[nread - 1] = 0;
+	}
+	return answer;
+}
+
 /**
  * Print client usage.
  */
@@ -330,6 +391,7 @@ static void usage(void)
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Options:\n");
 	fprintf(stderr, "  -k: kill active client\n");
+	fprintf(stderr, "  -c: check if there is an active client\n");
 	fprintf(stderr, "  -l: don't create lock file\n");
 	fprintf(stderr, "  -V: display version\n");
 	fprintf(stderr, "\n");
@@ -338,6 +400,12 @@ static void usage(void)
 	fprintf(stderr, "  -A AUTHFILE: authority certificate filename\n");
 	fprintf(stderr, "  -K KEYFILE:  key filename\n");
 	fprintf(stderr, "  -W CERTPASS: certificate password\n");
+	fprintf(stderr, "  -R CRLFILE: crl filename\n");
+	fprintf(stderr, "  -Q: suppress warning if no certificate authority is configured\n");
+	fprintf(stderr, "  -N: suppress error if server FQDN does not match certificate CN.\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "SASL options:\n");
+	fprintf(stderr, "  -Z SERVICE: Kerberos service name (nuauth)\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Other options:\n");
 	fprintf(stderr, "  -p PORT: nuauth port number\n");
@@ -353,6 +421,8 @@ static void usage(void)
 void process_hup(int signum)
 {
 	forced_reconnect = 1;
+	nu_client_reset(session);
+	connected = 0;
 }
 
 /**
@@ -469,26 +539,28 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 	nu_client_set_debug(session, context->debug_mode);
 
 	/* Set hostname from libnuclient if it wasn't specified by the user */
-	if(*context->port == '\0')
-	{
-		if(nu_client_default_port())
+	if (*context->port == '\0') {
+		if (nu_client_default_port()) {
 			SECURE_STRNCPY(context->port, nu_client_default_port(),
 			       sizeof(context->port));
-		else
+		} else {
 			SECURE_STRNCPY(context->port, USERPCKT_SERVICE,
 				       sizeof(context->port));
+		}
 	}
 
-	if(*context->srv_addr == '\0')
-	{
-		if(nu_client_default_hostname())
+	if (*context->srv_addr == '\0') {
+		if (nu_client_default_hostname()) {
 			SECURE_STRNCPY(context->srv_addr, nu_client_default_hostname(),
 			       sizeof(context->srv_addr));
-		else
+		} else {
 			SECURE_STRNCPY(context->srv_addr, NUAUTH_IP,
 				       sizeof(context->srv_addr));
+		}
 	}
 
+	printf("Connecting to NuFW gateway (%s)\n", context->srv_addr);
+
 	if (!nu_client_setup_tls(session, context->keyfile, context->certfile,
 	     context->cafile, context->cert_password, err)) {
 		nu_client_delete(session);
@@ -496,6 +568,27 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 		return NULL;
 	}
 
+	if (context->crlfile) {
+		if (!nu_client_set_crlfile(session, context->crlfile, err)) {
+			nu_client_delete(session);
+			fprintf(stderr, "Error when setting crlfile: %s\n",
+				nu_client_strerror(err));
+			return NULL;
+		}
+	}
+
+	session->need_fqdn_verif = !suppress_fqdn_verif;
+
+	if (session->need_ca_verif == 0) {
+		if (suppress_ca_warning == 0)
+			fprintf(stderr,"\nWARNING: you have not provided any certificate authority.\n"
+				"nutcpc will *NOT* verify server certificate trust.\n"
+				"Use the -A <cafile> option to set up CA.\n"
+				"As certificate will not be trusted, disabling FQDN check.\n\n"
+				);
+		session->need_fqdn_verif = 0;
+	}
+
 	if (context->nuauthdn) {
 		if (!nu_client_set_nuauth_cert_dn(session,
 						  context->nuauthdn,
@@ -506,6 +599,14 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 		}
 	}
 
+	if (context->krb5_service) {
+		if (!nu_client_set_krb5_service(session, context->krb5_service)) {
+			nu_client_delete(session);
+			fprintf(stderr, "Unable to setup Kerberos5 service\n");
+			return NULL;
+		}
+	}
+
 	if (!nu_client_connect
 	    (session, context->srv_addr, context->port, err)) {
 		fprintf(stderr, "Unable to setup connect\n");
@@ -608,7 +709,7 @@ void parse_cmdline_options(int argc, char **argv,
 
 	/* Parse all command line arguments */
 	opterr = 0;
-	while ((ch = getopt(argc, argv, "kldqVu:H:I:U:p:P:a:K:C:A:W:")) != -1) {
+	while ((ch = getopt(argc, argv, "klcdqNQVH:I:U:p:P:a:K:C:A:R:W:Z:")) != -1) {
 		switch (ch) {
 		case 'H':
 			SECURE_STRNCPY(context->srv_addr, optarg,
@@ -635,6 +736,17 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'U':
 			*username = strdup(optarg);
 			break;
+		case 'c': {
+				  pid_t pid;
+				  if (test_nutcpc(&pid) == EXIT_SUCCESS) {
+					  printf("nutcpc already running (pid %u)\n",
+						 pid);
+					  exit(EXIT_SUCCESS);
+				  }
+				  printf("No running nutcpc\n");
+				  exit(EXIT_FAILURE);
+			  }
+			break;
 		case 'k':
 			kill_nutcpc();
 			break;
@@ -648,6 +760,12 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'q':
 			stealth = 1;
 			break;
+		case 'N':
+			suppress_fqdn_verif = 1;
+			break;
+		case 'Q':
+			suppress_ca_warning = 1;
+			break;
 		case 'a':
 			SECURE_STRNCPY(context->nuauthdn, optarg,
 				       sizeof(context->nuauthdn));
@@ -661,9 +779,15 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'A':
 			context->cafile = copy_filename(optarg);
 			break;
+		case 'R':
+			context->crlfile = copy_filename(optarg);
+			break;
 		case 'W':
 			context->cert_password = strdup(optarg);
 			break;
+		case 'Z':
+			context->krb5_service = strdup(optarg);
+			break;
 		default:
 			usage();
 		}
@@ -686,7 +810,7 @@ void parse_cmdline_options(int argc, char **argv,
 /**
  * Stupid wrapper
  */
-char *wrap_error_message(char *str)
+const char *wrap_error_message(const char *str)
 {
 	if (strncmp(str,"A TLS packet with unexpected length was received.",49) == 0) {
 		RETURN_NO_LOG("Access denied");
@@ -720,15 +844,45 @@ void init_library(nutcpc_context_t * context, char *username)
 		printf("Problem: %s\n", nu_client_strerror(err));
 		exit(EXIT_FAILURE);
 	}
+	/* options specificied on command line are taken prior
+	 * to options from configuration file
+	 */
+	suppress_fqdn_verif |= nu_client_default_suppress_fqdn_verif();
+	if (!context->cafile)
+		context->cafile = (char *)nu_client_default_tls_ca();
+	if (!context->certfile)
+		context->certfile = (char *)nu_client_default_tls_cert();
+	if (!context->keyfile)
+		context->keyfile = (char *)nu_client_default_tls_key();
+	if (!context->crlfile)
+		context->crlfile = (char *)nu_client_default_tls_crl();
+
+	if (!suppress_ca_warning && !context->cafile) {
+		char *reply;
+
+		reply = get_user_validation(
+				"*******   WARNING   ******\n"
+				"You are trying to connect to nuauth without configuring a certificate authority (CA).\n"
+				"You are vulnerable to attacks like man-in-the-middle.\n"
+				"Do you really want to do that ? Type \"yes\" to continue: ");
+
+		if (reply==NULL || strcasecmp(reply,"YES")!=0) {
+			fprintf(stderr,"Aborted");
+			exit(EXIT_FAILURE);
+		}
+		free(reply);
+
+		/* continue, libnuclient does not require a CA */
+	}
 
 	/* Init. library */
-	printf("Connecting to NuFW gateway (%s)\n", context->srv_addr);
 	session = do_connect(context, username);
 
 	/* Library failure? */
 	if (session == NULL) {
 		printf("Unable to initiate connection to NuFW gateway\n");
-		printf("Problem: %s\n", wrap_error_message(nu_client_strerror(err)));
+		if (err->error != 0)
+			printf("Problem: %s\n", wrap_error_message(nu_client_strerror(err)));
 		printf("Authentication failed (check parameters)\n");
 		exit(EXIT_FAILURE);
 	}
@@ -789,6 +943,10 @@ int main(int argc, char **argv)
 
 	install_signals();
 
+	if (!username) {
+		username = nu_get_user_name();
+	}
+
 	init_library(&context, username);
 
 	/*
