@@ -4,7 +4,7 @@
  ** INL http://www.inl.fr/
  ** Based on pam_permit by Andrew Morgan <morgan@parc.power.net> 1996/3/11
  **
- ** $Id: pam_nufw.c 3848 2007-10-29 08:36:17Z regit $
+ ** $Id: pam_nufw.c 5333 2008-11-25 16:30:10Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
  */
 
 
-#define _GNU_SOURCE
 #include "../lib/nuclient.h"
 #include <sys/resource.h>	/* setrlimit() */
 #include <stdio.h>
@@ -68,7 +67,9 @@ char *locale_charset = NULL;
 /* internal data */
 struct pam_nufw_s {
 	char nuauth_srv[BUFSIZ];	/* auth server to connect to */
+	unsigned char set_nuauth_srv;
 	char nuauth_port[20];	/* port to use on auth server */
+	unsigned char set_nuauth_port;
 	char file_lock[BUFSIZ];	/* file lock used to store pid */
 	char **no_auth_users;
 	int no_auth_cpt;
@@ -100,13 +101,15 @@ static char *_init_pam_nufw_s(struct pam_nufw_s *pn_s)
 
 	memset(pn_s, 0, sizeof(*pn_s));
 	SECURE_STRNCPY(pn_s->nuauth_srv, NUAUTH_SRV,
-		       sizeof(pn_s->nuauth_srv));
+			sizeof(pn_s->nuauth_srv));
 	SECURE_STRNCPY(pn_s->nuauth_port, NUAUTH_PORT,
-		       sizeof(pn_s->nuauth_port));
+			sizeof(pn_s->nuauth_port));
 	SECURE_STRNCPY(pn_s->file_lock, FILE_LOCK,
 		       sizeof(pn_s->file_lock));
 	pn_s->no_auth_users = NULL;
 	pn_s->no_auth_cpt = 0;
+	pn_s->set_nuauth_port = 0;
+	pn_s->set_nuauth_srv = 0;
 	return NULL;
 }
 
@@ -123,9 +126,11 @@ static int _pam_parse(int argc, const char **argv, struct pam_nufw_s *pn)
 		if (!strncmp(*argv, "server=", 7)) {
 			SECURE_STRNCPY(pn->nuauth_srv, *argv + 7,
 				       sizeof(pn->nuauth_srv));
+			pn->set_nuauth_srv = 1;
 		} else if (!strncmp(*argv, "port=", 5)) {
 			SECURE_STRNCPY(pn->nuauth_port, *argv + 5,
 				       sizeof(pn->nuauth_port));
+			pn->set_nuauth_port = 1;
 		} else if (!strncmp(*argv, "lock=", 5)) {
 			SECURE_STRNCPY(pn->file_lock, *argv + 5,
 				       sizeof(pn->file_lock));
@@ -251,6 +256,11 @@ int do_auth_on_user(const char *username)
 nuauth_session_t *do_connect(char *username, char *password, nuclient_error_t * err)
 {
 	nuauth_session_t *session = nu_client_new(username, password, 1, err);
+	char *cafile = (char *)nu_client_default_tls_ca();
+	char *certfile = (char *)nu_client_default_tls_cert();
+	char *keyfile = (char *)nu_client_default_tls_key();
+	char *crlfile = (char *)nu_client_default_tls_crl();
+
 	if (session == NULL) {
 		return NULL;
 	}
@@ -261,14 +271,34 @@ nuauth_session_t *do_connect(char *username, char *password, nuclient_error_t *
 	free(username);
 	free(password);
 
-#if 0
-	nu_client_set_debug(session, context->debug_mode);
+	if (nu_client_default_hostname() &&
+	    ! pn_s.set_nuauth_srv) {
+		SECURE_STRNCPY(pn_s.nuauth_srv,
+			       nu_client_default_hostname(),
+			       sizeof(pn_s.nuauth_srv));
+	}
+	if (nu_client_default_port() &&
+	    ! pn_s.set_nuauth_port) {
+		SECURE_STRNCPY(pn_s.nuauth_port,
+			       nu_client_default_port(),
+			       sizeof(pn_s.nuauth_port));
+	}
 
-	if (!nu_client_setup_tls(session, NULL, NULL, NULL, NULL, err)) {
+	if (!nu_client_setup_tls(session, keyfile, certfile,
+				 cafile, NULL, err)) {
 		nu_client_delete(session);
 		return NULL;
 	}
-#endif
+
+	if (crlfile) {
+		if (!nu_client_set_crlfile(session, crlfile, err)) {
+			nu_client_delete(session);
+			return NULL;
+		}
+	}
+
+
+	session->need_fqdn_verif = !nu_client_default_suppress_fqdn_verif();
 
 	if (!nu_client_connect
 	    (session, pn_s.nuauth_srv, pn_s.nuauth_port, err)) {
@@ -292,9 +322,9 @@ static void main_loop(struct pam_nufw_s *pn_s)
 				tempo = tempo * 2;
 			}
 
-			if (nu_client_connect
-			    (session, pn_s->nuauth_srv, pn_s->nuauth_port,
-			     pn_s->err) != 0) {
+			if (nu_client_connect(session, pn_s->nuauth_srv,
+					      pn_s->nuauth_port,
+					      pn_s->err) != 0) {
 				tempo = 1;
 				connected = 1;
 			} else {
@@ -344,11 +374,17 @@ static int nufw_client_func(struct pam_nufw_s *pn_s,
 	int res_err;
 
 	/* set user and group identifier, and home directory */
-	if (setuid(user_info->uid) != 0) {
-		syslog(LOG_ERR, "(pam_nufw) Fail to set sigaction");
+	if (setgid(user_info->gid) != 0) {
+		syslog(LOG_ERR, "(pam_nufw) Fail to set gid (%d)",
+		       user_info->gid);
 		return PAM_AUTH_ERR;
 	}
-	setgid(user_info->gid);
+	if (setuid(user_info->uid) != 0) {
+		syslog(LOG_ERR, "(pam_nufw) Fail to set uid (%d)",
+		       user_info->uid);
+		return PAM_AUTH_ERR;
+	}
+
 	setenv("HOME", user_info->home_dir, 1);
 
 	/* catch SIGINT and SIGTERM signals, install handler: exit_client() */
@@ -460,6 +496,11 @@ static int read_user_info(struct user_info_s *user_info,
 
 	/* read password, user and group identifier */
 	pw = (struct passwd *) getpwnam(user_info->username);
+	if (! pw) {
+		syslog(LOG_ERR, "pam_nufw failed to get username");
+		*pam_result = PAM_AUTH_ERR;
+		return 0;
+	}
 	user_info->uid = pw->pw_uid;
 	user_info->gid = pw->pw_gid;
 	user_info->home_dir = pw->pw_dir;
@@ -471,7 +512,7 @@ static int read_user_info(struct user_info_s *user_info,
  * used to open the connection to the nuauth server
  */
 PAM_EXTERN
-    int pam_sm_authenticate(pam_handle_t * pamh, int flags,
+int pam_sm_authenticate(pam_handle_t * pamh, int flags,
 			    int argc, const char **argv)
 {
 	int retval;
@@ -541,7 +582,7 @@ PAM_EXTERN
 }
 
 PAM_EXTERN
-    int pam_sm_setcred(pam_handle_t * pamh, int flags, int argc,
+int pam_sm_setcred(pam_handle_t * pamh, int flags, int argc,
 		       const char **argv)
 {
 	/*D(("pam_nufw sm_setcred")); */
@@ -551,7 +592,7 @@ PAM_EXTERN
 /* --- account management functions --- */
 
 PAM_EXTERN
-    int pam_sm_acct_mgmt(pam_handle_t * pamh, int flags, int argc,
+int pam_sm_acct_mgmt(pam_handle_t * pamh, int flags, int argc,
 			 const char **argv)
 {
 	D(("pam_nufw sm_acct_mgmt"));
@@ -561,7 +602,7 @@ PAM_EXTERN
 /* --- password management --- */
 
 PAM_EXTERN
-    int pam_sm_chauthtok(pam_handle_t * pamh, int flags, int argc,
+int pam_sm_chauthtok(pam_handle_t * pamh, int flags, int argc,
 			 const char **argv)
 {
 	D(("pam_nufw sm_chauthok"));
@@ -571,7 +612,7 @@ PAM_EXTERN
 /* --- session management --- */
 
 PAM_EXTERN
-    int pam_sm_open_session(pam_handle_t * pamh, int flags, int argc,
+int pam_sm_open_session(pam_handle_t * pamh, int flags, int argc,
 			    const char **argv)
 {
 	syslog(LOG_INFO, "(pam_nufw) session opened");
@@ -583,7 +624,7 @@ PAM_EXTERN
  * -> get pid file, and kill process
  */
 PAM_EXTERN
-    int pam_sm_close_session(pam_handle_t * pamh, int flags, int argc,
+int pam_sm_close_session(pam_handle_t * pamh, int flags, int argc,
 			     const char **argv)
 {
 	/*D(("pam_nufw sm_close_session")); */
