@@ -3,7 +3,7 @@
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nufw_source.h 3793 2007-10-08 16:48:27Z regit $
+ ** $Id: nufw_source.h 4401 2008-01-30 15:59:24Z lds $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -22,35 +22,10 @@
 #ifndef NUFW_SOURCE_H
 #define NUFW_SOURCE_H
 
-#ifdef _FEATURES_H
-#   error "nufw_source.h have to be included before <features.h>"
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
 #endif
 
-/**
- * Use POSIX standard, version "IEEE 1003.1-2004",
- * Neded by sigaction (signal.h) and timespec (time.h) for example
- */
-#if defined(__linux__)
-#  define _POSIX_C_SOURCE 199506L
-#endif
-
-/**
- * Use ISO C99 standard, needed by snprintf for example
- */
-#define _ISOC99_SOURCE
-
-/**
- * Use 4.3BSD standard,
- * needed to get 'tcphdr' structure and snprintf() function.
- */
-#define _BSD_SOURCE
-
-/**
- * Use SVID standard,
- * needed to get 'strdup' from <string.h>.
- */
-#define _SVID_SOURCE
-
 /* Disable inline keyword when compiling in strict ANSI conformance */
 #if defined(__STRICT_ANSI__) && !defined(__cplusplus)
 #  undef inline
