@@ -1,7 +1,7 @@
 /*
 ** Copyright(C) 2003-2007 Eric Leblond <regit@inl.fr>
 **
-** $Id: auth_srv.h 4056 2007-12-08 11:07:02Z regit $
+** $Id: auth_srv.h 5020 2008-10-28 13:28:35Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -119,6 +119,10 @@ typedef enum {
 #define NUAUTH_TLS_MAX_SERVERS 16
 #define TLS_CLIENT_MIN_DELAY 25000
 #define AUTH_NEGO_TIMEOUT 30
+/* define maximum of messages that can wait on a logging pool */
+#define MAX_UNASSIGNED_MESSAGES 2000
+
+#define POP_DELAY 500000 /* delay in nanosecond for timed pop call */
 
 #define UNKNOWN_STRING "UNKNOWN"
 
@@ -180,7 +184,7 @@ typedef enum {
  */
 #define MAX_NUFW_PACKET_SIZE 1800
 
-/*----------------------- SSL stuffs ----------------------------------*/
+/*----------------------- SSL stuff ----------------------------------*/
 
 /**
  * Default value for "nuauth_tls_key" option: filename of
@@ -217,17 +221,18 @@ typedef enum {
  * everybody have the same API version.
  */
 #define NUAUTH_API_VERSION ((uint32_t)( _NUAUTH_API_ID + sizeof(connection_t) \
-    + sizeof(module_t) + sizeof(nufw_session_t) + sizeof(struct nuauth_datas) \
-    + sizeof(tracking_t) + sizeof(user_session_t) ))
+	+ sizeof(module_t) + sizeof(nufw_session_t) + sizeof(struct nuauth_datas) \
+	+ sizeof(tracking_t) + sizeof(user_session_t) ))
 
 /**
  * Nuauth full version, eg. "nuauth 2.1.2 (Revision: 2730)"
  */
-#define NUAUTH_FULL_VERSION (VERSION " ($Revision: 4056 $)")
+#define NUAUTH_FULL_VERSION (VERSION " ($Revision: 5020 $)")
 
 void nuauth_ask_exit();
 void stop_all_thread_pools(gboolean wait);
 void block_thread_pools();
+void wait_all_thread_pools();
 void release_thread_pools();
 void start_all_thread_pools();
 void stop_thread_pool(const char *name, GThreadPool **pool);
@@ -235,4 +240,12 @@ void nuauth_install_signals(gboolean action);
 
 int nuauth_bind(char **errmsg, const char *addr, const char *port, char *context);
 
+/*----------------------- Kerberos stuff ----------------------------------*/
+
+/**
+ * Default value for "nuauth_krb5_service" option:
+ * Service name to use for Kerberos 5 authentication
+ */
+#define DEFAULT_KRB5_SERVICE "nuauth"
+
 #endif
