@@ -2,7 +2,7 @@
  ** Copyright(C) 2004-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: authsrv.c 4601 2008-04-02 20:22:38Z gryzor $
+ ** $Id: authsrv.c 5470 2008-12-31 11:13:39Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -71,6 +71,38 @@ void cleanup_func_remove(cleanup_func_t func)
 	cleanup_func_list = g_list_remove(cleanup_func_list, func);
 }
 
+/*
+ * wait one thread pool
+ */
+void wait_thread_pool(const char *name, GThreadPool *pool)
+{
+	gint count = 1;
+	while (count) {
+		count = g_thread_pool_unprocessed(pool);
+		usleep(10000);
+	}
+	log_message(DEBUG, DEBUG_AREA_MAIN,
+			"thread pool '%s' free", name);
+}
+
+void wait_all_thread_pools()
+{
+	wait_thread_pool("session logger", nuauthdatas->user_session_loggers);
+	wait_thread_pool("packet logger", nuauthdatas->user_loggers);
+	wait_thread_pool("acl checker", nuauthdatas->acl_checkers);
+	wait_thread_pool("users checker", nuauthdatas->user_checkers);
+
+	if (nuauthconf->log_users_sync) {
+		wait_thread_pool("decision worker",
+				nuauthdatas->decisions_workers);
+	}
+
+	if (nuauthconf->do_ip_authentication) {
+		wait_thread_pool("ip auth worker",
+				nuauthdatas->ip_authentication_workers);
+	}
+}
+
 void block_thread_pools()
 {
 	nuauthdatas->need_reload = 1;
@@ -158,6 +190,7 @@ void stop_all_thread_pools(gboolean soft)
 		stop_thread_pool("session logger", &nuauthdatas->user_session_loggers);
 		stop_thread_pool("packet logger", &nuauthdatas->user_loggers);
 		stop_thread_pool("acl checker", &nuauthdatas->acl_checkers);
+		stop_thread_pool("users checker", &nuauthdatas->user_checkers);
 
 		if (nuauthconf->log_users_sync) {
 			stop_thread_pool("decision worker",
@@ -406,8 +439,8 @@ void daemonize()
 
 	pidf = fork();
 	if (pidf < 0) {
-		g_error("Unable to fork");
-		exit(EXIT_FAILURE);	/* this should be useless !! */
+		g_warning("Unable to fork");
+		exit(EXIT_FAILURE);
 	} else {
 		if (pidf > 0) {
 			/* child process */
@@ -692,8 +725,7 @@ void configure_app(int argc, char **argv)
 		    nuauthconf->debug_level);
 
 	/* init credential */
-	if(! create_x509_credentials())
-	{
+	if (! create_x509_credentials()) {
 		fprintf(stderr,"Certificate initialization failed\n");
 		exit(EXIT_FAILURE);
 	}
@@ -702,8 +734,10 @@ void configure_app(int argc, char **argv)
 		daemonize();
 	} else {
 		g_message("[+] Starting nuauth " VERSION
-			  " ($Revision: 4601 $) with config "
+			  " ($Revision: 5470 $) with config "
 			  DEFAULT_CONF_FILE );
+		g_message("Nuauth started in foreground (without -D option)"
+			  ", logging on stderr and stdout only (no syslog)");
 	}
 }
 
@@ -755,7 +789,7 @@ void init_nuauthdatas()
 
 	/* init and load modules */
 	init_modules_system();
-	if(!load_modules())
+	if (!load_modules())
 		exit(EXIT_FAILURE);
 
 	/* init periods */
@@ -856,6 +890,8 @@ void main_cleanup()
 		}
 	}
 
+	act_on_loggers_processing();
+
 	if (nuauthconf->acl_cache) {
 		/* send refresh message to acl cache thread */
 		cmessage = g_new0(struct cache_message, 1);
