@@ -2,7 +2,7 @@
  ** Copyright(C) 2003-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: conffile.c 2860 2007-03-02 11:54:46Z haypo $
+ ** $Id: conffile.c 5470 2008-12-31 11:13:39Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ int parse_conffile(const char *filename, gint array_size,
 	scanner = g_scanner_new(NULL);
 	fd = open(filename, O_RDONLY);
 	if (fd == -1) {
-		g_error("Can not open config file : %s", filename);
+		g_warning("Can not open config file : %s", filename);
 		exit(EXIT_FAILURE);
 	}
 	g_scanner_input_file(scanner, fd);
