@@ -1,8 +1,8 @@
 /*
-** Copyrigh 2002-2004 Vincent Deffontaines <vincent@gryzor.com>
+** Copyright 2002-2004 Vincent Deffontaines <vincent@gryzor.com>
 ** INL http://www.inl.fr/
 **
-** $Id: debug.c 3793 2007-10-08 16:48:27Z regit $
+** $Id: debug.c 5467 2008-12-30 16:31:39Z regit $
 **
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -30,12 +30,12 @@ void process_g_syslog(const gchar * log_domain, GLogLevelFlags log_level,
 {
 	int syslog_level;
 	syslog_level = rint(log((double)log_level) / log(2.0));
-	syslog(LOG_FACILITY || syslog_level, message);
+	syslog(LOG_FACILITY | syslog_level, message);
 }
 
 void set_glib_loghandlers()
 {
-	openlog("nuauth", LOG_CONS || LOG_PID, LOG_DAEMON);
+	openlog("nuauth", LOG_CONS | LOG_PID, LOG_DAEMON);
 	g_log_set_handler(NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
 			  | G_LOG_FLAG_RECURSION, process_g_syslog, NULL);
 }
