@@ -2,7 +2,7 @@
  ** Copyright(C) 2005 Eric Leblond <regit@inl.fr>
  ** INL : http://www.inl.fr/
  **
- ** $Id: localid_auth.c 4125 2007-12-18 13:19:03Z haypo $
+ ** $Id: localid_auth.c 5229 2008-11-19 14:56:37Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -93,16 +93,14 @@ void localid_insert_message(connection_t * pckt,
 				thread_pool_push(nuauthdatas->
 						   acl_checkers, element,
 						   NULL);
-				/* remove element from hash without destroy */
-				g_hash_table_steal(localid_auth_hash,
-						   GINT_TO_POINTER(pckt->
-								   packet_id));
 			} else {
 				log_message(WARNING, DEBUG_AREA_USER,
 					    "Looks like a spoofing attempt from %s.",
 					    pckt->username);
-				/* TODO : kill bad guy */
 			}
+			/* remove element from hash without destroy */
+			g_hash_table_steal(localid_auth_hash,
+					   GINT_TO_POINTER(pckt->packet_id));
 			pckt->user_groups = NULL;
 			pckt->username = NULL;
 			/* free pckt */
@@ -150,9 +148,6 @@ void *localid_auth(GMutex * mutex)
 	long current_timestamp;
 	GTimeVal tv;
 
-/** \todo
- * protocol v3 compatibility */
-
 	global_msg.msg = (struct nu_srv_message *) msg;
 	msg->type = SRV_REQUIRED_HELLO;
 	msg->option = 0;
@@ -166,9 +161,9 @@ void *localid_auth(GMutex * mutex)
 	while (g_mutex_trylock(mutex)) {
 		g_mutex_unlock(mutex);
 
-		/* wait a message during 1000ms */
+		/* wait a message during POP_DELAY */
 		g_get_current_time(&tv);
-		g_time_val_add(&tv, 1000);
+		g_time_val_add(&tv, POP_DELAY);
 		message =
 		    g_async_queue_timed_pop(nuauthdatas->
 					    localid_auth_queue, &tv);
