@@ -2,7 +2,7 @@
  ** Copyright(C) 2005-2008 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **
- ** $Id: modules.c 4601 2008-04-02 20:22:38Z gryzor $
+ ** $Id: modules.c 5470 2008-12-31 11:13:39Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -195,20 +195,30 @@ gchar *modules_ip_auth(tracking_t * header)
 /**
  * log authenticated packets
  */
-int modules_user_logs(void *element, tcp_state_t state)
+nu_error_t modules_user_logs(void *element, tcp_state_t state)
 {
 	/* iter through all modules list */
 	GSList *walker = hooks[MOD_LOG_PACKETS].modules;
+	nu_error_t ret = NU_EXIT_OK;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
 		user_logs_callback *handler =
 		    (user_logs_callback *) ((module_t *) walker->data)->
 		    func;
-		handler(element, state,
-			((module_t *) walker->data)->params);
+		if (handler(element, state,
+			((module_t *) walker->data)->params) == -1) {
+			ret = NU_EXIT_ERROR;
+			/* A module has failed, this packet will be dropped if
+			 * drop_if_no_logging is set */
+			if (nuauthconf->drop_if_no_logging) {
+				((connection_t *)element)->decision = DECISION_DROP;
+				/* stop iterating over modules (nuauth is in DOS mode there) */
+				return ret;
+			}
+		}
 	}
-	return 0;
+	return ret;
 }
 
 /**
@@ -313,13 +323,17 @@ int modules_user_session_modify(user_session_t * c_session)
 {
 	/* iter through all modules list */
 	GSList *walker = hooks[MOD_SESSION_MODIFY].modules;
+	int ret;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
 		user_session_modify_callback *handler =
 		    (user_session_modify_callback
 		     *) (((module_t *) walker->data)->func);
-		handler(c_session, ((module_t *) walker->data)->params);
+		ret = handler(c_session, ((module_t *) walker->data)->params);
+		if (ret != SASL_OK) {
+			return ret;
+		}
 	}
 
 	return SASL_OK;
@@ -454,13 +468,20 @@ int check_module_version(GModule * module)
 static int load_modules_from(gchar * confvar, gchar * func,
 			     GSList ** target, module_hook_t hook)
 {
-	gchar **modules_list = g_strsplit(confvar, " ", 0);
+	gchar **modules_list;
 	gchar *module_path;
 	init_module_from_conf_t *initmod;
 	gchar **params_list;
 	module_t *current_module;
 	int i;
 
+	if (confvar == NULL)
+		return 1;
+
+	modules_list = g_strsplit(confvar, " ", 0);
+	if (modules_list == NULL)
+		return 1;
+
 	for (i = 0; modules_list[i] != NULL; i++) {
 		current_module = g_new0(module_t, 1);
 
@@ -501,8 +522,9 @@ static int load_modules_from(gchar * confvar, gchar * func,
 			    current_module->module_name,
 			    current_module->configfile);
 		if (current_module->module == NULL) {
-			g_error("Unable to load module %s in %s",
-				modules_list[i], MODULE_PATH);
+			g_warning("Unable to load module %s in %s\nError: %s",
+				  modules_list[i], MODULE_PATH,
+				  g_module_error ());
 			free_module_t(current_module);
 			return 0;
 		}
@@ -517,8 +539,8 @@ static int load_modules_from(gchar * confvar, gchar * func,
 		if (!g_module_symbol
 		    (current_module->module, func,
 		     (gpointer *) & current_module->func)) {
-			g_error("Unable to load function %s in %s", func,
-				g_module_name(current_module->module));
+			g_warning("Unable to load function %s in %s", func,
+				  g_module_name(current_module->module));
 			free_module_t(current_module);
 			g_strfreev(params_list);
 			return 0;
@@ -588,7 +610,7 @@ int load_modules()
 		{"nuauth_ip_authentication_module", G_TOKEN_STRING, 1,
 		 g_strdup(DEFAULT_IPAUTH_MODULE)},
 		{"nuauth_certificate_check_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_CERTIFICATE_CHECK_MODULE)},
+		 NULL},
 		{"nuauth_certificate_to_uid_module", G_TOKEN_STRING, 1,
 		 g_strdup(DEFAULT_CERTIFICATE_TO_UID_MODULE)},
 		{"nuauth_user_session_modify_module", G_TOKEN_STRING, 1,
@@ -602,16 +624,15 @@ int load_modules()
 	char *configfile = DEFAULT_CONF_FILE;
 
 	/* parse conf file */
-	if(!parse_conffile(configfile,
+	if (!parse_conffile(configfile,
 		       sizeof(nuauth_vars) / sizeof(confparams_t),
-		       nuauth_vars))
-	{
+		       nuauth_vars)) {
 	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
 		return 0;
 	}
 
 #define READ_CONF(KEY) \
-    get_confvar_value(nuauth_vars, sizeof(nuauth_vars)/sizeof(confparams_t), KEY);
+	get_confvar_value(nuauth_vars, sizeof(nuauth_vars)/sizeof(confparams_t), KEY);
 
 	hooks[MOD_USER_CHECK].config =
 		(char *) READ_CONF(hooks[MOD_USER_CHECK].configstring);
@@ -631,7 +652,7 @@ int load_modules()
 
 #undef READ_CONF
 #define READ_CONF(KEY) \
-    get_confvar_value(deps_check_vars, sizeof(deps_check_vars)/sizeof(confparams_t), KEY);
+	get_confvar_value(deps_check_vars, sizeof(deps_check_vars)/sizeof(confparams_t), KEY);
 	if (hooks[MOD_USER_CHECK].config) {
 		confparams_t deps_check_vars[] = {
 			{"nuauth_get_user_groups_module", G_TOKEN_STRING, 1,
@@ -641,10 +662,9 @@ int load_modules()
 		};
 
 		/* parse conf file for user_check sub vars*/
-		if(!parse_conffile(configfile,
+		if (!parse_conffile(configfile,
 				sizeof(deps_check_vars) / sizeof(confparams_t),
-				deps_check_vars))
-		{
+				deps_check_vars)) {
 		        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
 			return 0;
 		}
@@ -662,12 +682,12 @@ int load_modules()
 	g_mutex_lock(modules_mutex);
 
 #define LOAD_MODULE(HOOK) \
-    log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN, "Loading %s modules:", hooks[HOOK].message); \
-    if(!load_modules_from(hooks[HOOK].config, hooks[HOOK].funcstring, &(hooks[HOOK].modules), HOOK)) \
-    { \
-        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load modules %s", hooks[HOOK].message); \
-    	return 0; \
-    }
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN, "Loading %s modules:", hooks[HOOK].message); \
+	if (!load_modules_from(hooks[HOOK].config, hooks[HOOK].funcstring, &(hooks[HOOK].modules), HOOK)) \
+	{ \
+		log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load modules %s", hooks[HOOK].message); \
+		return 0; \
+	}
 
 
 	/* loading modules */
