@@ -2,7 +2,7 @@
  ** Copyright(C) 2005 Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: modules.h 3763 2007-09-25 22:03:22Z regit $
+ ** $Id: modules.h 5020 2008-10-28 13:28:35Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -131,7 +131,7 @@ GSList *modules_acl_check(connection_t * element);
 /* ip auth */
 gchar *modules_ip_auth(tracking_t * tracking);
 
-int modules_user_logs(void *element, tcp_state_t state);
+nu_error_t modules_user_logs(void *element, tcp_state_t state);
 int modules_user_session_logs(user_session_t * user,
 			      session_state_t state);
 
