@@ -3,7 +3,7 @@
  **     written by Eric Leblond <eric@regit.org>
  **                Vincent Deffontaines <vincent@gryzor.com>
  **
- ** $Id: ldap.c 4590 2008-03-27 17:17:46Z regit $
+ ** $Id: ldap.c 5479 2009-01-10 13:02:31Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
  ** along with this program; if not, write to the Free Software
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#define LDAP_DEPRECATED 1
 
 #include <auth_srv.h>
 #include <auth_ldap.h>
@@ -45,6 +44,7 @@ typedef unsigned long digit_t;
 #define BASE 1000000	 /** Use 6 decimal digits in each number digit */
 #define BASE_LOG10 6
 #define BASE2STR "%06lu"
+#define IPADDR_STRLEN 42
 #define DIGIT_COUNT 7	 /** BASE ^ DIGIT_COUNT should be able to store 2 ^ 128 */
 #define INIT_NUMBER {0, 0, 0, 0, 0, 0, 0}
 #if ULONG_MAX < (BASE*256)
@@ -111,13 +111,15 @@ int number_add(number_t number, digit_t value)
 
 /**
  * Convert a "Base 10^n" number to decimal string.
- *
- * \return Returns new allocated string
+ * \param number a number
+ * \param ipaddr pre-allocated string that whill contain the stringified IP
+ * 
+ * \return NU_EXIT_ERROR on error, NU_EXIT_OK on success
  */
-char *number_to_decimal(number_t number)
+static nu_error_t number_to_decimal(number_t number, char *ipaddr)
 {
 	char ascii[DIGIT_COUNT * BASE_LOG10 + 1];
-	char *text;
+	gchar *text;
 	signed char index;
 	for (index = DIGIT_COUNT - 1; 0 <= index; index--) {
 		sprintf(ascii + (DIGIT_COUNT - index - 1) * BASE_LOG10,
@@ -126,49 +128,33 @@ char *number_to_decimal(number_t number)
 	text = ascii;
 	while (text[0] == '0')
 		text++;
-	return strdup(text);
+
+	if (strlen(text) < IPADDR_STRLEN) {
+		memcpy(ipaddr, text, strlen(text));
+	} else {
+		return NU_EXIT_ERROR;
+	}
+	return NU_EXIT_OK;
 }
 
-/**
- * Convert a decimal string to a "Base 10^n" number.
- *
- * \return Returns 0 on error, 1 otherwise
- */
-int decimal_to_number(const char *orig_decimal, number_t number)
+#if SASL_VERSION_FULL > 0x020116
+static void ldap_conn_destroy(void * connection)
 {
-	ssize_t decimal_len = strlen(orig_decimal);
-	char *decimal = g_strdup(orig_decimal);
-	char *err;
-	unsigned char index;
-	for (index = 0; index < DIGIT_COUNT; index++)
-		number[index] = 0;
-	index = 0;
-	while (BASE_LOG10 < decimal_len) {
-		decimal[decimal_len] = 0;
-		decimal_len -= BASE_LOG10;
-		number[index] = strtol(decimal + decimal_len, &err, 10);
-		index++;
-		if (err == NULL || *err != 0 || DIGIT_COUNT <= index) {
-			g_free(decimal);
-			return 0;
-		}
+	if (connection) {
+		ldap_unbind_ext_s(connection, NULL, NULL);
 	}
-	decimal[decimal_len] = 0;
-	number[index] = strtol(decimal, &err, 10);
-	g_free(decimal);
-	if (err == NULL || *err != 0)
-		return 0;
-	else
-		return 1;
 }
+#endif
+
+
 
 G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 {
 	struct ldap_params *params = (struct ldap_params *) params_p;
 	if (params) {
+		g_free(params->ldap_server);
 		g_free(params->binddn);
 		g_free(params->bindpasswd);
-		g_free(params->ldap_server);
 		g_free(params->ldap_acls_base_dn);
 		g_free(params->ldap_users_base_dn);
 	}
@@ -184,19 +170,15 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	char *configfile = NULL;
 	gpointer vpointer;
 	struct ldap_params *params = g_new0(struct ldap_params, 1);
-	char *ldap_base_dn = LDAP_BASE;
+	gchar *ldap_base_dn = g_strdup(LDAP_BASE);
 	confparams_t ldap_nuauth_vars[] = {
-		{"ldap_server_addr", G_TOKEN_STRING, 0,
-		 g_strdup(LDAP_SERVER)},
+		{"ldap_server_addr", G_TOKEN_STRING, 0, g_strdup(LDAP_SERVER)},
 		{"ldap_server_port", G_TOKEN_INT, LDAP_SERVER_PORT, NULL},
 		{"ldap_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
-		{"ldap_users_base_dn", G_TOKEN_STRING, 0,
-		 g_strdup(LDAP_BASE)},
-		{"ldap_acls_base_dn", G_TOKEN_STRING, 0,
-		 g_strdup(LDAP_BASE)},
+		{"ldap_users_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
+		{"ldap_acls_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
 		{"ldap_bind_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_USER)},
-		{"ldap_bind_password", G_TOKEN_STRING, 0,
-		 g_strdup(LDAP_CRED)},
+		{"ldap_bind_password", G_TOKEN_STRING, 0, g_strdup(LDAP_CRED)},
 		{"ldap_request_timeout", G_TOKEN_INT, LDAP_REQUEST_TIMEOUT,
 		 NULL},
 		{"ldap_use_ipv4_schema", G_TOKEN_INT, 1, NULL},
@@ -205,7 +187,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Ldap module ($Revision: 4590 $)");
+		    "Ldap module ($Revision: 5479 $)");
 	if (!module->configfile) {
 		configfile = DEFAULT_CONF_FILE;
 	} else {
@@ -254,10 +236,10 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	    (char *) (vpointer ? vpointer : params->ldap_acls_base_dn);
 
 	if (!strcmp(params->ldap_acls_base_dn, LDAP_BASE)) {
-		params->ldap_acls_base_dn = ldap_base_dn;
+		params->ldap_acls_base_dn = g_strdup(ldap_base_dn);
 	}
 	if (!strcmp(params->ldap_users_base_dn, LDAP_BASE)) {
-		params->ldap_users_base_dn = ldap_base_dn;
+		params->ldap_users_base_dn = g_strdup(ldap_base_dn);
 	}
 
 	vpointer =
@@ -290,12 +272,22 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	/* free config struct */
+
+	g_free(ldap_base_dn);
+
 	free_confparams(ldap_nuauth_vars,
 			sizeof(ldap_nuauth_vars) / sizeof(confparams_t));
 
 
 	/* init thread private stuff */
-	params->ldap_priv = g_private_new((GDestroyNotify) ldap_unbind);
+	/* cyrus-sasl is buggy in version <= to 2.1.22
+	* See http://archives.free.net.ph/message/20080920.124141.b3d1fc95.fr.html
+	*/
+#if SASL_VERSION_FULL > 0x020116
+	params->ldap_priv = g_private_new((GDestroyNotify) ldap_conn_destroy);
+#else
+	params->ldap_priv = g_private_new(NULL);
+#endif
 
 	module->params = params;
 
@@ -318,9 +310,17 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 {
 	LDAP *ld = NULL;
 	int err, version = 3;
+	char uri[1024];
+	struct berval password;
 
 	/* init connection */
-	ld = ldap_init(params->ldap_server, params->ldap_server_port);
+	if ( ! secure_snprintf(uri, 1024, "%s://%s:%u",
+		(params->ldap_server_port == LDAPS_PORT) ? "ldaps" : "ldap",
+		params->ldap_server, params->ldap_server_port) ) {
+		log_message(WARNING, DEBUG_AREA_MAIN, "LDAP: could not build URI");
+		return NULL;
+	}
+	ldap_initialize(&ld, uri);
 	if (!ld) {
 		log_message(WARNING, DEBUG_AREA_MAIN, "Ldap init error");
 		return NULL;
@@ -344,16 +344,17 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 		}
 #endif /* LDAP_OPT_X_TLS */
 
-		err =
-		    ldap_bind_s(ld, params->binddn, params->bindpasswd,
-				LDAP_AUTH_SIMPLE);
+		password.bv_val = params->bindpasswd;
+		password.bv_len = strlen(password.bv_val);
+		err = ldap_sasl_bind_s(ld, params->binddn, LDAP_SASL_SIMPLE,
+				&password, NULL, NULL, NULL);
 		if (err != LDAP_SUCCESS) {
 			if (err == LDAP_SERVER_DOWN) {
 				log_message(INFO, DEBUG_AREA_AUTH,
 					    "Can not connect to ldap: %s",
 					    ldap_err2string(err));
 				/* we lost connection, so disable current one */
-				ldap_unbind(ld);
+				ldap_unbind_ext_s(ld, NULL, NULL);
 				ld = NULL;
 				g_private_set(params->ldap_priv, ld);
 				return NULL;
@@ -367,81 +368,18 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 	return ld;
 }
 
-static char *ipv6_to_base10(struct in6_addr *addr)
+static nu_error_t ipv6_to_base10(struct in6_addr *addr, gchar *ipaddr)
 {
 	number_t number = INIT_NUMBER;
 	unsigned char index = 0;
 
 	for (index = 0; index < 16; index++) {
 		if (number_add(number, addr->s6_addr[index]) != 1)
-			return NULL;
+			return NU_EXIT_ERROR;
 		number_multiply(number, 256);
 	}
 
-	return number_to_decimal(number);
-}
-
-
-/**
- * \brief Escape character to protect them in query
- *
- * \verbatim
-Abstract from RFC 2254
-	Character       ASCII value
-	---------------------------
-	 *               0x2a
-	 (               0x28
-	 )               0x29
-	 \               0x5c
-	 NUL             0x00
-For example * is coded \2a
-\endverbatim
- *
- * \param basestring the string to convert
- * \return a newly allocated string
- */
-
-gchar *escape_string_for_ldap(const gchar * basestring)
-{
-	int length = strlen(basestring) + 1;
-	gchar *result = g_new0(gchar, length);
-	const gchar *c_char = basestring;
-	int i = 0;
-
-	while (*c_char) {
-		switch (*c_char) {
-		case '*':
-			length += 2;
-			result = g_realloc(result, length);
-			g_strlcat(result, "\\2a", length);
-			i += 3;
-			break;
-		case '(':
-			length += 2;
-			result = g_realloc(result, length);
-			g_strlcat(result, "\\28", length);
-			i += 3;
-			break;
-		case ')':
-			length += 2;
-			result = g_realloc(result, length);
-			g_strlcat(result, "\\29", length);
-			i += 3;
-			break;
-		case '\\':
-			length += 2;
-			result = g_realloc(result, length);
-			g_strlcat(result, "\\5c", length);
-			i += 3;
-			break;
-		default:
-			result[i] = *c_char;
-			i++;
-		}
-		c_char++;
-	}
-	result[length - 1] = 0;
-	return result;
+	return number_to_decimal(number, ipaddr);
 }
 
 struct weighted_acl {
@@ -460,6 +398,37 @@ static void local_free(gpointer data, gpointer userdata)
 	g_free(data);
 }
 
+/**
+ * \return A nu_error_t::, NU_EXIT_CONTINUE if filter did not match, NU_EXIT_OK if filter did match.
+ */
+
+static nu_error_t field_match_pattern(gchar * value, LDAP *ld, LDAPMessage *result, gchar *attribute)
+{
+	nu_error_t ret = NU_EXIT_CONTINUE;
+	struct berval **attrs_array;
+	struct berval **pattrs_array;
+
+	attrs_array = ldap_get_values_len(ld, result, attribute);
+	if (attrs_array && *attrs_array) {
+		pattrs_array = attrs_array;
+		while (*pattrs_array) {
+			if (g_pattern_match_simple(
+						(*pattrs_array)->bv_val,
+						value
+						)) {
+				ret = NU_EXIT_OK;
+				break;
+			}
+			pattrs_array++;
+		}
+	} else {
+		/* No attributes in LDAP, thus criteria filtering is a success */
+		ret = NU_EXIT_OK;
+	}
+	ldap_value_free_len(attrs_array);
+	return ret;
+}
+
 /**
  * \brief Acl check function
  *
@@ -480,18 +449,18 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 	GSList *g_acl_list = NULL;
 	GSList *temp_list = NULL;
 	char filter[LDAP_QUERY_SIZE];
-	char **attrs_array, **walker;
+	struct berval **attrs_array, **walker;
 	int attrs_array_len, i, integer;
 	struct timeval timeout;
 	struct acl_group *this_acl;
 	struct weighted_acl *this = NULL;
-	LDAPMessage *res, *result;
+	LDAPMessage *res = NULL;
+	LDAPMessage *result;
 	int err, try;
 	struct ldap_params *params = (struct ldap_params *) params_p;
 	LDAP *ld = g_private_get(params->ldap_priv);
-	gchar *ip_src;
-	gchar *ip_dst;
-	gchar *prov_string;
+	gchar ip_src[IPADDR_STRLEN];
+	gchar ip_dst[IPADDR_STRLEN];
 
 	if (params->ldap_use_ipv4_schema) {
 		if (!is_ipv4(&element->tracking.saddr) ||
@@ -500,20 +469,19 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 				    "ldap: IPv4 schema but IPv6 address\n");
 			return NULL;
 		}
-		ip_src = g_new0(char,11);
-		snprintf(ip_src, 11, "%u", 
+		snprintf(ip_src, IPADDR_STRLEN, "%u",
 				ntohl(element->tracking.saddr.s6_addr32[3]));
-		ip_dst = g_new0(char,11);
-		snprintf(ip_dst, 11, "%u", 
+		snprintf(ip_dst, IPADDR_STRLEN, "%u",
 				ntohl(element->tracking.daddr.s6_addr32[3]));
 	} else {
-		ip_src = ipv6_to_base10(&element->tracking.saddr);
-		ip_dst = ipv6_to_base10(&element->tracking.daddr);
-	}
-	if (ip_src == NULL || ip_dst == NULL) {
-		g_free(ip_src);
-		g_free(ip_dst);
-		return NULL;
+		if (ipv6_to_base10(&element->tracking.saddr, ip_src) ==
+				NU_EXIT_ERROR) {
+			return NULL;
+		}
+		if (ipv6_to_base10(&element->tracking.daddr, ip_dst) ==
+				NU_EXIT_ERROR) {
+			return NULL;
+		}
 	}
 
 	/* contruct filter */
@@ -537,8 +505,6 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 				log_message(WARNING, DEBUG_AREA_MAIN,
 					    "LDAP query too big (more than %d bytes)\n",
 					    LDAP_QUERY_SIZE);
-				g_free(ip_src);
-				g_free(ip_dst);
 				return NULL;
 			}
 			break;
@@ -562,74 +528,59 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 				log_message(WARNING, DEBUG_AREA_MAIN,
 					    "LDAP query too big (more than %d bytes)\n",
 					    LDAP_QUERY_SIZE);
-				g_free(ip_src);
-				g_free(ip_dst);
 				return NULL;
 			}
 		}
-		g_free(ip_src);
-		g_free(ip_dst);
 
 		/* finish filter */
-		if (element->os_sysname) {
-			g_strlcat(filter, "(|(OsName=",
-				  LDAP_QUERY_SIZE);
-			prov_string =
-			    escape_string_for_ldap(element->os_sysname);
-			g_strlcat(filter, prov_string, LDAP_QUERY_SIZE);
-			g_free(prov_string);
-			g_strlcat(filter, ")(!(OsName=*)))",
-				  LDAP_QUERY_SIZE);
-		} else {
+		if (! element->os_sysname) {
 			g_strlcat(filter, "(!(OsName=*))",
 				  LDAP_QUERY_SIZE);
 		}
-		if (! element->app_name) {
-			g_strlcat(filter, "(!(AppName=*))",
-				  LDAP_QUERY_SIZE);
-		}
-		if (element->os_release) {
-			g_strlcat(filter, "(|(OsRelease=",
-				  LDAP_QUERY_SIZE);
-			prov_string =
-			    escape_string_for_ldap(element->os_release);
-			g_strlcat(filter, prov_string, LDAP_QUERY_SIZE);
-			g_free(prov_string);
-			g_strlcat(filter, ")(!(OsRelease=*)))",
-				  LDAP_QUERY_SIZE);
-		} else {
+		if (! element->os_release) {
 			g_strlcat(filter, "(!(OsRelease=*))",
 				  LDAP_QUERY_SIZE);
 		}
-		if (element->os_version) {
-			g_strlcat(filter, "(|(OsVersion=",
-				  LDAP_QUERY_SIZE);
-			prov_string =
-			    escape_string_for_ldap(element->os_version);
-			g_strlcat(filter, prov_string, LDAP_QUERY_SIZE);
-			g_free(prov_string);
-			g_strlcat(filter, ")(!(OsVersion=*)))",
-				  LDAP_QUERY_SIZE);
-		} else {
+		if (! element->os_version) {
 			g_strlcat(filter, "(!(OsVersion=*))",
 				  LDAP_QUERY_SIZE);
 		}
-		if (element->app_md5) {
-			g_strlcat(filter, "(|(AppSig=",
-				  LDAP_QUERY_SIZE);
-			prov_string =
-			    escape_string_for_ldap(element->app_md5);
-			g_strlcat(filter, prov_string, LDAP_QUERY_SIZE);
-			g_free(prov_string);
-			g_strlcat(filter, ")(!(AppSig=*)))",
-				  LDAP_QUERY_SIZE);
-		} else {
-			g_strlcat(filter, "(!(AppSig=*))",
-				  LDAP_QUERY_SIZE);
+		if (! element->app_name) {
+			g_strlcat(filter, "(!(AppName=*))", LDAP_QUERY_SIZE);
 		}
 
+		if (element->iface_nfo.indev[0] == '\0') {
+			g_strlcat(filter, "(!(InDev=*))", LDAP_QUERY_SIZE);
+		} else {
+			gchar iffilter[256];
+			g_snprintf(iffilter, 256, "(|(InDev=%s)(!(InDev=*)))", element->iface_nfo.indev);
+			g_strlcat(filter, iffilter, LDAP_QUERY_SIZE);
+		}
+		if (element->iface_nfo.outdev[0] == '\0') {
+			g_strlcat(filter, "(!(OutDev=*))", LDAP_QUERY_SIZE);
+		} else {
+			gchar iffilter[256];
+			g_snprintf(iffilter, 256, "(|(OutDev=%s)(!(OutDev=*)))", element->iface_nfo.outdev);
+			g_strlcat(filter, iffilter, LDAP_QUERY_SIZE);
+		}
+		if (element->iface_nfo.physindev[0] == '\0') {
+			g_strlcat(filter,  "(!(PhysInDev=*))", LDAP_QUERY_SIZE);
+		} else {
+			gchar iffilter[256];
+			g_snprintf(iffilter, 256, "(|(PhysInDev=%s)(!(PhysInDev=*)))", element->iface_nfo.physindev);
+			g_strlcat(filter, iffilter, LDAP_QUERY_SIZE);
+		}
+		if (element->iface_nfo.physoutdev[0] == '\0') {
+			g_strlcat(filter, "(!(PhysOutDev=*))", LDAP_QUERY_SIZE);
+		} else {
+			gchar iffilter[256];
+			g_snprintf(iffilter, 256, "(|(PhysOutDev=%s)(!(PhysOutDev=*)))",
+				   element->iface_nfo.physoutdev);
+			g_strlcat(filter, iffilter, LDAP_QUERY_SIZE);
+		}
 
 		g_strlcat(filter, ")", LDAP_QUERY_SIZE);
+
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				  "LDAP filter : \n%s\n", filter);
 
@@ -652,12 +603,8 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			log_message(WARNING, DEBUG_AREA_MAIN,
 				    "LDAP query too big (more than %d bytes)\n",
 				    LDAP_QUERY_SIZE);
-			g_free(ip_src);
-			g_free(ip_dst);
 			return NULL;
 		}
-		g_free(ip_src);
-		g_free(ip_dst);
 	}
 
 	try = 0;
@@ -669,7 +616,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			if (ld == NULL) {
 				log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
 						"Can not initiate LDAP conn\n");
-				break;
+				return NULL;
 			}
 			g_private_set(params->ldap_priv, ld);
 		}
@@ -685,10 +632,10 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			}
 #endif
 
-			err =
-				ldap_search_st(ld, params->ldap_acls_base_dn,
+			err = ldap_search_ext_s(ld, params->ldap_acls_base_dn,
 						LDAP_SCOPE_SUBTREE, filter, NULL, 0,
-						&timeout, &res);
+						NULL, NULL,
+						&timeout, LDAP_NO_LIMIT, &res);
 
 #ifdef PERF_DISPLAY_ENABLE
 			if (nuauthconf->debug_areas & DEBUG_AREA_PERF) {
@@ -700,18 +647,33 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 		}
 #endif
 		if (err != LDAP_SUCCESS) {
-			if (err == LDAP_SERVER_DOWN) {
-				/* we lost connection, so disable current one */
-				log_message(WARNING, DEBUG_AREA_MAIN,
-						"disabling current connection");
-				ldap_unbind(ld);
-				ld = NULL;
-				g_private_set(params->ldap_priv, ld);
-			} else {
-				break;
+			switch (err) {
+				case LDAP_SERVER_DOWN:
+					/* we lost connection, so disable current one */
+					log_message(WARNING, DEBUG_AREA_MAIN,
+							"disabling current connection");
+					ldap_unbind_ext_s(ld, NULL, NULL);
+					ld = NULL;
+					g_private_set(params->ldap_priv, ld);
+					break;
+				case LDAP_NO_SUCH_OBJECT:
+					debug_log_message(DEBUG, DEBUG_AREA_MAIN, "no ldap object");
+					ldap_msgfree(res);
+					return NULL;
+				default:
+					log_message(INFO, DEBUG_AREA_MAIN,
+						    "ldap search failed: %s",
+						    ldap_err2string(err));
+					if (res) {
+						debug_log_message(DEBUG, DEBUG_AREA_MAIN,
+								  "freeing allocated ldap result");
+						ldap_msgfree(res);
+						res = NULL;
+					}
+					break;
 			}
 		}
-	} while ((err != LDAP_SUCCESS) || (try < LDAP_MAX_TRY));
+	} while ((err != LDAP_SUCCESS) && (try < LDAP_MAX_TRY));
 
 	if ((try == LDAP_MAX_TRY) && (err != LDAP_SUCCESS)) {
 		log_message(WARNING, DEBUG_AREA_MAIN,
@@ -721,40 +683,59 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 	}
 	/* parse result to feed a group_list */
 	if (ldap_count_entries(ld, res) >= 1) {
-		result = ldap_first_entry(ld, res);
-		while (result) {
+		for(result=ldap_first_entry(ld, res); result; result=ldap_next_entry(ld, result)) {
 			gboolean break_loop = FALSE;
-			/* get period */
-			attrs_array =
-			    ldap_get_values(ld, result, "AppName");
-			if (attrs_array && *attrs_array) {
-				char **pattrs_array = attrs_array;
-				while (*pattrs_array) {
-					if (g_pattern_match_simple(
-								*attrs_array,
-								element->app_name
-								)) {
-						break_loop = FALSE;
-						break;
-					} else {
-						break_loop = TRUE;
-					}
-					pattrs_array++;
-				}
-			}
-			ldap_value_free(attrs_array);
 
-			if (break_loop) {
-				break;
+#define TEST_PATTERN(x, y) switch (field_match_pattern(element->x, ld, result, y)) { \
+				case NU_EXIT_OK: \
+					break; \
+				case NU_EXIT_CONTINUE: \
+					/* this is not a match, going to test next acl */ \
+					break_loop = TRUE; \
+					break; \
+				case NU_EXIT_ERROR: \
+					log_message(WARNING, DEBUG_AREA_MAIN, \
+						    "Invalid return from field_match_pattern"); \
+					ldap_msgfree(res); \
+					return NULL; \
+				default: \
+					log_message(WARNING, DEBUG_AREA_MAIN, \
+						    "Impossible return from field_match_pattern"); \
+					ldap_msgfree(res); \
+					return NULL; \
+					break; \
+			} \
+			if (break_loop) { \
+				continue; \
 			}
 
+			TEST_PATTERN(app_name, "AppName");
+			TEST_PATTERN(os_sysname, "OsName");
+			TEST_PATTERN(os_release, "OsRelease");
+			TEST_PATTERN(os_version, "OsVersion");
+
+#undef TEST_PATTERN
 
 			/* allocate a new acl_group */
 			this_acl = g_new0(struct acl_group, 1);
+			if (this_acl == NULL) {
+				log_message(WARNING, DEBUG_AREA_MAIN,
+						"Unable to allocate new acl");
+				g_slist_free(g_list);
+				ldap_msgfree(res);
+				return NULL;
+			}
 			if (nuauthconf->prio_to_nok == 2) {
 				this = g_new0(struct weighted_acl, 1);
+				if (this == NULL) {
+					log_message(WARNING, DEBUG_AREA_MAIN,
+						    "Unable to allocate new weighted_acl");
+					g_free(this_acl);
+					g_slist_free(g_list);
+					ldap_msgfree(res);
+					return NULL;
+				}
 			}
-			g_assert(this_acl);
 			this_acl->users = NULL;
 			this_acl->groups = NULL;
 			this_acl->period = NULL;
@@ -762,82 +743,71 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			this_acl->flags = ACL_FLAGS_NONE;
 
 			/* get period */
-			attrs_array =
-			    ldap_get_values(ld, result, "TimeRange");
+			attrs_array = ldap_get_values_len(ld, result, "TimeRange");
 			if (attrs_array && *attrs_array) {
-				this_acl->period = g_strdup(*attrs_array);
+				this_acl->period = g_strdup((*attrs_array)->bv_val);
 			}
-			ldap_value_free(attrs_array);
+			ldap_value_free_len(attrs_array);
 
 			/* get description (log prefix) */
-			attrs_array =
-			    ldap_get_values(ld, result, "description");
+			attrs_array = ldap_get_values_len(ld, result, "description");
 			if (attrs_array && *attrs_array) {
-				this_acl->log_prefix =
-				    g_strdup(*attrs_array);
+				this_acl->log_prefix = g_strdup((*attrs_array)->bv_val);
 			}
-			ldap_value_free(attrs_array);
+			ldap_value_free_len(attrs_array);
 
 			/* get flags */
-			attrs_array =
-			    ldap_get_values(ld, result, "AclFlags");
+			attrs_array = ldap_get_values_len(ld, result, "AclFlags");
 			if (attrs_array && *attrs_array) {
-				sscanf(*attrs_array, "%d",
-						(int *) &(this_acl->flags));
+				sscanf((*attrs_array)->bv_val, "%d", (int *) &(this_acl->flags));
 			}
-			ldap_value_free(attrs_array);
+			ldap_value_free_len(attrs_array);
 
 			if (nuauthconf->prio_to_nok == 2) {
 				/* get weight */
-				attrs_array =
-					ldap_get_values(ld, result, "AclWeight");
+				attrs_array = ldap_get_values_len(ld, result, "AclWeight");
 				if (attrs_array && *attrs_array) {
-					sscanf(*attrs_array, "%d",
-							(int *) &(this->weight));
+					sscanf((*attrs_array)->bv_val, "%d", (int *) &(this->weight));
 				} else {
 					this->weight = 0;
 				}
-				ldap_value_free(attrs_array);
+				ldap_value_free_len(attrs_array);
 			}
 
 			/* get decision */
-			attrs_array =
-			    ldap_get_values(ld, result, "Decision");
-			sscanf(*attrs_array, "%d",
-			       (int *) &(this_acl->answer));
+			attrs_array = ldap_get_values_len(ld, result, "Decision");
+			sscanf((*attrs_array)->bv_val, "%d", (int *) &(this_acl->answer));
 			debug_log_message(DEBUG, DEBUG_AREA_AUTH,
 					  "Acl found with decision %d (timerange: %s)\n",
 					  this_acl->answer,
 					  this_acl->period);
-			ldap_value_free(attrs_array);
+			ldap_value_free_len(attrs_array);
 			/* build groups list */
-			attrs_array = ldap_get_values(ld, result, "Group");
-			attrs_array_len = ldap_count_values(attrs_array);
+			attrs_array = ldap_get_values_len(ld, result, "Group");
+			attrs_array_len = ldap_count_values_len(attrs_array);
 			walker = attrs_array;
 			for (i = 0; i < attrs_array_len; i++) {
-				sscanf(*walker, "%d", &integer);
+				sscanf((*walker)->bv_val, "%d", &integer);
 				this_acl->groups =
 				    g_slist_prepend(this_acl->groups,
 						    GINT_TO_POINTER
 						    (integer));
 				walker++;
 			}
-			ldap_value_free(attrs_array);
+			ldap_value_free_len(attrs_array);
 			/* build users  list */
-			attrs_array = ldap_get_values(ld, result, "User");
-			attrs_array_len = ldap_count_values(attrs_array);
+			attrs_array = ldap_get_values_len(ld, result, "User");
+			attrs_array_len = ldap_count_values_len(attrs_array);
 			walker = attrs_array;
 			for (i = 0; i < attrs_array_len; i++) {
-				sscanf(*walker, "%d", &integer);
+				sscanf((*walker)->bv_val, "%d", &integer);
 				this_acl->users =
 				    g_slist_prepend(this_acl->users,
 						    GINT_TO_POINTER
 						    (integer));
 				walker++;
 			}
-			ldap_value_free(attrs_array);
-
-			result = ldap_next_entry(ld, result);
+			ldap_value_free_len(attrs_array);
 
 			if (nuauthconf->prio_to_nok == 2) {
 				this->acl = this_acl;
@@ -853,6 +823,12 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 					g_list = g_slist_prepend(g_list, this_acl);
 				}
 			} else {
+				g_free(this_acl->period);
+				g_free(this_acl->log_prefix);
+				if (this_acl->users)
+					g_slist_free(this_acl->users);
+				if (this_acl->groups)
+					g_slist_free(this_acl->groups);
 				g_free(this_acl);
 				if (nuauthconf->prio_to_nok == 2) {
 					g_free(this);
