@@ -4,7 +4,7 @@
  **	       Vincent Deffontaines <vincent@gryzor.com>
  **	       Victor Stinner <haypo@inl.fr>
  **
- ** $Id: mysql.c 4621 2008-04-05 07:45:24Z regit $
+ ** $Id: mysql.c 5230 2008-11-19 15:00:25Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -248,7 +248,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	    g_new0(struct log_mysql_params, 1);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_mysql module ($Revision: 4621 $)");
+		    "Log_mysql module ($Revision: 5230 $)");
 	/* init global variables */
 	params->mysql_ssl_cipher = MYSQL_SSL_CIPHER;
 	params->hook = module->hook;
@@ -308,7 +308,6 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 			    "Resetting mysql_admin_bofh to 0 because multiple logins are allowed");
 			params->mysql_admin_bofh = 0;
 		}
-		/** \todo Reset mysql_admin_bofh if we more than one nufw */
 		log_message(WARNING, DEBUG_AREA_MAIN,
 			    "mysql_admin_bofh will not work properly if you have multiple nufw");
 	}
@@ -626,8 +625,9 @@ static inline int log_state_open(MYSQL * ld, connection_t * element,
 	char *request;
 	int mysql_ret;
 
-	if (element->tracking.protocol == IPPROTO_TCP
-	    && nuauthconf->log_users_strict) {
+	if (element->tracking.protocol == IPPROTO_TCP &&
+		(nuauthconf->log_users_strict ||
+		 (element->flags & ACL_FLAGS_STRICT))) {
 		gboolean ok;
 		char request[SHORT_REQUEST_SIZE];
 		char src_ascii[IPV6_SQL_STRLEN];
@@ -756,7 +756,6 @@ static inline int log_state_established(MYSQL * ld,
 	return 0;
 }
 
-/** \todo Dump accounting counters in the table */
 static inline int log_state_close(MYSQL * ld,
 				  struct accounted_connection *element,
 				  struct log_mysql_params *params)
@@ -1056,7 +1055,8 @@ nu_error_t destroy_user_connections(user_session_t * c_session,
 							  &msgdatas,
 							  params)
 					!= NU_EXIT_OK) {
-				/** \todo log error treatment */
+				log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
+					    "Unable to build conntrack message");
 				mysql_free_result(result);
 				return NU_EXIT_ERROR;
 			}
@@ -1064,7 +1064,8 @@ nu_error_t destroy_user_connections(user_session_t * c_session,
 					(&msgdatas,
 					 AUTH_CONN_DESTROY)
 					!= NU_EXIT_OK) {
-				/** \todo log error treatment */
+				log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
+					    "Unable to send conntrack message");
 				mysql_free_result(result);
 				return NU_EXIT_ERROR;
 			}
