@@ -3,7 +3,7 @@
  **	written by Pierre Chifflier <chifflier AT inl DOT fr>
  **	written by Victor Stinner <victor.stinner AT haypocalc.com>
  **
- ** $Id: nuprelude.c 4607 2008-04-03 12:58:08Z pollux $
+ ** $Id: nuprelude.c 5220 2008-11-19 14:11:55Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_nuprelude module ($Revision: 4607 $)");
+		    "Log_nuprelude module ($Revision: 5220 $)");
 
 	params->packet_tpl = g_private_new((GDestroyNotify) destroy_idmef);
 	params->session_tpl = g_private_new((GDestroyNotify) destroy_idmef);
@@ -642,12 +642,12 @@ static idmef_message_t *create_message_autherr(idmef_message_t * tpl,
 	return idmef;
 }
 
-/** \todo Take into account connection_t* to void* change */
-G_MODULE_EXPORT gint user_packet_logs(connection_t * element,
+G_MODULE_EXPORT gint user_packet_logs(void *pelement,
 				      tcp_state_t state,
 				      gpointer params_ptr)
 {
 	struct log_prelude_params *params = params_ptr;
+	connection_t *element = (connection_t *)pelement;
 	idmef_message_t *tpl;
 	idmef_message_t *message;
 	char *impact;
