@@ -39,7 +39,7 @@
 #define APPNAME_MAX_SIZE 256
 
 #define SHORT_REQUEST_SIZE 400
-#define INSERT_REQUEST_FIEDLS_SIZE 200
+#define INSERT_REQUEST_FIELDS_SIZE 200
 #define INSERT_REQUEST_VALUES_SIZE 800
 
 struct log_pgsql_params {
