@@ -2,7 +2,7 @@
  ** Copyright(C) 2003-2006 Eric Leblond <eric@regit.org>
  **		     Vincent Deffontaines <vincent@gryzor.com>
  **                   INL
- ** $Id: pgsql.c 3654 2007-08-17 19:12:59Z regit $
+ ** $Id: pgsql.c 4967 2008-10-14 14:09:06Z pollux $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ static int formatINET(struct log_pgsql_params *params,
 	if (params->pgsql_use_ipv4) {
 		if (!is_ipv4(addr6)) {
 			log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-				    "MySQL: Packet has IPV6 address but MySQL use IPV4 only schema");
+				    "PostgreSQL: Packet has IPV6 address but PostgreSQL use IPV4 only schema");
 			return 0;
 		}
 		addr4.s_addr = addr6->s6_addr32[3];
@@ -185,7 +185,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	module->params = params;
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_pgsql module ($Revision: 3654 $)");
+		    "Log_pgsql module ($Revision: 4967 $)");
 	/* parse conf file */
 	if (module->configfile) {
 		parse_conffile(module->configfile, nb_params,
@@ -197,9 +197,9 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 	/* set variables */
 #define READ_CONF(KEY) \
-    get_confvar_value(pgsql_nuauth_vars, nb_params, KEY)
+	get_confvar_value(pgsql_nuauth_vars, nb_params, KEY)
 #define READ_CONF_INT(VAR, KEY, DEFAULT) \
-    do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT; } while (0)
+	do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT; } while (0)
 
 	params->pgsql_server = (char *) READ_CONF("pgsql_server_addr");
 	READ_CONF_INT(params->pgsql_server_port, "pgsql_server_port",
@@ -300,7 +300,7 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 			char *oob_prefix, tcp_state_t state,
 			struct log_pgsql_params *params)
 {
-	char request_fields[INSERT_REQUEST_FIEDLS_SIZE];
+	char request_fields[INSERT_REQUEST_FIELDS_SIZE];
 	char request_values[INSERT_REQUEST_VALUES_SIZE];
 	char tmp_buffer[INSERT_REQUEST_VALUES_SIZE];
 	char ip_src[INET6_ADDRSTRLEN];
@@ -360,7 +360,7 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 		else
 			quoted_appname = g_strdup("");
 
-		/* Quote strings send to MySQL */
+		/* Quote strings send to PostgreSQL */
 		g_strlcat(request_fields,
 			  ", user_id, username, client_os, client_app",
 			  sizeof(request_fields));
@@ -483,8 +483,8 @@ static int pgsql_update_state(PGconn * ld,
 {
 	char request[SHORT_REQUEST_SIZE];
 	PGresult *Result;
-	char tmp_inet1[INET_ADDRSTRLEN + 1];
-	char tmp_inet2[INET_ADDRSTRLEN + 1];
+	char tmp_inet1[INET6_ADDRSTRLEN + 1];
+	char tmp_inet2[INET6_ADDRSTRLEN + 1];
 	u_int16_t tcp_src, tcp_dst;
 	char *ip_src, *ip_dst;
 	int nb_try = 0;
@@ -614,8 +614,9 @@ G_MODULE_EXPORT gint user_packet_logs(void *element,
 
 	switch (state) {
 	case TCP_STATE_OPEN:
-		if (((connection_t *)element)->tracking.protocol == IPPROTO_TCP
-		    && nuauthconf->log_users_strict) {
+		if (((connection_t *)element)->tracking.protocol == IPPROTO_TCP &&
+			(nuauthconf->log_users_strict ||
+			 (((connection_t *)element)->flags & ACL_FLAGS_STRICT))) {
 			int ret = pgsql_update_close(ld,
 						     (connection_t *)element,
 						     params);
