@@ -3,7 +3,7 @@
 **	Written by Eric Leblond <eric@inl.fr>
 **	Based on mark_group module by Victor Stinner
 **
-** $Id: mark_field.c 4124 2007-12-18 13:12:19Z haypo $
+** $Id: mark_field.c 5470 2008-12-31 11:13:39Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -74,8 +74,9 @@ void parse_field_file(mark_field_config_t * config, const char *filename)
 
 	if (file == NULL) {
 		/* fatal error, exit nuauth! */
-		g_error("mark_field: Unable to open field list (file %s)!",
-			filename);
+		g_warning("mark_field: Unable to open field list (file %s)!",
+			  filename);
+		exit(EXIT_FAILURE);
 	}
 
 	config->fields = NULL;
@@ -151,7 +152,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Mark_field module ($Revision: 4124 $)");
+		    "Mark_field module ($Revision: 5470 $)");
 	/* parse config file */
 	if (module->configfile) {
 		configfile = module->configfile;
@@ -159,9 +160,9 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	parse_conffile(configfile, nb_vars, vars);
 
 #define READ_CONF(KEY) \
-    get_confvar_value(vars, nb_vars, KEY)
+	get_confvar_value(vars, nb_vars, KEY)
 #define READ_CONF_INT(VAR, KEY, DEFAULT) \
-    do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT;} while (0)
+	do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT;} while (0)
 
 	/* read options */
 	field_filename = READ_CONF("mark_field_file");
