@@ -1,8 +1,8 @@
 /*
-** Copyright(C) 2006, INL
+** Copyright(C) 2006-2008 INL
 **	Written by Victor Stinner <vstinner@inl.fr>
 **
-** $Id: mark_group.c 4124 2007-12-18 13:12:19Z haypo $
+** $Id: mark_group.c 5470 2008-12-31 11:13:39Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -67,8 +67,13 @@ void parse_group_file(mark_group_config_t * config, const char *filename)
 
 	if (file == NULL) {
 		/* fatal error, exit nuauth! */
-		g_error("mark_group: Unable to open group list (file %s)!",
-			filename);
+		g_warning("mark_group: Unable to open group list (file %s)!",
+			  filename);
+		exit(EXIT_FAILURE);
+	} else {
+		log_message(DEBUG, DEBUG_AREA_MAIN,
+			    "mark_group: Using group file \"%s\"",
+			    filename);
 	}
 
 	while (fgets(line, sizeof(line), file) != NULL) {
@@ -130,6 +135,9 @@ void parse_group_file(mark_group_config_t * config, const char *filename)
 			group->mark = mark;
 			config->groups =
 			    g_list_append(config->groups, group);
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					  "mark_group: Will mark group %d with mark %d",
+					  group->id, group->mark);
 			groups_item++;
 		}
 		g_strfreev(groups_list);
@@ -161,7 +169,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Mark_group module ($Revision: 4124 $)");
+		    "Mark_group module ($Revision: 5470 $)");
 	/* parse config file */
 	if (module->configfile) {
 		configfile = module->configfile;
@@ -169,9 +177,9 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	parse_conffile(configfile, nb_vars, vars);
 
 #define READ_CONF(KEY) \
-    get_confvar_value(vars, nb_vars, KEY)
+	get_confvar_value(vars, nb_vars, KEY)
 #define READ_CONF_INT(VAR, KEY, DEFAULT) \
-    do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT;} while (0)
+	do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT;} while (0)
 
 	/* read options */
 	group_filename = READ_CONF("mark_group_group_file");
@@ -249,5 +257,8 @@ G_MODULE_EXPORT nu_error_t finalize_packet(connection_t * conn, gpointer params)
 
 	conn->mark = (conn->mark & config->mask)
 	    | ((mark << config->shift) & ~config->mask);
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+		    "mark_group: Setting mark %d on conn (init mark: %d)",
+		    conn->mark, mark);
 	return NU_EXIT_OK;
 }
