@@ -3,7 +3,7 @@
  ** Copyright(C) 2004-2005 Mikael Berthe <mikael+nufw@lists.lilotux.net>
  ** Copyright(C) 2005-2008 INL
  **
- ** $Id: plaintext.c 4650 2008-04-15 11:53:26Z regit $
+ ** $Id: plaintext.c 5479 2009-01-10 13:02:31Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -331,7 +331,6 @@ static int read_user_list(struct plaintext_params *params)
 
 		if (!p_username)
 			continue;
-		/*  TODO: check for bad characters */
 
 		/*  User Name */
 		if (!p_username) {
@@ -556,7 +555,7 @@ static int read_acl_list(struct plaintext_params *params)
 			default:
 				{
 					log_message(FATAL, DEBUG_AREA_MAIN,
-						    "L.%d: Malformed line (decision should be 0 or 1)",
+						    "L.%d: Malformed line, decision should be 0 (DROP) or 1 (ACCEPT) or 3 (REJECT)",
 						    ln);
 					fclose(fd);
 					return 2;
@@ -676,7 +675,6 @@ static int read_acl_list(struct plaintext_params *params)
 						  "L.%d: Read App MD5 [%s]",
 						  ln, newapp->appmd5);
 			}
-			/*  TODO checks */
 			newacl->apps =
 			    g_slist_prepend(newacl->apps, newapp);
 		} else if (!strcasecmp("os", p_key)) {	/*  OS */
@@ -713,13 +711,32 @@ static int read_acl_list(struct plaintext_params *params)
 						  ln, newos->version);
 			}
 
-			/*  TODO checks */
 			newacl->os = g_slist_prepend(newacl->os, newos);
 		} else if (!strcasecmp("period", p_key)) {	/*  Period */
 			newacl->period = g_strdup(p_value);
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					  "L.%d: Read  period [%s]", ln,
 					  newacl->period);
+		} else if (!strcasecmp("indev", p_key)) {	/*  input dev */
+			memcpy(newacl->iface_nfo.indev, p_value, IFNAMSIZ);
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"L.%d: Read  indev [%s]", ln,
+					newacl->iface_nfo.indev);
+		} else if (!strcasecmp("physindev", p_key)) {	/*  phys input dev */
+			memcpy(newacl->iface_nfo.physindev, p_value, IFNAMSIZ);
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"L.%d: Read  physindev [%s]", ln,
+					newacl->iface_nfo.physindev);
+		} else if (!strcasecmp("outdev", p_key)) {	/*  output dev */
+			memcpy(newacl->iface_nfo.outdev, p_value, IFNAMSIZ);
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"L.%d: Read  indev [%s]", ln,
+					newacl->iface_nfo.outdev);
+		} else if (!strcasecmp("physoutdev", p_key)) {	/*  phys output dev */
+			memcpy(newacl->iface_nfo.physoutdev, p_value, IFNAMSIZ);
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"L.%d: Read  physoutdev [%s]", ln,
+					newacl->iface_nfo.physoutdev);
 		} else if (!strcasecmp("log_prefix", p_key)) {
 			newacl->log_prefix = g_strdup(p_value);
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
@@ -767,11 +784,11 @@ static int read_acl_list(struct plaintext_params *params)
 
 	fclose(fd);
 	return 0;
-}
+	}
 
-G_MODULE_EXPORT gboolean unload_module_with_params(struct plaintext_params
+	G_MODULE_EXPORT gboolean unload_module_with_params(struct plaintext_params
 						   * params)
-{
+	{
 	if (!params) {
 		return TRUE;
 	}
@@ -879,7 +896,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Plaintext module ($Revision: 4650 $)");
+		    "Plaintext module ($Revision: 5479 $)");
 	/*  parse conf file */
 	if (module->configfile) {
 		parse_conffile(module->configfile,
@@ -916,7 +933,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	/*  Depending on the use of the module load user list or acl list */
 	if (module->hook == MOD_USER_CHECK || module->hook == MOD_USER_ID || module->hook == MOD_USER_GROUPS) {
 		/*  Initialization of the user list */
-		if( read_user_list(params)) {
+		if (read_user_list(params)) {
 			log_message(FATAL, DEBUG_AREA_AUTH,
 				    "Can't parse users file [%s]",
 				    ((struct plaintext_params *) params)->
@@ -1162,6 +1179,12 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			continue;
 		}
 
+		if (compare_iface_nfo_t(&p_acl->iface_nfo, &element->iface_nfo) == NU_EXIT_ERROR) {
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"(DBG) skip ACL %s: interfaces doesn't match", p_acl->aclname);
+			continue;
+		}
+
 		/*  O.S. filtering? */
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				  "(DBG) current ACL os=%p", p_acl->os);
@@ -1265,7 +1288,12 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			    "[plaintext] matching with ACL %s and decision %d",
 			    p_acl->aclname, p_acl->decision);
 		this_acl = g_new0(struct acl_group, 1);
-		g_assert(this_acl);
+		if (this_acl == NULL) {
+			log_message(WARNING, DEBUG_AREA_MAIN,
+				    "Unable to allocate new acl");
+			g_slist_free(g_list);
+			return NULL;
+		}
 		this_acl->answer = p_acl->decision;
 		this_acl->users = g_slist_copy(p_acl->users);
 		this_acl->groups = g_slist_copy(p_acl->groups);
@@ -1281,7 +1309,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 		}
 		this_acl->flags = p_acl->flags;
 
-		g_list = g_slist_prepend(g_list, this_acl);
+		g_list = g_slist_append(g_list, this_acl);
 	}
 
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
