@@ -2,7 +2,7 @@
  ** Copyright(C) 2004-2005 INL
  ** written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: system.c 3762 2007-09-24 21:04:56Z regit $
+ ** $Id: system.c 5144 2008-11-13 11:50:09Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -275,9 +275,11 @@ G_MODULE_EXPORT uint32_t get_user_id(const char *username, gpointer params)
 		g_static_mutex_unlock(&pam_mutex);
 	}
 	if (ret != 0 || (!result_bufp)) {
+		g_free(user);
 		return SASL_BADAUTH;
 	}
 
+	g_free(user);
 	return result_bufp->pw_uid;
 }
 
@@ -304,6 +306,7 @@ G_MODULE_EXPORT GSList *get_user_groups(const char *username,
 		g_static_mutex_unlock(&pam_mutex);
 	}
 	if (ret != 0 || (!result_bufp)) {
+		g_free(user);
 		return NULL;
 	}
 
@@ -312,8 +315,9 @@ G_MODULE_EXPORT GSList *get_user_groups(const char *username,
 		userlist = getugroups(user, result_bufp->pw_gid);
 		g_static_mutex_unlock(&pam_mutex);
 	} else {
-		return getugroups(user, result_bufp->pw_gid);
+		userlist = getugroups(user, result_bufp->pw_gid);
 	}
+	g_free(user);
 	return userlist;
 }
 
@@ -325,7 +329,7 @@ G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "System module ($Revision: 3762 $)");
+		    "System module ($Revision: 5144 $)");
 	return TRUE;
 }
 
