@@ -1,8 +1,8 @@
 /*
- ** Copyright(C) 2003-2007 INL
+ ** Copyright(C) 2003-2009 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: nuauth_params.h 3956 2007-11-19 21:18:18Z regit $
+ ** $Id: nuauth_params.h 5488 2009-02-03 14:28:02Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -73,18 +73,22 @@ struct nuauth_params {
     /** \brief See log_user_packet() */
 	int log_users_sync;
 
-	/* \todo Switch this on a per-module basis */
 	int log_users_strict;
 	int log_users_without_realm;
 
-    /** decision related, see take_decision()
-     *
-     * It is used to set acl policy:
-     *  - if set to 1: if a DROP acl matches, the packet
-     *  is dropped
-     *  - if set to 0: if an ACCEPT acl matches, the packet
-     *  is accepted
-     */
+	/* absolute logging : drop packet if it is not possible to log due to 
+	 * loggers problem. */
+	int drop_if_no_logging;
+	unsigned int max_unassigned_messages;
+
+	/** decision related, see take_decision()
+	 *
+	 * It is used to set acl policy:
+	 *  - if set to 1: if a DROP acl matches, the packet
+	 *  is dropped
+	 *  - if set to 0: if an ACCEPT acl matches, the packet
+	 *  is accepted
+	 */
 	int prio_to_nok;
 
     /* Max number of client connections per user */
@@ -114,6 +118,7 @@ struct nuauth_params {
      * the old poll mode is used */
 	int push;
 	int proto_wait_delay;
+	int user_check_ip_equality;
 	int do_ip_authentication;	/*<! nuauth uses ip_authentication
 					   fallback mode if set to 1 */
 	int hello_authentication;
@@ -142,6 +147,11 @@ struct nuauth_params {
 	int nbloggers;
 	int nb_session_loggers;
 	int nb_auth_checkers;
+
+	/* Kerberos / GSSAPI stuff */
+	char * krb5_service;
+	char * krb5_hostname;
+	char * krb5_realm;
 };
 
 struct nuauth_datas {
@@ -170,6 +180,10 @@ struct nuauth_datas {
 	GThreadPool *decisions_workers;
 	GThreadPool *ip_authentication_workers;
 
+	/* Pool related */
+	gboolean loggers_pool_full;
+	gboolean session_loggers_pool_full;
+
 	/* private datas */
 	GPrivate *aclqueue;
 	GPrivate *userqueue;
@@ -178,9 +192,6 @@ struct nuauth_datas {
 	GAsyncQueue *limited_connections_queue;
 	GAsyncQueue *tls_push_queue;
 	GAsyncQueue *localid_auth_queue;
-	/** \todo Need to really use it to support user
-	 * disconnect from command line */
-	GAsyncQueue *user_cmd_queue;
 	char *program_fullpath;
 
 	/* cache related structure */
