@@ -1,8 +1,8 @@
 /*
- ** Copyright(C) 2005 INL
+ ** Copyright(C) 2005-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: nuauthconf.h 2738 2007-02-17 13:59:56Z regit $
+ ** $Id: nuauthconf.h 3909 2007-11-07 22:09:19Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 #ifndef NUAUTHCONF_H
 #define NUAUTHCONF_H
 
-void init_nuauthconf(struct nuauth_params **);
+int init_nuauthconf(struct nuauth_params **);
 
 int build_prenuauthconf(struct nuauth_params *prenuauthconf,
-			char *gwsrv_addr);
+			char *gwsrv_addr, policy_t connect_policy);
 
 void nuauth_reload(int signal);
 
