@@ -1,8 +1,8 @@
 /*
- ** Copyright(C) 2007 INL
+ ** Copyright(C) 2007-2008 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **
- ** $Id: nufw_servers.c 4125 2007-12-18 13:19:03Z haypo $
+ ** $Id: nufw_servers.c 4611 2008-04-04 08:32:37Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -91,14 +91,14 @@ void clean_nufw_session(nufw_session_t * c_session)
 		g_free(c_session->tls);
 	} else {
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW,
-				  "close nufw session was called but NULL");
+				  "nufw session close was called but returned NULL");
 
 	}
 	g_mutex_free(c_session->tls_lock);
 	g_free(c_session);
 
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW,
-			  "close nufw session: done");
+			  "nufw session cleaning: done");
 }
 
 
@@ -220,7 +220,7 @@ nu_error_t nufw_session_send(nufw_session_t * session, char * buffer, int length
 	g_mutex_unlock(session->tls_lock);
 	if (ret < 0) {
 		log_message(DEBUG, DEBUG_AREA_GW,
-			"nufw_servers: send failure (%s)",
+			"nufw_servers: send() failure (%s)",
 			gnutls_strerror(ret));
 		return NU_EXIT_ERROR;
 	} else if (ret == 0) {
