@@ -3,7 +3,7 @@
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
  **
- ** $Id: parsing.c 2895 2007-03-05 16:49:03Z regit $
+ ** $Id: parsing.c 4125 2007-12-18 13:19:03Z haypo $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -67,12 +67,7 @@ struct in6_addr *generate_inaddr_list(gchar * gwsrv_addr)
 				*authorized_server = addr6;
 				authorized_server++;
 			} else if (0 < inet_pton(AF_INET, *iter, &addr4)) {
-				authorized_server->s6_addr32[0] = 0;
-				authorized_server->s6_addr32[1] = 0;
-				authorized_server->s6_addr32[2] =
-				    0xffff0000;
-				authorized_server->s6_addr32[3] =
-				    addr4.s_addr;
+				ipv4_to_ipv6(addr4, authorized_server);
 				authorized_server++;
 			}
 
@@ -91,8 +86,8 @@ gboolean check_inaddr_in_array(struct in6_addr * check_ip,
 	/* test if server is in the list of authorized servers */
 	if (iparray) {
 		ipitem = iparray;
-		while (memcmp(ipitem, &in6addr_any, sizeof(*ipitem)) != 0) {
-			if (memcmp(ipitem, check_ip, sizeof(*ipitem)) == 0)
+		while (!ipv6_equal(ipitem, &in6addr_any)) {
+			if (ipv6_equal(ipitem, check_ip))
 				return TRUE;
 			ipitem++;
 		}
