@@ -1,9 +1,9 @@
 /*
- ** Copyright(C) 2006, INL
+ ** Copyright(C) 2006-2008, INL
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: pckt_authsrv_v3.c 3786 2007-10-08 15:54:34Z regit $
+ ** $Id: pckt_authsrv_v3.c 4601 2008-04-02 20:22:38Z gryzor $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -127,7 +127,7 @@ nu_error_t authpckt_conntrack_v3(unsigned char *dgram, unsigned int dgram_size)
 	/* Check message content size */
 	if (dgram_size != sizeof(struct nuv4_conntrack_message_t)) {
 		debug_log_message(WARNING, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
-				  "Auth conntrack: Improper length of packet");
+				  "Auth conntrack: Improper packet length ");
 		return NU_EXIT_ERROR;
 	}
 
