@@ -2,7 +2,7 @@
  ** Copyright(C) 2005-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: sasl.c 4611 2008-04-04 08:32:37Z regit $
+ ** $Id: sasl.c 5028 2008-10-28 23:52:54Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -285,7 +285,7 @@ nu_error_t get_proto_info(user_session_t * c_session)
 				    gnutls_record_recv(*(c_session->tls),
 						       buffer,
 						       sizeof(buffer) - 1);
-				if (ret < 0) {
+				if (ret <= 0) {
 					return NU_EXIT_ERROR;
 				}
 				if (((int) strlen(PROTO_STRING) + 2) <= ret
@@ -461,18 +461,18 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 		c_session->groups =
 		    modules_get_user_groups(c_session->user_name);
 		if (c_session->groups == NULL) {
-			debug_log_message(DEBUG, DEBUG_AREA_AUTH,
+			log_message(INFO, DEBUG_AREA_AUTH,
 					  "error when searching groups for user \"%s\"",
 					  c_session->user_name);
-			if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
-				return SASL_FAIL;
 			return SASL_BADAUTH;
 		}
 		c_session->user_id =
 		    modules_get_user_id(c_session->user_name);
 		if (c_session->user_id == 0) {
 			log_message(INFO, DEBUG_AREA_AUTH,
-				    "Couldn't get user ID!");
+				    "Couldn't get user ID for \"%s\"!",
+				    c_session->user_name);
+			return SASL_BADAUTH;
 		}
 	}
 #if 0
@@ -685,7 +685,7 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 	memset(buf, 0, sizeof buf);
 	tls_len = gnutls_record_recv(session, buf, sizeof(buf));
 	if (tls_len != 1) {
-		if (tls_len < 0) {
+		if (tls_len <= 0) {
 			return SASL_FAIL;
 		}
 		debug_log_message(DEBUG, DEBUG_AREA_AUTH,
@@ -701,7 +701,7 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 
 		memset(buf, 0, sizeof(buf));
 		tls_len = gnutls_record_recv(session, buf, sizeof(buf));
-		if (tls_len < 0) {
+		if (tls_len <= 0) {
 			return SASL_FAIL;
 		}
 		/* start libsasl negotiation */
@@ -834,9 +834,6 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 
 int sasl_user_check(user_session_t * c_session)
 {
-	char *service = "nuauth";
-	char *myhostname = "nuserver";
-	char *myrealm = "nufw";
 	sasl_conn_t *conn = NULL;
 	sasl_security_properties_t secprops;
 	gboolean external_auth = FALSE;
@@ -859,9 +856,11 @@ int sasl_user_check(user_session_t * c_session)
 
 	if (c_session->user_name) {
 		external_auth = TRUE;
+		c_session->auth_type = AUTH_TYPE_EXTERNAL;
 		callbacks = external_callbacks;
 	} else {
 		callbacks = internal_callbacks;
+		c_session->auth_type = AUTH_TYPE_INTERNAL;
 		if (!nuauthconf->nuauth_uses_fake_sasl) {
 			callbacks = NULL;
 		}
@@ -873,8 +872,9 @@ int sasl_user_check(user_session_t * c_session)
 	secure_snprintf(ipremoteport+len, sizeof(ipremoteport)-len,
 		";%hu", c_session->sport);
 
-	ret =
-		sasl_server_new(service, myhostname, myrealm,
+	ret = sasl_server_new(nuauthconf->krb5_service,
+			nuauthconf->krb5_hostname,
+			nuauthconf->krb5_realm,
 			iplocalport, ipremoteport,
 			callbacks, 0, &conn);
 	if (ret != SASL_OK) {
@@ -904,13 +904,17 @@ int sasl_user_check(user_session_t * c_session)
 		    sasl_setprop(conn, SASL_AUTH_EXTERNAL,
 				 c_session->user_name);
 		if (ret != SASL_OK) {
+			sasl_dispose(&conn);
 			log_message(INFO, DEBUG_AREA_AUTH,
 				    "Error setting external auth");
+			return SASL_FAIL;
 		}
 		ret = sasl_setprop(conn, SASL_SSF_EXTERNAL, &extssf);
 		if (ret != SASL_OK) {
+			sasl_dispose(&conn);
 			log_message(INFO, DEBUG_AREA_AUTH,
 				    "Error setting external SSF");
+			return SASL_FAIL;
 		}
 	}
 
@@ -933,6 +937,8 @@ int sasl_user_check(user_session_t * c_session)
 		ret = SASL_BADPARAM;
 	}
 
+	sasl_dispose(&conn);
+
 	if (ret != SASL_OK) {
 		nuauth_auth_error_t err;
 		const char *message;
@@ -949,7 +955,6 @@ int sasl_user_check(user_session_t * c_session)
 		return ret;
 	}
 
-	sasl_dispose(&conn);
 
 	/* recv OS datas from client */
 	buf_size = gnutls_record_recv(*(c_session->tls), buf, sizeof buf);
