@@ -2,8 +2,9 @@
  ** Copyright(C) 2004-2008 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
+ **             Pierre Chifflier <chifflier@inl.fr>
  **
- ** $Id: tls.c 4633 2008-04-06 13:21:16Z gryzor $
+ ** $Id: tls.c 5467 2008-12-30 16:31:39Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -29,6 +30,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <gnutls/x509.h>
+
 #include "security.h"
 
 /**
@@ -43,10 +46,6 @@
  * Contain common functions tor TLS handling
  */
 
-
-/* These are global */
-struct nuauth_tls_t nuauth_tls;
-
 /**
  * Strictly close a TLS session: call gnutls_deinit() and free memory.
  * Nothing to care about client.
@@ -73,7 +72,7 @@ void close_tls_session(int socket_fd, gnutls_session * session)
  *
  * \return SASL_OK if ok, SASL error code else
  */
-gint check_certs_for_tls_session(gnutls_session session)
+gint check_certs_for_tls_session(gnutls_session session, const char *peername)
 {
 	int ret;
 	unsigned int status = 0;
@@ -84,45 +83,52 @@ gint check_certs_for_tls_session(gnutls_session session)
 	 */
 	ret = gnutls_certificate_verify_peers2(session, &status);
 	if (ret < 0) {
-		g_warning("Certificate verification failed: %s",
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"Certificate verification failed: %s",
 			  gnutls_strerror(ret));
 		return SASL_BADPARAM;
 	}
 
 	if (status) {
-		if (status & GNUTLS_CERT_INVALID) {
-			g_warning("The certificate is invalid.");
-
-			return SASL_FAIL;
-		}
-
 		if (status & GNUTLS_CERT_REVOKED) {
-			g_warning("The certificate has been revoked.");
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"The certificate has been revoked.");
 
 			return SASL_EXPIRED;
 		}
 
 		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND) {
-			g_warning("The certificate does not have a known issuer.");
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"The certificate does not have a known issuer.");
 
 			return SASL_NOVERIFY;
 		}
 
 		if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
-			g_warning("The signer is not a CA.");
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"The signer is not a CA.");
 
 			return SASL_FAIL;
 		}
 
 #ifdef GNUTLS_CERT_INSECURE_ALGORITHM
 		if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
-			g_warning("The certificate uses an insecure algorithm.");
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"The certificate uses an insecure algorithm.");
 
 			return SASL_FAIL;
 		}
 #endif
 
-		g_warning("The certificate is not trusted.");
+		if (status & GNUTLS_CERT_INVALID) {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"The certificate is invalid.");
+
+			return SASL_FAIL;
+		}
+
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"The certificate is not trusted.");
 
 		return SASL_FAIL;
 	}
@@ -133,8 +139,9 @@ gint check_certs_for_tls_session(gnutls_session session)
 		buffer[0] = 0;
 
 		SECURE_STRNCAT(buffer, ", server certificate not yet activated", sizeof(buffer));
-		g_warning("TLS: Invalid certificates received from nuauth server:%s",
-			  buffer);
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS: Invalid certificates received: %s",
+				buffer);
 		return SASL_FAIL;
 	}
 	if (gnutls_certificate_expiration_time_peers(session) < now) {
@@ -142,8 +149,9 @@ gint check_certs_for_tls_session(gnutls_session session)
 		buffer[0] = 0;
 
 		SECURE_STRNCAT(buffer, ", server certificate expired", sizeof(buffer));
-		g_warning("TLS: Invalid certificates received from nuauth server:%s",
-			  buffer);
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS: Invalid certificates received: %s",
+				buffer);
 		return SASL_FAIL;
 	}
 
@@ -157,6 +165,263 @@ gint check_certs_for_tls_session(gnutls_session session)
 	return SASL_OK;
 }
 
+static int get_first_x509_cert_from_tls_session(gnutls_session session,
+					  gnutls_x509_crt * cert)
+{
+	const gnutls_datum *cert_list;
+	unsigned int cert_list_size = 0;
+
+	if (gnutls_certificate_type_get(session) != GNUTLS_CRT_X509)
+		return SASL_BADPARAM;
+
+	cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
+
+	if (cert_list_size > 0) {
+		/* we only print information about the first certificate. */
+		gnutls_x509_crt_init(cert);
+		gnutls_x509_crt_import(*cert, &cert_list[0],
+				       GNUTLS_X509_FMT_DER);
+	} else {
+		return SASL_BADPARAM;
+	}
+	return SASL_OK;
+}
+
+/* read the file indicated (by na1me) in the fname parameter.  store
+   its entire contents in a single datum. */
+int set_datum_file(gnutls_datum_t* d, const char* fname) {
+	struct stat sbuf;
+	unsigned char* c = NULL;
+	FILE* file = NULL;
+	size_t x = 0;
+
+	if (0 != stat(fname, &sbuf)) {
+		log_message(FATAL, DEBUG_AREA_MAIN, "failed to stat '%s'\n", fname);
+		return -1;
+	}
+
+	c = gnutls_realloc(d->data, sbuf.st_size);
+	if (NULL == c) {
+		log_message(FATAL, DEBUG_AREA_MAIN, "failed to allocate %ld bytes for '%s'\n", (long)sbuf.st_size, fname);
+		return -1;
+	}
+
+	d->data = c;
+	d->size = sbuf.st_size;
+
+	file = fopen(fname, "r");
+	if (NULL == file) {
+		log_message(FATAL, DEBUG_AREA_MAIN, "failed to open '%s' for reading\n",  fname);
+		return -1;
+	}
+
+	x = fread(d->data, d->size, 1, file);
+	if (x != 1) {
+		log_message(FATAL, DEBUG_AREA_MAIN, "tried to read %d bytes, read %zd instead from '%s'\n", d->size, x, fname);
+		fclose(file);
+		return -1;
+	}
+
+	fclose(file);
+	return 0;
+}
+
+
+static int check_crl_validity(const char *crl_file, const char *ca_file)
+{
+	gnutls_x509_crt_t ca;
+	gnutls_x509_crl_t crl;
+	gnutls_datum_t datum_crl;
+	gnutls_datum_t datum_ca;
+	time_t t, now;
+	int ret;
+	int return_value;
+	char buffer[256];
+	size_t s;
+
+
+	log_message(DEBUG, DEBUG_AREA_MAIN, "Checking CRL file %s against %s", crl_file, ca_file);
+	if (!ca_file || !crl_file)
+		return -1;
+
+	datum_ca.data = NULL;
+	datum_crl.data = NULL;
+
+	ret = set_datum_file(&datum_crl, crl_file);
+	if (ret) {
+	        log_message(WARNING, DEBUG_AREA_MAIN, "TLS: Could not read CRL file %s", crl_file);
+		nuauth_tls.invalid_tls_params = 1;
+		return -1;
+	}
+	ret = set_datum_file(&datum_ca, ca_file);
+	if (ret) {
+	        log_message(WARNING, DEBUG_AREA_MAIN, "TLS: Could not read CA file %s", ca_file);
+		gnutls_free(datum_crl.data);
+		nuauth_tls.invalid_tls_params = 1;
+		return -1;
+	}
+
+	gnutls_x509_crt_init(&ca);
+	gnutls_x509_crl_init(&crl);
+
+	ret = gnutls_x509_crl_import(crl, &datum_crl, GNUTLS_X509_FMT_PEM);
+	if (ret) {
+	        log_message(WARNING, DEBUG_AREA_MAIN, "TLS: Could not import CRL data");
+		gnutls_free(datum_ca.data);
+		gnutls_free(datum_crl.data);
+		nuauth_tls.invalid_tls_params = 1;
+		return -1;
+	}
+
+	ret = gnutls_x509_crt_import(ca, &datum_ca, GNUTLS_X509_FMT_PEM);
+	if (ret) {
+	        log_message(WARNING, DEBUG_AREA_MAIN, "TLS: Could not import CA data");
+		gnutls_free(datum_ca.data);
+		gnutls_free(datum_crl.data);
+		nuauth_tls.invalid_tls_params = 1;
+		return -1;
+	}
+
+
+	/* debug stuff */
+	s = sizeof(buffer);
+	ret = gnutls_x509_crl_get_issuer_dn(crl, buffer, &s);
+	log_message(DEBUG, DEBUG_AREA_MAIN, "TLS: CRL issuer DN: %s", buffer);
+
+	/* Check if CRL was signed by configured CA */
+	return_value = 0;
+	ret = gnutls_x509_crl_check_issuer (crl, ca);
+	if (ret != 1) {
+	        log_message(WARNING, DEBUG_AREA_MAIN, "TLS: CRL issuer is NOT the configured certificate authority");
+		return_value--;
+	}
+
+	/* Check if CRL has expired */
+	now = time(NULL);
+	t = gnutls_x509_crl_get_next_update (crl);
+	/* This field is optional in a CRL */
+	if (t != (time_t)-1 ) {
+		if (now > t) {
+			log_message(WARNING, DEBUG_AREA_MAIN, "TLS: CRL has expired and should be re-issued");
+			return_value--;
+		}
+	}
+
+	gnutls_x509_crt_deinit(ca);
+	gnutls_x509_crl_deinit(crl);
+	gnutls_free(datum_ca.data);
+	gnutls_free(datum_crl.data);
+
+	if (return_value < 0) {
+		nuauth_tls.invalid_tls_params = 1;
+		t = -1;
+	}
+	return t;
+}
+
+static int get_reverse_dns_info(struct sockaddr_storage *addr, char *buffer, size_t size)
+{
+	int ret;
+
+	ret = getnameinfo((const struct sockaddr*)addr,
+			  sizeof(*addr),
+			  buffer,
+			  size,
+			  NULL,
+			  0,
+			  0);
+
+	return ret;
+}
+
+/**
+ * Check that CN field of certificate matches peer FQDN.
+ *
+ * \return SASL_OK if ok, SASL error code else
+ */
+int tls_check_fqdn(struct sockaddr_storage *sockaddr, struct in6_addr *addr, gnutls_session * session_ptr)
+{
+	gnutls_x509_crt cert;
+	char certbuf[256];
+	size_t size;
+	int ret;
+	char hostname[256];
+
+	/* get session certificate */
+	if (get_first_x509_cert_from_tls_session(*session_ptr, &cert)
+			!= SASL_OK) {
+		if (nuauth_tls.request_cert == GNUTLS_CERT_REQUIRE) {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS error: Cannot get first x509 cert from TLS session");
+			return SASL_BADPARAM;
+		} else {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS error: Cannot get first x509 cert from TLS session, ignoring (not required)");
+			return SASL_OK;
+		}
+	}
+
+	size = sizeof(certbuf);
+	ret = gnutls_x509_crt_get_dn_by_oid(cert, GNUTLS_OID_X520_COMMON_NAME,
+			0, /* the first and only one */
+			0,
+			certbuf,
+			&size);
+
+	if (ret) {
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS: error fetching CN from cert:%s\n",
+				gnutls_strerror(ret));
+		gnutls_x509_crt_deinit(cert);
+		return SASL_BADPARAM;
+	}
+
+	/* get peer reverse DNS */
+	ret = get_reverse_dns_info(sockaddr, hostname, sizeof(hostname));
+	if (ret) {
+		char addr_ascii[INET6_ADDRSTRLEN];
+		FORMAT_IPV6(addr, addr_ascii);
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"TLS: could not get reverse DNS for nufw server %s\n",
+				addr_ascii);
+		gnutls_x509_crt_deinit(cert);
+		if (!nuauth_tls.disable_nufw_fqdn_check) {
+			return SASL_OK;
+		} else {
+			return SASL_BADPARAM;
+		}
+	}
+
+
+
+	/* This function will check if the given certificate's subject matches the
+	 * given hostname. This is a basic implementation of the matching described
+	 * in RFC2818 (HTTPS), which takes into account wildcards, and the subject
+	 * alternative name PKIX extension. Returns non zero on success, and zero on
+	 * failure. */
+	ret = gnutls_x509_crt_check_hostname(cert, hostname);
+
+	if (!ret) {
+		if (!nuauth_tls.disable_nufw_fqdn_check) {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"SSL: nufw certificate subject name (%s) does not match target host name '%s'\n",
+				certbuf, hostname);
+			gnutls_x509_crt_deinit(cert);
+			return SASL_BADPARAM;
+		}
+		else
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"SSL: nufw certificate subject name (%s) does not match target host name '%s', but continuing\n",
+				certbuf, hostname);
+	}
+
+
+	gnutls_x509_crt_deinit(cert);
+
+
+	return SASL_OK;
+}
+
 /**
  * Create the TLS server using the credentials ::x509_cred.
  *
@@ -220,6 +485,60 @@ static int generate_dh_params(gnutls_dh_params * dh_params)
 	return 0;
 }
 
+void force_refresh_crl_file()
+{
+	struct stat stats;
+
+	if (stat(nuauth_tls.crl_file, &stats) < 0) {
+		g_warning("[%i] TLS : error while trying to read CRL file %s, not reloading",
+				getpid(), nuauth_tls.crl_file);
+
+		nuauth_tls.crl_refresh_counter = 0;
+		nuauth_tls.invalid_tls_params = 1;
+		disconnect_client_by_socket(-1);
+		return;
+	}
+
+	if (nuauth_tls.crl_file_mtime < stats.st_mtime) {
+		int ret;
+
+		nuauth_tls.crl_file_exptime = check_crl_validity(nuauth_tls.crl_file,
+				nuauth_tls.ca_file);
+		if (nuauth_tls.crl_file_exptime == -1) {
+			g_warning("[%i] TLS : invalid CRL file %s",
+					getpid(), nuauth_tls.crl_file);
+			nuauth_tls.crl_refresh_counter = 0;
+			disconnect_client_by_socket(-1);
+			return;
+		}
+
+		ret = gnutls_certificate_set_x509_crl_file(nuauth_tls.
+				x509_cred,
+				nuauth_tls.crl_file,
+				GNUTLS_X509_FMT_PEM);
+		if (ret < 0) {
+			log_message(INFO, DEBUG_AREA_PERF,
+					"[%i] NuFW TLS: CRL file reloading failed (%s)",
+					getpid(), gnutls_strerror(ret));
+			disconnect_client_by_socket(-1);
+			nuauth_tls.invalid_tls_params = 1;
+			nuauth_tls.crl_refresh_counter = 0;
+			return;
+		}
+		nuauth_tls.crl_file_mtime = stats.st_mtime;
+	}
+
+	if (nuauth_tls.crl_file_exptime < time(NULL)) {
+		disconnect_client_by_socket(-1);
+		nuauth_tls.invalid_tls_params = 1;
+		nuauth_tls.crl_refresh_counter = 0;
+		return;
+	}
+
+	nuauth_tls.crl_refresh_counter = 0;
+	nuauth_tls.invalid_tls_params = 0;
+}
+
 /**
  * Refresh crl file
  *
@@ -231,23 +550,10 @@ static int generate_dh_params(gnutls_dh_params * dh_params)
 void refresh_crl_file()
 {
 	nuauth_tls.crl_refresh_counter++;
-	if (nuauth_tls.crl_refresh == nuauth_tls.crl_refresh_counter) {
-		struct stat stats;
-		stat(nuauth_tls.crl_file, &stats);
-		if (nuauth_tls.crl_file_mtime < stats.st_mtime) {
-			int ret;
-			ret = gnutls_certificate_set_x509_crl_file(nuauth_tls.
-							     x509_cred,
-							     nuauth_tls.crl_file,
-							     GNUTLS_X509_FMT_PEM);
-			if (ret < 0) {
-				log_message(INFO, DEBUG_AREA_PERF,
-						"[%i] NuFW TLS: CRL file reloading failed (%s)",
-						getpid(), gnutls_strerror(ret));
-			}
-			nuauth_tls.crl_file_mtime = stats.st_mtime;
+	if (nuauth_tls.crl_file) {
+		if (nuauth_tls.crl_refresh <= nuauth_tls.crl_refresh_counter) {
+			force_refresh_crl_file();
 		}
-		nuauth_tls.crl_refresh_counter = 0;
 	}
 }
 
@@ -275,6 +581,8 @@ static ssize_t tls_push_func(gnutls_transport_ptr ptr, const void *buf,
  */
 int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 {
+	const gnutls_datum *certs;
+	unsigned int count;
 	int ret;
 	gnutls_session *session;
 #ifdef PERF_DISPLAY_ENABLE
@@ -283,7 +591,14 @@ int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 	/* check arguments */
 	if (session_ptr == NULL) {
 		log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
-			    "NuFW TLS Init failure (session_ptr is NULL)");
+			    "TLS Init failure (session_ptr is NULL)");
+		close(socket_fd);
+		return SASL_BADPARAM;
+	}
+
+	if (nuauth_tls.invalid_tls_params) {
+		log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				"Invalid TLS params (CRL problem ?), refusing connection");
 		close(socket_fd);
 		return SASL_BADPARAM;
 	}
@@ -292,7 +607,7 @@ int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 	session = initialize_tls_session();
 	if (session == NULL) {
 		log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
-			    "NuFW TLS Init failure (initialize_tls_session())");
+			    "TLS Init failure (initialize_tls_session())");
 		close(socket_fd);
 		return SASL_BADPARAM;
 	}
@@ -310,7 +625,7 @@ int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 #endif
 	do {
 		log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
-			    "NuFW TLS Handshaking (last error: %i)", ret);
+			    "TLS Handshaking (last error: %i)", ret);
 		ret = gnutls_handshake(*session);
 	} while (ret < 0 && !gnutls_error_is_fatal(ret));
 #ifdef PERF_DISPLAY_ENABLE
@@ -320,10 +635,10 @@ int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 #endif
 
 	if (ret < 0) {
-		close_tls_session(socket_fd, session);
-		log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
-			    "NuFW TLS Handshake has failed (%s)",
+		log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+			    "TLS Handshake has failed (%s)",
 			    gnutls_strerror(ret));
+		close_tls_session(socket_fd, session);
 		return SASL_BADPARAM;
 	}
 #ifdef PERF_DISPLAY_ENABLE
@@ -336,29 +651,47 @@ int tls_connect(int socket_fd, gnutls_session ** session_ptr)
 	}
 #endif
 
-	debug_log_message(DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "NuFW TLS Handshaked");
+	debug_log_message(DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "TLS Handshaked");
 
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "NuFW TLS mac: %s",
+	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "TLS mac: %s",
 			  gnutls_mac_get_name(gnutls_mac_get(*session)));
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "NuFW TLS kx: %s",
+	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER, "TLS kx: %s",
 			  gnutls_kx_get_name(gnutls_kx_get(*session)));
 
 	debug_log_message(DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER,
-			  "NuFW TLS Handshake was completed");
+			  "TLS Handshake was completed");
 
-	if (nuauth_tls.request_cert == GNUTLS_CERT_REQUIRE) {
+	certs = gnutls_certificate_get_peers(*session, &count);
+	if (!certs) {
+		if (nuauth_tls.request_cert == GNUTLS_CERT_REQUIRE) {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"TLS error: Peer did not provide any certificate");
+			close_tls_session(socket_fd, session);
+			return SASL_BADPARAM;
+		} else {
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
+					"TLS warning: Peer did not provide any certificate, it will NOT be verified\n. Set nuauth_tls_request_cert = 2 to require certificates.");
+		}
+	}
+
+	if (certs) {
+		struct in6_addr server_addr;
+		char address[INET6_ADDRSTRLEN];
+
+		(void)getsockname_ipv6(socket_fd, &server_addr);
+		FORMAT_IPV6(&server_addr, address);
 		/* certicate verification */
-		ret = check_certs_for_tls_session(*session);
+		ret = check_certs_for_tls_session(*session, address);
 		if (ret != SASL_OK) {
-			log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
+			log_message(WARNING, DEBUG_AREA_GW | DEBUG_AREA_USER,
 				    "Certificate verification failed : %s",
 				    gnutls_strerror(ret));
 			close_tls_session(socket_fd, session);
 			return SASL_BADPARAM;
 		}
 	} else {
-		debug_log_message(DEBUG, DEBUG_AREA_GW | DEBUG_AREA_USER,
-				  "Certificate verification is not done, as specified in configuration");
+		debug_log_message(INFO, DEBUG_AREA_GW | DEBUG_AREA_USER,
+				  "Certificate verification is not done, as specified in configuration\n. Set nuauth_tls_request_cert = 2 to verify certificates.");
 	}
 
 	return SASL_OK;
@@ -372,8 +705,8 @@ int check_key_perms(const char* filename)
 		return 0;
 	}
 
-	/* Check it's owned by the current user */
-	if (infos.st_mode & S_IRGRP || infos.st_mode & S_IROTH) {
+	/* File should not be readable or writable by others */
+	if (infos.st_mode & S_IROTH || infos.st_mode & S_IWOTH) {
 		return 0;
 	}
 
@@ -400,15 +733,17 @@ int create_x509_credentials()
 		{"nuauth_tls_crl", G_TOKEN_STRING, 0, NULL},
 		{"nuauth_tls_crl_refresh", G_TOKEN_INT, DEFAULT_REFRESH_CRL_INTERVAL, NULL},
 		{"nuauth_tls_key_passwd", G_TOKEN_STRING, 0, NULL},
-		{"nuauth_tls_request_cert", G_TOKEN_INT, FALSE, NULL},
-		{"nuauth_tls_auth_by_cert", G_TOKEN_INT, FALSE, NULL}
+		{"nuauth_tls_request_cert", G_TOKEN_INT, 2, NULL},
+		{"nuauth_tls_auth_by_cert", G_TOKEN_INT, FALSE, NULL},
+		{"nuauth_tls_disable_request_warning", G_TOKEN_INT, FALSE, NULL},
+		{"nuauth_tls_disable_nufw_fqdn_check", G_TOKEN_INT, FALSE, NULL},
 	};
 	const unsigned int nb_params =
 	    sizeof(nuauth_tls_vars) / sizeof(confparams_t);
 	int int_authcert;
+	int disable_request_warning;
 
-	if(!parse_conffile(configfile, nb_params, nuauth_tls_vars))
-	{
+	if (!parse_conffile(configfile, nb_params, nuauth_tls_vars)) {
 	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
 		return 0;
 	}
@@ -428,6 +763,8 @@ int create_x509_credentials()
 	nuauth_tls.crl_refresh =
 	    *(int *) READ_CONF("nuauth_tls_crl_refresh");
 	int_authcert = *(int *) READ_CONF("nuauth_tls_auth_by_cert");
+	disable_request_warning = *(int *) READ_CONF("nuauth_tls_disable_request_warning");
+	nuauth_tls.disable_nufw_fqdn_check = *(int *) READ_CONF("nuauth_tls_disable_nufw_fqdn_check");
 #undef READ_CONF
 
 	if ((int_authcert >= NO_AUTH_BY_CERT)
@@ -466,6 +803,8 @@ int create_x509_credentials()
 
 	/* don't refresh crl if there is none */
 	if (nuauth_tls_crl == NULL) {
+		g_warning ("[%i] nuauth: no revocation list configured.\n",
+				getpid());
 		nuauth_tls.crl_refresh = 0;
 	}
 	nuauth_tls.crl_refresh_counter = 0;
@@ -490,10 +829,24 @@ int create_x509_credentials()
 					|| nuauth_tls.auth_by_cert == MANDATORY_AUTH_BY_CERT)
 				return 0;
 		}
+	} else {
+		g_warning ("[%i] nuauth: no certificate authority defined",
+					getpid());
+	}
+
+	if (!disable_request_warning) {
+		if (nuauth_tls.request_cert != 2) {
+			g_warning ("[%i] nuauth: client certificates are not required\n"
+				"nuauth will *NOT* check client certificates.\n"
+				"Set nuauth_tls_request_cert=2 to request certificates.\n",
+						getpid());
+		} else {
+			g_message("[+] Client certificates are required.");
+		}
 	}
 
 	if (!check_key_perms(nuauth_tls_key)) {
-		g_warning("[%i] Permissions on private key (%s) are not restrictive enough, it should be readable only by nuauth user.",
+		g_warning("[%i] Permissions on private key (%s) are not restrictive enough, it should be readable only by nuauth user or group.",
 			getpid(), nuauth_tls_key);
 
 		if (nuauth_tls.request_cert == GNUTLS_CERT_REQUIRE
@@ -520,9 +873,11 @@ int create_x509_credentials()
 			g_message("TLS require cert from client");
 	}
 #endif
+
+	nuauth_tls.ca_file = nuauth_tls_cacert;
+
 	g_free(nuauth_tls_key);
 	g_free(nuauth_tls_cert);
-	g_free(nuauth_tls_cacert);
 
 	if (nuauth_tls_crl) {
 		struct stat stats;
@@ -536,6 +891,14 @@ int create_x509_credentials()
 
 			return 0;
 		}
+
+		nuauth_tls.crl_file_exptime = check_crl_validity(nuauth_tls_crl,
+								 nuauth_tls.ca_file);
+		if (nuauth_tls.crl_file_exptime == -1) {
+			g_warning("[%i] TLS : invalid CRL file %s",
+				getpid(), nuauth_tls_crl);
+			return 0;
+		}
 		nuauth_tls.crl_file = nuauth_tls_crl;
 		ret = gnutls_certificate_set_x509_crl_file(nuauth_tls.x509_cred,
 						     nuauth_tls.crl_file,
