@@ -2,7 +2,7 @@
 ** Copyright(C) 2005,2006,2007 INL
 ** Written by Eric Leblond <regit@inl.fr>
 **
-** $Id: tls.h 3840 2007-10-26 15:48:56Z regit $
+** $Id: tls.h 5467 2008-12-30 16:31:39Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -58,6 +58,8 @@ GAsyncQueue *mx_queue;
 
 int tls_connect(int c, gnutls_session ** session_ptr);
 
+int tls_check_fqdn(struct sockaddr_storage *sockaddr, struct in6_addr *addr, gnutls_session * session_ptr);
+
 /* cache system related */
 struct client_connection {
 	/** Socket file descriptor, init. with accept() and set to SO_KEEPALIVE mode */
@@ -127,12 +129,16 @@ struct tls_insert_data {
 struct nuauth_tls_t {
 	gnutls_certificate_credentials x509_cred;
 	int request_cert;
+	int disable_nufw_fqdn_check;
 	auth_cert_type_t auth_by_cert;
 	int crl_refresh;
 	int crl_refresh_counter;
 	gchar *crl_file;
 	time_t crl_file_mtime;
+	time_t crl_file_exptime;
+	int invalid_tls_params;
 	gnutls_dh_params dh_params;
+	gchar *ca_file;
 };
 
 void clean_nufw_session(nufw_session_t * c_session);
@@ -159,10 +165,11 @@ void end_tls();
 gboolean remove_socket_from_pre_client_list(int c);
 
 void tls_sasl_connect(gpointer userdata, gpointer data);
-gint check_certs_for_tls_session(gnutls_session session);
+gint check_certs_for_tls_session(gnutls_session session, const char *peername);
 void close_tls_session(int c, gnutls_session * session);
 
 void refresh_crl_file();
+void force_refresh_crl_file();
 
 struct tls_user_context_t {
 	int mx;
@@ -178,6 +185,7 @@ struct tls_user_context_t {
 
 typedef struct {
 	int socket;
+	GCond *cond;
 	GMutex *mutex;
 	int result;
 } disconnect_user_msg_t;
@@ -187,4 +195,7 @@ extern struct tls_user_context_t tls_user_context;
 void tls_user_remove_client(int sock);
 void tls_user_start_servers(GSList *servers);
 
+struct nuauth_tls_t nuauth_tls;
+
+
 #endif
