@@ -3,7 +3,7 @@
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
  **
- ** $Id: tls_user.c 4633 2008-04-06 13:21:16Z gryzor $
+ ** $Id: tls_user.c 5487 2009-02-03 14:26:42Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -172,10 +172,11 @@ static nu_error_t treat_user_request(user_session_t * c_session)
 	g_mutex_unlock(c_session->tls_lock);
 	if (datas->buffer_len < (int) sizeof(struct nu_header)) {
 #ifdef DEBUG_ENABLE
-		if (datas->buffer_len < 0)
+		if (datas->buffer_len <= 0)
 			log_message(DEBUG, DEBUG_AREA_USER,
-				    "Received error from user \"%s\"",
-				    c_session->user_name);
+				    "Received error from user \"%s\": %s",
+				    c_session->user_name,
+				    gnutls_strerror(datas->buffer_len));
 #endif
 		free_buffer_read(datas);
 		return NU_EXIT_OK;
@@ -213,7 +214,7 @@ static nu_error_t treat_user_request(user_session_t * c_session)
 				       CLASSIC_NUFW_PACKET_SIZE,
 				       header_length - datas->buffer_len);
 		g_mutex_unlock(c_session->tls_lock);
-		if (tmp_len < 0) {
+		if (tmp_len <= 0) {
 			free_buffer_read(datas);
 			return NU_EXIT_ERROR;
 		}
@@ -468,17 +469,33 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 			c_pop = g_async_queue_try_pop(mx_queue);
 		}
 
-		/** \todo carefully check disconnect system */
 		/*
 		 * execute client destruction task
 		 */
 		while ((disconnect_msg = g_async_queue_try_pop(context->cmd_queue)) != NULL){
 			if (disconnect_msg->socket == -1) {
-				disconnect_msg->result = kill_all_clients();
+				nu_error_t ret;
+				for (i = 0; i < context->mx; ++i) {
+					if (FD_ISSET(i, &context->tls_rx_set) &&
+					    context->sck_inet != i) {
+						ret = delete_client_by_socket(i);
+						if (ret != NU_EXIT_OK) {
+							break;
+						}
+					}
+				}
+				disconnect_msg->result = ret;
 			} else {
-				disconnect_msg->result = delete_client_by_socket(disconnect_msg->socket);
+				if (FD_ISSET(disconnect_msg->socket,
+					     &context->tls_rx_set) &&
+				    context->sck_inet != disconnect_msg->socket) {
+					disconnect_msg->result =
+						delete_client_by_socket(disconnect_msg->socket);
+				} else {
+					disconnect_msg->result = NU_EXIT_CONTINUE;
+				}
 			}
-			g_mutex_unlock(disconnect_msg->mutex);
+			g_cond_signal(disconnect_msg->cond);
 		}
 
 		/* wait new events during 1 second */
@@ -565,8 +582,6 @@ void tls_user_servers_init()
 	    g_thread_pool_new((GFunc) tls_sasl_connect, NULL,
 			      nuauthconf->nb_auth_checkers, TRUE,
 			      NULL);
-
-	nuauthdatas->user_cmd_queue =  g_async_queue_new();
 }
 
 /**
@@ -594,10 +609,9 @@ int tls_user_init(struct tls_user_context_t *context)
 
 	/* get config file setup */
 	/* parse conf file */
-	if(!parse_conffile(DEFAULT_CONF_FILE,
+	if (!parse_conffile(DEFAULT_CONF_FILE,
 		       sizeof(nuauth_tls_vars) / sizeof(confparams_t),
-		       nuauth_tls_vars))
-	{
+		       nuauth_tls_vars)) {
 	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", DEFAULT_CONF_FILE);
 		return 0;
 	}
@@ -621,7 +635,8 @@ int tls_user_init(struct tls_user_context_t *context)
 	/* listen */
 	result = listen(context->sck_inet, 20);
 	if (result == -1) {
-		g_error("user listen() failed, exiting");
+		g_warning("user listen() failed, exiting");
+		exit(EXIT_FAILURE);
 		return 0;
 	}
 
@@ -662,9 +677,9 @@ void *push_worker(GMutex * mutex)
 	while (g_mutex_trylock(mutex)) {
 		g_mutex_unlock(mutex);
 
-		/* wait a message during 1000ms */
+		/* wait a message during POP_DELAY */
 		g_get_current_time(&tv);
-		g_time_val_add(&tv, 1000);
+		g_time_val_add(&tv, POP_DELAY);
 		message =
 		    g_async_queue_timed_pop(nuauthdatas->tls_push_queue,
 					    &tv);
