@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: user_logs.c 4346 2008-01-22 10:36:42Z regit $
+ ** $Id: user_logs.c 5303 2008-11-24 15:12:15Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -34,27 +34,35 @@ struct conn_state {
  * Log user packet or by a direct call to log module or by sending log
  * message to logger thread pool.
  *
- * If nuauth_params::log_user_sync is set to 1, we log synchronously
+ * If nuauth_params::log_users_sync is set to 1, we log synchronously
  * to be sure that the packet is logged by all the modules before
  * the decision leaves nuauth and reach nufw. This is only done for
  * packet which initiate a connection (in Netfilter meaning).
  *
- * If nuauth_params::log_user_sync is set to 0, log_user_packet() directly
+ * If nuauth_params::log_users_sync is set to 0, log_user_packet() directly
  * sends packet to the pool of threads waiting for logging.
  *
  * \param element A connection
  * \param state A ::tcp_state_t, TCP state of the connection
  */
 
-void log_user_packet(connection_t * element, tcp_state_t state)
+nu_error_t log_user_packet(connection_t * element, tcp_state_t state)
 {
 	if (element->flags & ACL_FLAGS_NOLOG) {
-		return;
+		return NU_EXIT_OK;
 	}
-	if ((nuauthconf->log_users_sync) && (state == TCP_STATE_OPEN)
-			&& (!(element->flags & ACL_FLAGS_ASYNC))) {
+
+	if (nuauthdatas->loggers_pool_full == TRUE) {
+		log_message(INFO, DEBUG_AREA_USER, "No packet logging to avoid logger DOS");
+		return NU_EXIT_ERROR;
+	}
+
+	if (nuauthconf->drop_if_no_logging ||
+			((nuauthconf->log_users_sync) && (state == TCP_STATE_OPEN)
+			 && (!(element->flags & ACL_FLAGS_ASYNC)))
+	   ) {
 		if (nuauthconf->log_users & 8) {
-			modules_user_logs(element, state);
+			return modules_user_logs(element, state);
 		}
 	} else {
 		if (((nuauthconf->log_users & 2)
@@ -69,7 +77,7 @@ void log_user_packet(connection_t * element, tcp_state_t state)
 			    (void *) duplicate_connection(element);
 			if (!conn_state_copy->conn) {
 				g_free(conn_state_copy);
-				return;
+				return NU_EXIT_ERROR;
 			}
 			conn_state_copy->state = state;
 
@@ -77,7 +85,7 @@ void log_user_packet(connection_t * element, tcp_state_t state)
 					   conn_state_copy, NULL);
 		}
 	}
-	/* end */
+	return NU_EXIT_OK;
 }
 
 /**
@@ -119,12 +127,21 @@ void real_log_user_packet(gpointer userdata, gpointer data)
 	modules_user_logs(((struct conn_state *) userdata)->conn,
 			  ((struct conn_state *) userdata)->state);
 	/* free userdata */
-	if ((((struct conn_state *) userdata)->state == TCP_STATE_OPEN) ||
-	    (((struct conn_state *) userdata)->state == TCP_STATE_DROP)) {
-		((connection_t *) ((struct conn_state *) userdata)->conn)->
-		    state = AUTH_STATE_DONE;
-		free_connection((connection_t *) ((struct conn_state *)
-						  userdata)->conn);
+	switch (((struct conn_state *) userdata)->state) {
+		case TCP_STATE_OPEN:
+		case TCP_STATE_DROP:
+			((connection_t *) ((struct conn_state *) userdata)->conn)->state = AUTH_STATE_DONE;
+			free_connection((connection_t *) ((struct conn_state *) userdata)->conn);
+			break;
+		case TCP_STATE_ESTABLISHED:
+		case TCP_STATE_CLOSE:
+			g_free(((struct conn_state *)userdata)->conn);
+			break;
+		case TCP_STATE_UNKNOW:
+		default:
+			g_warning("Should not be there, bad TCP state (%s:%d)",
+				  __FILE__,
+				  __LINE__);
 	}
 	g_free(userdata);
 }
@@ -158,6 +175,11 @@ void log_user_session(user_session_t * usession, session_state_t state)
 	struct session_event *sessevent;
 	char * str_groups = NULL;
 
+	if (nuauthdatas->session_loggers_pool_full == TRUE) {
+		log_message(INFO, DEBUG_AREA_USER, "No session logging to avoid logger DOS");
+		return;
+	}
+
 	if (state == SESSION_OPEN) {
 		if (usession->groups) {
 			g_slist_foreach(usession->groups, print_group,
@@ -220,3 +242,21 @@ void log_user_session_thread(gpointer event_ptr, gpointer unused_optional)
 	g_free(session);
 	g_free(event);
 }
+
+/** If there is too much unhandled messages modify
+ * switch to enable action in other nuauth part.*/
+void act_on_loggers_processing()
+{
+	if (g_thread_pool_unprocessed(nuauthdatas->user_loggers) >
+			nuauthconf->max_unassigned_messages) {
+		nuauthdatas->loggers_pool_full = TRUE;
+	} else {
+		nuauthdatas->loggers_pool_full = FALSE;
+	}
+	if (g_thread_pool_unprocessed(nuauthdatas->user_session_loggers) >
+			nuauthconf->max_unassigned_messages) {
+		nuauthdatas->session_loggers_pool_full = TRUE;
+	} else {
+		nuauthdatas->session_loggers_pool_full = FALSE;
+	}
+}
