@@ -3,7 +3,7 @@
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: users.h 3871 2007-10-30 07:56:51Z regit $
+ ** $Id: users.h 4839 2008-06-12 21:01:57Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -34,6 +34,11 @@ struct user_cached_datas {
 	GSList *groups;
 };
 
+typedef enum {
+	AUTH_TYPE_EXTERNAL, /*!< authentication SSL */
+	AUTH_TYPE_INTERNAL, /*!< authentification SASL */
+} auth_type_t;
+
 /**
  * \brief Stores all information relative to a TLS user session.
  *
@@ -66,6 +71,7 @@ typedef struct {
 	gchar *version;		/*!< \brief OS full version */
 	time_t expire;		/*!< \brief Timeout of the session (-1 means unlimited) */
 	int client_version;	/*!< \brief Client protocol version */
+	auth_type_t auth_type;
 	time_t connect_timestamp;
 	gboolean activated;	/*!< \brief TRUE if user server listen for event for this session */
 } user_session_t;
