@@ -1,10 +1,11 @@
 /*
- ** Copyright (C) 2002-2007 INL
+ ** Copyright (C) 2002-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@gryzor.com>
+ **            Pierre Chifflier <chifflier@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: main.c 4581 2008-03-21 21:16:56Z regit $
+ ** $Id: main.c 5477 2009-01-09 10:51:49Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -44,7 +45,9 @@ char *key_file = NULL;
 char *cert_file = NULL;
 
 /* packet server thread */
-struct nufw_threadtype thread;
+struct nufw_threadtype packet_thread;
+/* maintenance thread */
+struct nufw_threadtype maint_thread;
 
 /* packet server thread */
 struct nufw_signals signals;
@@ -60,7 +63,10 @@ void nufw_stop_thread()
 {
 	/* ask threads to stop */
 	pthread_mutex_lock(&tls.auth_server_mutex);
-	pthread_mutex_lock(&thread.mutex);
+	pthread_mutex_lock(&packet_thread.mutex);
+	if (crl_file) {
+		pthread_mutex_lock(&maint_thread.mutex);
+	}
 
 	/* wait for thread end */
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_MESSAGE,
@@ -69,8 +75,12 @@ void nufw_stop_thread()
 		pthread_join(tls.auth_server, NULL);
 	}
 	pthread_mutex_unlock(&tls.auth_server_mutex);
-	pthread_join(thread.thread, NULL);
-	pthread_mutex_unlock(&thread.mutex);
+	pthread_join(packet_thread.thread, NULL);
+	pthread_mutex_unlock(&packet_thread.mutex);
+	if (crl_file) {
+		pthread_join(maint_thread.thread, NULL);
+		pthread_mutex_unlock(&maint_thread.mutex);
+	}
 }
 
 /**
@@ -84,11 +94,6 @@ void nufw_prepare_quit()
 	pthread_mutex_destroy(&packets_list.mutex);
 
 	/* close tls session */
-#if USE_X509
-	if (tls.session) {
-		gnutls_certificate_free_credentials(tls.xcred);
-	}
-#endif
 	close_tls_session();
 	pthread_mutex_destroy(&tls.mutex);
 
@@ -104,6 +109,7 @@ void nufw_prepare_quit()
 	free(key_file);
 	free(cert_file);
 	free(ca_file);
+	free(crl_file);
 	freeaddrinfo(adr_srv);
 
 	/* destroy pid file */
@@ -162,6 +168,19 @@ void nufw_cleanup(int signal)
 	exit(EXIT_SUCCESS);
 }
 
+void *maintenance_thread_fn(void *data)
+{
+	struct nufw_threadargument *thread_arg = data;
+	struct nufw_threadtype *this = thread_arg->thread;
+	while (pthread_mutex_trylock(&this->mutex) == 0) {
+		pthread_mutex_unlock(&this->mutex);
+
+		sleep(NUFW_MAINT_DELAY);
+		tls_maintenance();
+	}
+	return NULL;
+}
+
 /**
  * Create packet server thread: init mutex and create thread
  * with packetsrv() function. Send pointer to ::thread to
@@ -171,8 +190,6 @@ void create_thread()
 {
 	/* should be static because thread may read data after this function exits */
 	static struct nufw_threadargument arg;
-	arg.thread = &thread;
-	arg.parent_pid = getpid();
 
 	/* set attribute to "joinable thread" */
 	pthread_attr_t attr;
@@ -180,11 +197,13 @@ void create_thread()
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
 
 	/* create mutex */
-	pthread_mutex_init(&thread.mutex, NULL);
+	pthread_mutex_init(&packet_thread.mutex, NULL);
 
 	/* try to create the thread */
-	if (pthread_create(&thread.thread, &attr, packetsrv, &arg) != 0) {
-		pthread_mutex_destroy(&thread.mutex);
+	arg.thread = &packet_thread;
+	arg.parent_pid = getpid();
+	if (pthread_create(&packet_thread.thread, &attr, packetsrv, &arg) != 0) {
+		pthread_mutex_destroy(&packet_thread.mutex);
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
 				"Fail to create thread!");
 		exit(EXIT_FAILURE);
@@ -199,8 +218,20 @@ void create_thread()
 	}
 #endif
 
+	if (crl_file) {
+		pthread_mutex_init(&maint_thread.mutex, NULL);
+		arg.thread = &maint_thread;
+		arg.parent_pid = getpid();
+		if (pthread_create
+				(&maint_thread.thread, &attr,
+				 maintenance_thread_fn, &arg) == EAGAIN) {
+			exit(EXIT_FAILURE);
+		}
+	}
+
 }
 
+
 /**
  * Install signals:
  *   - Set SIGTERM handler to nufw_cleanup()
@@ -241,7 +272,7 @@ void install_signals()
 	/* ignore "broken pipe" signal */
 	signal(SIGPIPE, SIG_IGN);
 
-	/* intercpet SIGUSR1 */
+	/* intercept SIGUSR1 */
 	memset(&action, 0, sizeof(action));
 	action.sa_handler = &process_usr1;
 	action.sa_flags = SIGUSR1;
@@ -250,7 +281,7 @@ void install_signals()
 				"Warning: Could not set signal USR1");
 	}
 
-	/* intercpet SIGUSR2 */
+	/* intercept SIGUSR2 */
 	memset(&action, 0, sizeof(action));
 	action.sa_handler = &process_usr2;
 	action.sa_flags = SIGUSR2;
@@ -259,7 +290,7 @@ void install_signals()
 				"Warning: Could not set signal USR2");
 	}
 
-	/* intercpet SIGPOLL */
+	/* intercept SIGPOLL */
 	memset(&action, 0, sizeof(action));
 	action.sa_handler = &process_poll;
 	action.sa_flags = SIGPOLL;
@@ -267,6 +298,16 @@ void install_signals()
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
 				"Warning: Could not set signal POLL");
 	}
+
+	/* intercept SIGHUP */
+	memset(&action, 0, sizeof(action));
+	action.sa_handler = &process_hup;
+	action.sa_flags = SIGHUP;
+	if (sigaction(SIGHUP, &action, NULL) == -1) {
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
+				"Warning: Could not set signal HUP");
+	}
+
 }
 
 /**
@@ -403,17 +444,17 @@ int main(int argc, char *argv[])
 {
 	/* option */
 #if USE_NFQUEUE
-	char *options_list = "SDhVvmq:"
+	char *options_list = "4sSNDhVvmq:"
 #ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
 	    "L:"
 #endif
-	    "c:k:a:n:d:p:t:T:A:"
+	    "c:k:a:n:r:d:p:t:T:A:"
 #ifdef HAVE_LIBCONNTRACK
 	    "CM"
 #endif
 	    ;
 #else
-	char *options_list = "SDhVvmc:k:a:n:d:p:t:T:A:";
+	char *options_list = "4sSNDhVvmc:k:a:n:r:d:p:t:T:A:";
 #endif
 	int option, daemonize = 0;
 	char *version = PACKAGE_VERSION;
@@ -427,6 +468,7 @@ int main(int argc, char *argv[])
 	cert_file = NULL;
 	key_file = NULL;
 	ca_file = NULL;
+	crl_file = NULL;
 	nuauth_cert_dn = NULL;
 	SECURE_STRNCPY(authreq_addr, AUTHREQ_ADDR, sizeof authreq_addr);
 	debug_level = DEFAULT_DEBUG_LEVEL;
@@ -443,7 +485,9 @@ int main(int argc, char *argv[])
 #endif
 #endif
 	nufw_set_mark = 0;
-	nufw_strict_tls = 0;
+	nufw_strict_tls = 1;
+	nufw_no_ipv6 = 0;
+	nufw_fqdn_check = 1;
 
 
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_VERBOSE_DEBUG,
@@ -476,6 +520,15 @@ int main(int argc, char *argv[])
 				exit(EXIT_FAILURE);
 			}
 			break;
+		case 'r':
+			crl_file = strdup(optarg);
+			if (crl_file == NULL) {
+				fprintf(stderr,
+					"Couldn't malloc! Exiting");
+				exit(EXIT_FAILURE);
+			}
+			break;
+
 		case 'n':
 			nuauth_cert_dn = strdup(optarg);
 			if (nuauth_cert_dn == NULL) {
@@ -520,9 +573,19 @@ int main(int argc, char *argv[])
 		case 'm':
 			nufw_set_mark = 1;
 			break;
+		case 's':
+			nufw_strict_tls = 0;
+			nufw_fqdn_check = 0;
+			break;
 		case 'S':
 			nufw_strict_tls = 1;
 			break;
+		case 'N':
+			nufw_fqdn_check = 0;
+			break;
+		case '4':
+			nufw_no_ipv6 = 1;
+			break;
 #if USE_NFQUEUE
 		case 'q':
 			sscanf(optarg, "%hu", &nfqueue_num);
@@ -547,7 +610,7 @@ int main(int argc, char *argv[])
 #endif				/* USE_NFQUEUE */
 
 		case 'h':
-			fprintf(stdout, "%s [-hVc"
+			fprintf(stdout, "%s [-hVcsS"
 #ifdef HAVE_LIBCONNTRACK
 				"CM"
 #endif
@@ -562,28 +625,33 @@ int main(int argc, char *argv[])
 \t-h: display this help and exit\n\
 \t-V: display version and exit\n\
 \t-D: daemonize\n\
-\t-S: do strict checking of TLS negociation\n\
+\t-S: do strict checking of TLS negociation [default]\n\
+\t-s: do not enforce strict checking of TLS certificates\n\
+\t-N: do not check nuauth fqdn (-d params) against provided certificate\n\
 \t-k: use specified file as key file\n\
 \t-c: use specified file as cert file\n\
 \t-a: use specified file as ca file (strict checking is done if selected) (default: none)\n\
-\t-n: use specified string as the needed DN of nuauth (inforce certificate checking) (default: none)\n\
+\t-r: use specified file as crl file (default: none)\n\
+\t-n: use specified string as the needed DN of nuauth (enforce certificate checking) (default is to\n\
+\t\tcheck the DN against nuauth FQDN specified using the -d option)\n\
 \t-v: increase debug level (+1 for each 'v') (max useful number: 10)\n\
 \t-A: debug areas (see man page for details)\n\
+\t-4: use this flag if your system does not have IPv6 support for nfnetlink\n\
 \t-m: mark packet with nuauth provided mark\n"
 #ifdef HAVE_LIBCONNTRACK
 				"\t-C: listen to conntrack events (needed for connection expiration)\n\
 \t-M: only report event on marked connections to nuauth (implies -C and -m)\n"
 #endif
-				"\t-d: remote address we send auth requests to (address of the nuauth server) (default: 127.0.0.1)\n\
-\t-p: remote port we send auth requests to (TCP port nuauth server listens on) (default: 4128)\n"
+				"\t-d: Address of the nuauth server (default: 127.0.0.1)\n\
+\t-p: TCP port of the nuauth server (default: 4128)\n"
 #if USE_NFQUEUE
 				"\t-q: use nfqueue number (default: 0)\n"
 #ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
-				"\t-L : set queue max len (default : 1024)\n"
+				"\t-L: set queue max len (default: 1024)\n"
 #endif
 #endif
-				"\t-t : timeout to forget about packets when they don't match (default : 15 s)\n\
-\t-T : track size (default : 1000)\n",
+				"\t-t: timeout to forget about packets when they don't match (default: 15 s)\n\
+\t-T: track size (default : 1000)\n",
 				PACKAGE_TARNAME);
 
 			exit(EXIT_SUCCESS);
@@ -619,11 +687,13 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	/* open ICMPv6 socket */
-	raw_sock6 = socket(PF_INET6, SOCK_RAW, 58);	/* 58: ICMPv6 protocol */
-	if (raw_sock6 == -1) {
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"Fail to create socket for ICMPv6!");
+	if (!nufw_no_ipv6) {
+		/* open ICMPv6 socket */
+		raw_sock6 = socket(PF_INET6, SOCK_RAW, 58);	/* 58: ICMPv6 protocol */
+		if (raw_sock6 == -1) {
+			log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
+					"Fail to create socket for ICMPv6!");
+		}
 	}
 
 	/* Create address adr_srv */
@@ -644,16 +714,39 @@ int main(int argc, char *argv[])
 	gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
 	gnutls_global_init();
 
+
 #ifdef HAVE_LIBCONNTRACK
 	cth = nfct_open(CONNTRACK, 0);
 #endif
 
+	invalid_tls_params = 0;
+	/* do initial connect */
+	tls_connect();
+
+	if (tls.session) {
+		log_area_printf(DEBUG_AREA_GW,
+				DEBUG_LEVEL_WARNING,
+				"[+] TLS connection to nuauth established");
+	} else {
+		log_area_printf(DEBUG_AREA_GW,
+				DEBUG_LEVEL_CRITICAL,
+				"[!] TLS connection to nuauth can NOT be established");
+	}
+
 
 	/* create packet server thread */
 	create_thread();
+
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
 			"[+] NuFW " VERSION " started");
 
+	if (daemonize == 0) {
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
+				"NuFW launched in foreground (without -D),"
+				"logging to stdout and stderr only (no "
+				"syslog).");
+	}
+
 	if ( ! nufw_strict_tls ) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
 				"NuFW did not start under the TLS strict mode");
