@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: nufw.h 4581 2008-03-21 21:16:56Z regit $
+ ** $Id: nufw.h 5458 2008-12-22 13:35:37Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -109,6 +109,9 @@ void *conntrack_event_handler(void *data);
 /** If equals to 1, compile with x509 certificate support */
 #define USE_X509 1
 
+/* delay between maintenance task (mainly CRL refresh) */
+#define NUFW_MAINT_DELAY 30
+
 /** Default value, prefixed with CONFIG_DIR, of ::key_file */
 #define KEYFILE "/nufw-key.pem"
 
@@ -145,12 +148,14 @@ struct queued_pckt {
 struct nuauth_conn tls;
 
 int init_x509_filenames();
-gnutls_session *tls_connect();
+void tls_connect();
 pthread_cond_t *session_destroyed_cond;
 pthread_cond_t *session_active_cond;
 pthread_mutex_t *session_destroyed_mutex;
 pthread_mutex_t *session_active_mutex;
 
+int invalid_tls_params;
+
 /**
  * Address informations of NuAuth server: hostname ::authreq_addr,
  * port ::authreq_port. Used in tls_connect().
@@ -177,8 +182,10 @@ void *authsrv(void *data);
 int auth_request_send(uint8_t type, struct queued_pckt *pckt);
 
 void close_tls_session();
+void shutdown_tls();
+void tls_maintenance();
 
-unsigned long padd(packet_idl * packet);
+int padd(packet_idl * packet);
 int psearch_and_destroy(uint32_t packet_id, uint32_t * mark);
 void clear_packet_list();
 void clean_old_packets();
@@ -186,6 +193,7 @@ void clean_old_packets();
 void process_usr1(int signum);
 void process_usr2(int signum);
 void process_poll(int signum);
+void process_hup(int signum);
 
 int send_icmp_unreach(char *payload);
 
