@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: packetsrv.c 4546 2008-03-07 08:05:43Z gryzor $
+ ** $Id: packetsrv.c 5475 2009-01-06 16:59:25Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -135,14 +135,15 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 		}
 	}
 	current = calloc(1, sizeof(packet_idl));
-	current->nfmark = q_pckt.mark;
-	current->timestamp = q_pckt.timestamp ;
-	current->id = 0;
 	if (current == NULL) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_MESSAGE,
 				"Can not allocate packet_id");
 		return 0;
 	}
+	current->nfmark = q_pckt.mark;
+	current->timestamp = q_pckt.timestamp ;
+	current->id = 0;
+	current->next = NULL;
 #ifdef PERF_DISPLAY_ENABLE
 	gettimeofday(&(current->arrival_time), NULL);
 #endif
@@ -159,10 +160,11 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 
 	/* Try to add the packet to the list */
 	pthread_mutex_lock(&packets_list.mutex);
-	q_pckt.packet_id = padd(current);
+	ret = padd(current);
+	q_pckt.packet_id = current->id;
 	pthread_mutex_unlock(&packets_list.mutex);
 
-	if (q_pckt.packet_id) {
+	if (ret == 0) {
 		/* send an auth request packet */
 		if (!auth_request_send(AUTH_REQUEST, &q_pckt)) {
 			int sandf = 0;
@@ -193,8 +195,6 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
  */
 int packetsrv_open(void *data)
 {
-	struct nfnl_handle *nh;
-
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
 			"Opening netfilter queue socket");
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
@@ -221,22 +221,23 @@ int packetsrv_open(void *data)
 		return -1;
 	}
 
-	/* unbinding existing nf_queue handler for AF_INET6 (if any) */
-	nfq_unbind_pf(h, AF_INET6);
+	if (!nufw_no_ipv6) {
+		/* unbinding existing nf_queue handler for AF_INET6 (if any) */
+		nfq_unbind_pf(h, AF_INET6);
 
-	/* binding nfnetlink_queue as nf_queue handler for AF_INET6 */
-	if (nfq_bind_pf(h, AF_INET6) < 0) {
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"[!] Error during nfq_bind_pf()");
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"Maybe you need to compile NF_NETLINK* kernel options as modules (not built in the kernel!)");
-		return -1;
+		/* binding nfnetlink_queue as nf_queue handler for AF_INET6 */
+		if (nfq_bind_pf(h, AF_INET6) < 0) {
+			log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
+					"[!] Error during nfq_bind_pf()");
+			log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
+					"Maybe you need to compile NF_NETLINK* kernel options as modules (not built in the kernel!)");
+			return -1;
+		}
 	}
 
 	/* binding this socket to queue number ::nfqueue_num
 	 * and install our packet handler */
-	hndl =
-	    nfq_create_queue(h, nfqueue_num,
+	hndl = nfq_create_queue(h, nfqueue_num,
 			     (nfq_callback *) & treat_packet, data);
 	if (!hndl) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
@@ -267,12 +268,17 @@ int packetsrv_open(void *data)
 	}
 #endif
 
-	nh = nfq_nfnlh(h);
-	return nfnl_fd(nh);
+	return nfq_fd(h);
 }
 
 void packetsrv_close(int smart)
 {
+	/* Clear all remaining packets, nfqueue will automatically drop 
+	 * queued packets when queue will be destroyed. */
+	pthread_mutex_lock(&packets_list.mutex);
+	clear_packet_list();
+	pthread_mutex_unlock(&packets_list.mutex);
+
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_SERIOUS_MESSAGE,
 			"Destroy netfilter queue socket");
 	if (smart)
@@ -292,6 +298,7 @@ void packetsrv_ipq_process(unsigned char *buffer)
 	packet_idl *current;
 	struct queued_pckt q_pckt;
 	uint32_t pcktid;
+	int ret;
 
 	pckt_rx++;
 	/* printf("Working on IP packet\n"); */
@@ -324,10 +331,13 @@ void packetsrv_ipq_process(unsigned char *buffer)
 
 	/* Adding packet to list */
 	pthread_mutex_lock(&packets_list.mutex);
-	pcktid = padd(current);
+	ret = padd(current);
+	pcktid = current->id;
 	pthread_mutex_unlock(&packets_list.mutex);
-	if (!pcktid) {
-		/* can't add packet to packet list (so already dropped): exit */
+	if (ret != 0) {
+		log_area_printf(DEBUG_AREA_MAIN | DEBUG_AREA_PACKET,
+				DEBUG_LEVEL_VERBOSE_DEBUG,
+				"Can not add packet to packet list (so already dropped): exit");
 		return;
 	}
 
@@ -437,6 +447,12 @@ void *packetsrv(void *void_arg)
 
 			if (err == EBADF) {
 				struct stat s;
+				int rebind;
+
+				if ((fstat(fd, &s)<0)) {
+					packetsrv_close(0);
+					rebind = 1;
+				}
 #ifdef HAVE_NFQ_INDEV_NAME
 				if ((fstat(if_fd, &s)<0)) {
 					iface_table_close(nlif_handle);
@@ -451,8 +467,7 @@ void *packetsrv(void *void_arg)
 					}
 				}
 #endif
-				if ((fstat(fd, &s)<0)) {
-					packetsrv_close(0);
+				if (rebind) {
 #ifdef HAVE_NFQ_INDEV_NAME
 					fd = packetsrv_open(nlif_handle);
 #else
@@ -511,13 +526,11 @@ void *packetsrv(void *void_arg)
 		pckt_rx++;
 	}
 
+	packetsrv_close(!fatal_error);
 #ifdef HAVE_NFQ_INDEV_NAME
 	iface_table_close(nlif_handle);
 #endif
-
-
-	packetsrv_close(!fatal_error);
-#else				/* USE_NFQUEUE */
+#else /* USE_NFQUEUE */
 	unsigned char buffer[BUFSIZ];
 	int size;
 
@@ -584,7 +597,7 @@ void *packetsrv(void *void_arg)
 		packetsrv_ipq_process(buffer);
 	}
 	ipq_destroy_handle(hndl);
-#endif
+#endif /* else USE_NFQUEUE */
 	log_area_printf(DEBUG_AREA_MAIN | DEBUG_AREA_PACKET,
 			DEBUG_LEVEL_WARNING,
 			"[+] Leave packet server thread");
@@ -599,19 +612,19 @@ void *packetsrv(void *void_arg)
  */
 void shutdown_tls()
 {
+	if (!tls.auth_server_running)
+		return;
+
 	log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
 			"tls send failure when sending request");
-	pthread_mutex_lock(&tls.mutex);
 
 	pthread_cancel(tls.auth_server);
 
 	close_tls_session();
 
-	/* put auth_server_running to 1 because this is this thread which has
+	/* put auth_server_running to 0 because this is this thread which has
 	 * just killed auth_server */
-	tls.auth_server_running = 1;
-
-	pthread_mutex_unlock(&tls.mutex);
+	tls.auth_server_running = 0;
 }
 
 /**
@@ -696,26 +709,11 @@ int auth_request_send(uint8_t type, struct queued_pckt *pckt_datas)
 	if (!tls.session) {
 		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_INFO,
 				"Not connected, trying TLS connection");
-		tls.session = tls_connect();
-
+		tls_connect();
 		if (tls.session) {
-			pthread_attr_t attr;
-			pthread_attr_init(&attr);
-			pthread_attr_setdetachstate(&attr,
-						    PTHREAD_CREATE_JOINABLE);
-
 			log_area_printf(DEBUG_AREA_GW,
 					DEBUG_LEVEL_WARNING,
 					"[+] TLS connection to nuauth restored");
-
-			/* create joinable thread for auth server */
-			pthread_mutex_init(&tls.auth_server_mutex, NULL);
-			if (pthread_create
-			    (&tls.auth_server, &attr, authsrv,
-			     NULL) == EAGAIN) {
-				exit(EXIT_FAILURE);
-			}
-			tls.auth_server_running = 1;
 		} else {
 			log_area_printf(DEBUG_AREA_GW,
 					DEBUG_LEVEL_CRITICAL,
@@ -726,7 +724,10 @@ int auth_request_send(uint8_t type, struct queued_pckt *pckt_datas)
 
 	/* send packet */
 	pthread_mutex_lock(&tls.mutex);
-	if (!gnutls_record_send(*(tls.session), datas, msg_length)) {
+	if (gnutls_record_send(*(tls.session), datas, msg_length) <= 0) {
+		log_area_printf(DEBUG_AREA_GW,
+				DEBUG_LEVEL_CRITICAL,
+				"[!] Error while sending TLS data");
 		shutdown_tls();
 		pthread_mutex_unlock(&tls.mutex);
 		return 0;
