@@ -1,17 +1,19 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 from unittest import TestCase, main
 from sys import stderr
-from common import createClient, connectClient, PASSWORD
+from common import createClientWithCerts, connectClient, PASSWORD
 from logging import info
 from nuauth import Nuauth
+from nuauth_conf import NuauthConf
 
 class TestClientAuth(TestCase):
     def setUp(self):
         # Load nuauth
-        self.nuauth = Nuauth()
+        nuconfig = NuauthConf()
+        self.nuauth = Nuauth(nuconfig)
 
         # Create client
-        self.client = createClient()
+        self.client = createClientWithCerts()
 
     def tearDown(self):
         self.client.stop()
