@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 from unittest import TestCase, main
 from common import startNufw, connectClient
 from nuauth import Nuauth
@@ -20,7 +20,7 @@ class TestICMPReject(TestCase):
 
         self.users.install(self.config)
         self.acls.install(self.config)
-        self.nufw = startNufw()
+        self.nufw = startNufw(["-s"])
 
     def tearDown(self):
         # Restore user DB and nuauth config
@@ -34,7 +34,7 @@ class TestICMPReject(TestCase):
         self.config["nuauth_reject_authenticated_drop"] = "0"
         self.nuauth = Nuauth(self.config)
         user = self.users[0]
-        client = user.createClient()
+        client = user.createClientWithCerts()
         testPortFailure(self, self.iptables, client, VALID_PORT, ETIMEDOUT)
         client.stop()
 
@@ -49,7 +49,7 @@ class TestICMPReject(TestCase):
         self.config["nuauth_reject_authenticated_drop"] = 1
         self.nuauth = Nuauth(self.config)
         user = self.users[0]
-        client = user.createClient()
+        client = user.createClientWithCerts()
         testPortFailure(self, self.iptables, client, VALID_PORT, ENETUNREACH)
         client.stop()
 
